/*
    w32loader
    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "user32.h"
#include "kernel32.h"
#include "w32private.h"
#include "misc/misc.h"
#include "misc/pixbuf.h"
#if GLIB_CHECK_VERSION(2,2,0)
# include <glib/gprintf.h>
#endif /* GLIB_CHECK_VERSION(2,2,0) */


#ifdef W32LDR_HEADER
#include "gdi32.h"
#include "w32btn.h"
#include "w32edit.h"
#include "w32stc.h"
#include "w32list.h"
#include "w32sbar.h"
#include "w32combo.h"

/* ja:システム */
#define COLOR_SCROLLBAR               0
#define COLOR_BACKGROUND              1
#define COLOR_DESKTOP                 COLOR_BACKGROUND
#define COLOR_ACTIVECAPTION           2
#define COLOR_INACTIVECAPTION         3
#define COLOR_MENU                    4
#define COLOR_WINDOW                  5
#define COLOR_WINDOWFRAME             6
#define COLOR_MENUTEXT                7
#define COLOR_WINDOWTEXT              8
#define COLOR_CAPTIONTEXT             9
#define COLOR_ACTIVEBORDER            10
#define COLOR_INACTIVEBORDER          11
#define COLOR_APPWORKSPACE            12
#define COLOR_HIGHLIGHT               13
#define COLOR_HIGHLIGHTTEXT           14
#define COLOR_BTNFACE                 15
#define COLOR_3DFACE                  COLOR_BTNFACE
#define COLOR_BTNSHADOW               16
#define COLOR_3DSHADOW                COLOR_BTNSHADOW
#define COLOR_GRAYTEXT                17
#define COLOR_BTNTEXT                 18
#define COLOR_INACTIVECAPTIONTEXT     19
#define COLOR_BTNHIGHLIGHT            20
#define COLOR_3DHIGHLIGHT             COLOR_BTNHIGHLIGHT
#define COLOR_3DHILIGHT               COLOR_BTNHIGHLIGHT
#define COLOR_BTNHILIGHT              COLOR_BTNHIGHLIGHT
#define COLOR_3DDKSHADOW              21
#define COLOR_3DLIGHT                 22
#define COLOR_INFOTEXT                23
#define COLOR_INFOBK                  24
#define COLOR_ALTERNATEBTNFACE        25
#define COLOR_HOTLIGHT                26
#define COLOR_GRADIENTACTIVECAPTION   27
#define COLOR_GRADIENTINACTIVECAPTION 28
#define COLOR_MENUHILIGHT             29
#define COLOR_MENUBAR                 30
#define SM_CXSCREEN          0
#define SM_CYSCREEN          1
#define SM_CXVSCROLL         2
#define SM_CYHSCROLL         3
#define SM_CYCAPTION         4
#define SM_CXBORDER          5
#define SM_CYBORDER          6
#define SM_CXDLGFRAME        7
#define SM_CYDLGFRAME        8
#define SM_CYVTHUMB          9
#define SM_CXHTHUMB          10
#define SM_CXICON            11
#define SM_CYICON            12
#define SM_CXCURSOR          13
#define SM_CYCURSOR          14
#define SM_CYMENU            15
#define SM_CXFULLSCREEN      16
#define SM_CYFULLSCREEN      17
#define SM_CYKANJIWINDOW     18
#define SM_MOUSEPRESENT      19
#define SM_CYVSCROLL         20
#define SM_CXHSCROLL         21
#define SM_DEBUG             22
#define SM_SWAPBUTTON        23
#define SM_RESERVED1         24
#define SM_RESERVED2         25
#define SM_RESERVED3         26
#define SM_RESERVED4         27
#define SM_CXMIN             28
#define SM_CYMIN             29
#define SM_CXSIZE            30
#define SM_CYSIZE            31
#define SM_CXFRAME           32
#define SM_CYFRAME           33
#define SM_CXMINTRACK        34
#define SM_CYMINTRACK        35
#define SM_CXDOUBLECLK       36
#define SM_CYDOUBLECLK       37
#define SM_CXICONSPACING     38
#define SM_CYICONSPACING     39
#define SM_MENUDROPALIGNMENT 40
#define SM_PENWINDOWS        41
#define SM_DBCSENABLED       42
#define SM_CMOUSEBUTTONS     43
#define SM_CXFIXEDFRAME      SM_CXDLGFRAME
#define SM_CYFIXEDFRAME      SM_CYDLGFRAME
#define SM_CXSIZEFRAME       SM_CXFRAME
#define SM_CYSIZEFRAME       SM_CYFRAME
#define SM_SECURE            44
#define SM_CXEDGE            45
#define SM_CYEDGE            46
#define SM_CXMINSPACING      47
#define SM_CYMINSPACING      48
#define SM_CXSMICON          49
#define SM_CYSMICON          50
#define SM_CYSMCAPTION       51
#define SM_CXSMSIZE          52
#define SM_CYSMSIZE          53
#define SM_CXMENUSIZE        54
#define SM_CYMENUSIZE        55
#define SM_ARRANGE           56
#define SM_CXMINIMIZED       57
#define SM_CYMINIMIZED       58
#define SM_CXMAXTRACK        59
#define SM_CYMAXTRACK        60
#define SM_CXMAXIMIZED       61
#define SM_CYMAXIMIZED       62
#define SM_NETWORK           63
#define SM_CLEANBOOT         67
#define SM_CXDRAG            68
#define SM_CYDRAG            69
#define SM_SHOWSOUNDS        70
#define SM_CXMENUCHECK       71
#define SM_CYMENUCHECK       72
#define SM_SLOWMACHINE       73
#define SM_MIDEASTENABLED    74
#define SM_MOUSEWHEELPRESENT 75
#define SM_XVIRTUALSCREEN    76
#define SM_YVIRTUALSCREEN    77
#define SM_CXVIRTUALSCREEN   78
#define SM_CYVIRTUALSCREEN   79
#define SM_CMONITORS         80
#define SM_SAMEDISPLAYFORMAT 81
#define SM_IMMENABLED        82
#define SM_CXFOCUSBORDER     83
#define SM_CYFOCUSBORDER     84
#define SM_TABLETPC          86
#define SM_MEDIACENTER       87
#define SM_STARTER           88
#define SM_SERVERR2          89
#define SM_CMETRICS          90
#define SPI_GETBEEP               1
#define SPI_SETBEEP               2
#define SPI_GETMOUSE              3
#define SPI_SETMOUSE              4
#define SPI_GETBORDER             5
#define SPI_SETBORDER             6
#define SPI_GETKEYBOARDSPEED      10
#define SPI_SETKEYBOARDSPEED      11
#define SPI_LANGDRIVER            12
#define SPI_ICONHORIZONTALSPACING 13
#define SPI_GETSCREENSAVETIMEOUT  14
#define SPI_SETSCREENSAVETIMEOUT  15
#define SPI_GETSCREENSAVEACTIVE   16
#define SPI_SETSCREENSAVEACTIVE   17
#define SPI_GETGRIDGRANULARITY    18
#define SPI_SETGRIDGRANULARITY    19
#define SPI_SETDESKWALLPAPER      20
#define SPI_SETDESKPATTERN        21
#define SPI_GETKEYBOARDDELAY      22
#define SPI_SETKEYBOARDDELAY      23
#define SPI_ICONVERTICALSPACING   24
#define SPI_GETICONTITLEWRAP      25
#define SPI_SETICONTITLEWRAP      26
#define SPI_GETMENUDROPALIGNMENT  27
#define SPI_SETMENUDROPALIGNMENT  28
#define SPI_SETDOUBLECLKWIDTH     29
#define SPI_SETDOUBLECLKHEIGHT    30
#define SPI_GETICONTITLELOGFONT   31
#define SPI_SETDOUBLECLICKTIME    32
#define SPI_SETMOUSEBUTTONSWAP    33
#define SPI_SETICONTITLELOGFONT   34
#define SPI_GETFASTTASKSWITCH     35
#define SPI_SETFASTTASKSWITCH     36
#define SPI_SETDRAGFULLWINDOWS    37
#define SPI_GETDRAGFULLWINDOWS    38
#define SPI_GETNONCLIENTMETRICS   41
#define SPI_SETNONCLIENTMETRICS   42
#define SPI_GETMINIMIZEDMETRICS   43
#define SPI_SETMINIMIZEDMETRICS   44
#define SPI_GETICONMETRICS        45
#define SPI_SETICONMETRICS        46
#define SPI_SETWORKAREA           47
#define SPI_GETWORKAREA           48
#define SPI_SETPENWINDOWS         49
#define SPI_GETFILTERKEYS         50
#define SPI_SETFILTERKEYS         51
#define SPI_GETTOGGLEKEYS         52
#define SPI_SETTOGGLEKEYS         53
#define SPI_GETMOUSEKEYS          54
#define SPI_SETMOUSEKEYS          55
#define SPI_GETSHOWSOUNDS         56
#define SPI_SETSHOWSOUNDS         57
#define SPI_GETSTICKYKEYS         58
#define SPI_SETSTICKYKEYS         59
#define SPI_GETACCESSTIMEOUT      60
#define SPI_SETACCESSTIMEOUT      61
#define SPI_GETSERIALKEYS         62
#define SPI_SETSERIALKEYS         63
#define SPI_GETSOUNDSENTRY        64
#define SPI_SETSOUNDSENTRY        65
#define SPI_GETHIGHCONTRAST       66
#define SPI_SETHIGHCONTRAST       67
#define SPI_GETKEYBOARDPREF       68
#define SPI_SETKEYBOARDPREF       69
#define SPI_GETSCREENREADER       70
#define SPI_SETSCREENREADER       71
#define SPI_GETANIMATION          72
#define SPI_SETANIMATION          73
#define SPI_GETFONTSMOOTHING      74
#define SPI_SETFONTSMOOTHING      75
#define SPI_SETDRAGWIDTH          76
#define SPI_SETDRAGHEIGHT         77
#define SPI_SETHANDHELD           78
#define SPI_GETLOWPOWERTIMEOUT    79
#define SPI_GETPOWEROFFTIMEOUT    80
#define SPI_SETLOWPOWERTIMEOUT    81
#define SPI_SETPOWEROFFTIMEOUT    82
#define SPI_GETLOWPOWERACTIVE     83
#define SPI_GETPOWEROFFACTIVE     84
#define SPI_SETLOWPOWERACTIVE     85
#define SPI_SETPOWEROFFACTIVE     86
#define SPI_SETCURSORS            87
#define SPI_SETICONS              88
#define SPI_GETDEFAULTINPUTLANG   89
#define SPI_SETDEFAULTINPUTLANG   90
#define SPI_SETLANGTOGGLE         91
#define SPI_GETWINDOWSEXTENSION   92
#define SPI_SETMOUSETRAILS        93
#define SPI_GETMOUSETRAILS        94
#define SPI_GETSNAPTODEFBUTTON    95
#define SPI_SETSNAPTODEFBUTTON    96
#define SPI_SETSCREENSAVERRUNNING 97
#define SPI_SCREENSAVERRUNNING    SPI_SETSCREENSAVERRUNNING
#define SPI_GETMOUSEHOVERWIDTH    98
#define SPI_SETMOUSEHOVERWIDTH    99
#define SPI_GETMOUSEHOVERHEIGHT   100
#define SPI_SETMOUSEHOVERHEIGHT   101
#define SPI_GETMOUSEHOVERTIME     102
#define SPI_SETMOUSEHOVERTIME     103
#define SPI_GETWHEELSCROLLLINES   104
#define SPI_SETWHEELSCROLLLINES   105
#define SPI_GETMENUSHOWDELAY      106
#define SPI_SETMENUSHOWDELAY      107
#define SPI_GETSHOWIMEUI          110
#define SPI_SETSHOWIMEUI          111
#define SPI_GETMOUSESPEED         112
#define SPI_SETMOUSESPEED         113
#define SPI_GETSCREENSAVERRUNNING 114
#define SPI_GETDESKWALLPAPER      115

/* ja:タイマ */
typedef VOID CALLBACK (*TIMERPROC)(HWND, UINT, UINT, DWORD);

/* ja:描画 */
#define RDW_INVALIDATE      0x0001
#define RDW_INTERNALPAINT   0x0002
#define RDW_ERASE           0x0004
#define RDW_VALIDATE        0x0008
#define RDW_NOINTERNALPAINT 0x0010
#define RDW_NOERASE         0x0020
#define RDW_NOCHILDREN      0x0040
#define RDW_ALLCHILDREN     0x0080
#define RDW_UPDATENOW       0x0100
#define RDW_ERASENOW        0x0200
#define RDW_FRAME           0x0400
#define RDW_NOFRAME         0x0800
#include "pshpack1.h"
typedef struct _POINT
{
  LONG x;
  LONG y;
} POINT, *PPOINT, *LPPOINT;
typedef struct _RECT
{
  LONG left;
  LONG top;
  LONG right;
  LONG bottom;
} RECT, *PRECT, *LPRECT;
typedef const RECT *LPCRECT;
typedef GdkRegion *HRGN;
typedef struct _PAINTSTRUCT
{
  HDC hdc;
  BOOL fErase;
  RECT rcPaint;
  BOOL fRestore;
  BOOL fIncUpdate;
  BYTE rgbReserved[32];
} PAINTSTRUCT, *PPAINTSTRUCT, *LPPAINTSTRUCT;
#include "poppack.h"

/* ja:イメージ */
#define IMAGE_BITMAP      0
#define IMAGE_ICON        1
#define IMAGE_CURSOR      2
#define IMAGE_ENHMETAFILE 3
#define LR_DEFAULTCOLOR     0x0000
#define LR_MONOCHROME       0x0001
#define LR_COLOR            0x0002
#define LR_COPYRETURNORG    0x0004
#define LR_COPYDELETEORG    0x0008
#define LR_LOADFROMFILE     0x0010
#define LR_LOADTRANSPARENT  0x0020
#define LR_DEFAULTSIZE      0x0040
#define LR_VGA_COLOR        0x0080
#define LR_LOADMAP3DCOLORS  0x1000
#define LR_CREATEDIBSECTION 0x2000
#define LR_COPYFROMRESOURCE 0x4000
#define LR_SHARED           0x8000

/* ja:アイコン */
#define OIC_SAMPLE      32512
#define OIC_HAND        32513
#define OIC_ERROR       OIC_HAND
#define OIC_QUES        32514
#define OIC_BANG        32515
#define OIC_WARNING     OIC_BANG
#define OIC_NOTE        32516
#define OIC_INFORMATION OIC_NOTE
#define OIC_WINLOGO     32517
#define IDI_APPLICATION MAKEINTRESOURCE(OIC_SAMPLE)
#define IDI_HAND        MAKEINTRESOURCE(OIC_HAND)
#define IDI_ERROR       IDI_HAND
#define IDI_QUESTION    MAKEINTRESOURCE(OIC_QUES)
#define IDI_EXCLAMATION MAKEINTRESOURCE(OIC_BANG)
#define IDI_WARNING     IDI_EXCLAMATION
#define IDI_ASTERISK    MAKEINTRESOURCE(OIC_NOTE)
#define IDI_INFORMATION IDI_ASTERISK
#define IDI_WINLOGO     MAKEINTRESOURCE(OIC_WINLOGO)
typedef GdkPixbuf *HICON;

/* ja:カーソル */
#define OCR_NORMAL      32512
#define OCR_IBEAM       32513
#define OCR_WAIT        32514
#define OCR_CROSS       32515
#define OCR_UP          32516
#define OCR_SIZE        32640
#define OCR_ICON        32641
#define OCR_SIZENWSE    32642
#define OCR_SIZENESW    32643
#define OCR_SIZEWE      32644
#define OCR_SIZENS      32645
#define OCR_SIZEALL     32646
#define OCR_NO          32648
#define OCR_HAND        32649
#define OCR_APPSTARTING 32650
#define OCR_HELP        32651
#define IDC_ARROW       MAKEINTRESOURCE(OCR_NORMAL)
#define IDC_IBEAM       MAKEINTRESOURCE(OCR_IBEAM)
#define IDC_WAIT        MAKEINTRESOURCE(OCR_WAIT)
#define IDC_CROSS       MAKEINTRESOURCE(OCR_CROSS)
#define IDC_UPARROW     MAKEINTRESOURCE(OCR_UP)
#define IDC_SIZE        MAKEINTRESOURCE(OCR_SIZE)
#define IDC_ICON        MAKEINTRESOURCE(OCR_ICON)
#define IDC_SIZENWSE    MAKEINTRESOURCE(OCR_SIZENWSE)
#define IDC_SIZENESW    MAKEINTRESOURCE(OCR_SIZENESW)
#define IDC_SIZEWE      MAKEINTRESOURCE(OCR_SIZEWE)
#define IDC_SIZENS      MAKEINTRESOURCE(OCR_SIZENS)
#define IDC_SIZEALL     MAKEINTRESOURCE(OCR_SIZEALL)
#define IDC_NO          MAKEINTRESOURCE(OCR_NO)
#define IDC_HAND        MAKEINTRESOURCE(OCR_HAND)
#define IDC_APPSTARTING MAKEINTRESOURCE(OCR_APPSTARTING)
#define IDC_HELP        MAKEINTRESOURCE(OCR_HELP)
typedef GdkCursor *HCURSOR;

/* ja:ビットマップ */
#define OBM_TRTYPE      32732
#define OBM_LFARROWI    32734
#define OBM_RGARROWI    32735
#define OBM_DNARROWI    32736
#define OBM_UPARROWI    32737
#define OBM_COMBO       32738
#define OBM_MNARROW     32739
#define OBM_LFARROWD    32740
#define OBM_RGARROWD    32741
#define OBM_DNARROWD    32742
#define OBM_UPARROWD    32743
#define OBM_RESTORED    32744
#define OBM_ZOOMD       32745
#define OBM_REDUCED     32746
#define OBM_RESTORE     32747
#define OBM_ZOOM        32748
#define OBM_REDUCE      32749
#define OBM_LFARROW     32750
#define OBM_RGARROW     32751
#define OBM_DNARROW     32752
#define OBM_UPARROW     32753
#define OBM_CLOSE       32754
#define OBM_OLD_RESTORE 32755
#define OBM_OLD_ZOOM    32756
#define OBM_OLD_REDUCE  32757
#define OBM_BTNCORNERS  32758
#define OBM_CHECKBOXES  32759
#define OBM_CHECK       32760
#define OBM_BTSIZE      32761
#define OBM_OLD_LFARROW 32762
#define OBM_OLD_RGARROW 32763
#define OBM_OLD_DNARROW 32764
#define OBM_OLD_UPARROW 32765
#define OBM_SIZE        32766
#define OBM_OLD_CLOSE   32767
typedef GdkPixbuf *HBITMAP;
#include "pshpack1.h"
typedef struct _BITMAPINFOHEADER
{
  DWORD biSize;
  LONG biWidth;
  LONG biHeight;
  WORD biPlanes;
  WORD biBitCount;
  DWORD biCompression;
  DWORD biSizeImage;
  LONG biXPelsPerMeter;
  LONG biYPelsPerMeter;
  DWORD biClrUsed;
  DWORD biClrImportant;
} BITMAPINFOHEADER, *PBITMAPINFOHEADER, *LPBITMAPINFOHEADER;
#include "poppack.h"

/* ja:メニュー */
#include "w32menu.h"

/* ja:アクセラレータ */
typedef LPVOID HACCEL;

/* ja:仮想キー */
#define VK_LBUTTON             0x01
#define VK_RBUTTON             0x02
#define VK_CANCEL              0x03
#define VK_MBUTTON             0x04
#define VK_XBUTTON1            0x05
#define VK_XBUTTON2            0x06
#define VK_BACK                0x08
#define VK_TAB                 0x09
#define VK_CLEAR               0x0c
#define VK_RETURN              0x0d
#define VK_SHIFT               0x10
#define VK_CONTROL             0x11
#define VK_MENU                0x12
#define VK_PAUSE               0x13
#define VK_CAPITAL             0x14
#define VK_KANA                0x15
#define VK_HANGEUL             0x15
#define VK_HANGUL              0x15
#define VK_JUNJA               0x17
#define VK_FINAL               0x18
#define VK_HANJA               0x19
#define VK_KANJI               0x19
#define VK_ESCAPE              0x1b
#define VK_CONVERT             0x1c
#define VK_NONCONVERT          0x1d
#define VK_ACCEPT              0x1e
#define VK_MODECHANGE          0x1f
#define VK_SPACE               0x20
#define VK_PRIOR               0x21
#define VK_NEXT                0x22
#define VK_END                 0x23
#define VK_HOME                0x24
#define VK_LEFT                0x25
#define VK_UP                  0x26
#define VK_RIGHT               0x27
#define VK_DOWN                0x28
#define VK_SELECT              0x29
#define VK_PRINT               0x2a
#define VK_EXECUTE             0x2b
#define VK_SNAPSHOT            0x2c
#define VK_INSERT              0x2d
#define VK_DELETE              0x2e
#define VK_HELP                0x2f
#define VK_0                   0x30
#define VK_1                   0x31
#define VK_2                   0x32
#define VK_3                   0x33
#define VK_4                   0x34
#define VK_5                   0x35
#define VK_6                   0x36
#define VK_7                   0x37
#define VK_8                   0x38
#define VK_9                   0x39
#define VK_A                   0x41
#define VK_B                   0x42
#define VK_C                   0x43
#define VK_D                   0x44
#define VK_E                   0x45
#define VK_F                   0x46
#define VK_G                   0x47
#define VK_H                   0x48
#define VK_I                   0x49
#define VK_J                   0x4a
#define VK_K                   0x4b
#define VK_L                   0x4c
#define VK_M                   0x4d
#define VK_N                   0x4e
#define VK_O                   0x4f
#define VK_P                   0x50
#define VK_Q                   0x51
#define VK_R                   0x52
#define VK_S                   0x53
#define VK_T                   0x54
#define VK_U                   0x55
#define VK_V                   0x56
#define VK_W                   0x57
#define VK_X                   0x58
#define VK_Y                   0x59
#define VK_Z                   0x5a
#define VK_LWIN                0x5b
#define VK_RWIN                0x5c
#define VK_APPS                0x5d
#define VK_SLEEP               0x5f
#define VK_NUMPAD0             0x60
#define VK_NUMPAD1             0x61
#define VK_NUMPAD2             0x62
#define VK_NUMPAD3             0x63
#define VK_NUMPAD4             0x64
#define VK_NUMPAD5             0x65
#define VK_NUMPAD6             0x66
#define VK_NUMPAD7             0x67
#define VK_NUMPAD8             0x68
#define VK_NUMPAD9             0x69
#define VK_MULTIPLY            0x6a
#define VK_ADD                 0x6b
#define VK_SEPARATOR           0x6c
#define VK_SUBTRACT            0x6d
#define VK_DECIMAL             0x6e
#define VK_DIVIDE              0x6f
#define VK_F1                  0x70
#define VK_F2                  0x71
#define VK_F3                  0x72
#define VK_F4                  0x73
#define VK_F5                  0x74
#define VK_F6                  0x75
#define VK_F7                  0x76
#define VK_F8                  0x77
#define VK_F9                  0x78
#define VK_F10                 0x79
#define VK_F11                 0x7a
#define VK_F12                 0x7b
#define VK_F13                 0x7c
#define VK_F14                 0x7d
#define VK_F15                 0x7e
#define VK_F16                 0x7f
#define VK_F17                 0x80
#define VK_F18                 0x81
#define VK_F19                 0x82
#define VK_F20                 0x83
#define VK_F21                 0x84
#define VK_F22                 0x85
#define VK_F23                 0x86
#define VK_F24                 0x87
#define VK_NUMLOCK             0x90
#define VK_SCROLL              0x91
#define VK_OEM_NEC_EQUAL       0x92
#define VK_OEM_FJ_JISHO        0x92
#define VK_OEM_FJ_MASSHOU      0x93
#define VK_OEM_FJ_TOUROKU      0x94
#define VK_OEM_FJ_LOYA         0x95
#define VK_OEM_FJ_ROYA         0x96
#define VK_LSHIFT              0xa0
#define VK_RSHIFT              0xa1
#define VK_LCONTROL            0xa2
#define VK_RCONTROL            0xa3
#define VK_LMENU               0xa4
#define VK_RMENU               0xa5
#define VK_BROWSER_BACK        0xa6
#define VK_BROWSER_FORWARD     0xa7
#define VK_BROWSER_REFRESH     0xa8
#define VK_BROWSER_STOP        0xa9
#define VK_BROWSER_SEARCH      0xaa
#define VK_BROWSER_FAVORITES   0xab
#define VK_BROWSER_HOME        0xac
#define VK_VOLUME_MUTE         0xad
#define VK_VOLUME_DOWN         0xae
#define VK_VOLUME_UP           0xaf
#define VK_MEDIA_NEXT_TRACK    0xb0
#define VK_MEDIA_PREV_TRACK    0xb1
#define VK_MEDIA_STOP          0xb2
#define VK_MEDIA_PLAY_PAUSE    0xb3
#define VK_LAUNCH_MAIL         0xb4
#define VK_LAUNCH_MEDIA_SELECT 0xb5
#define VK_LAUNCH_APP1         0xb6
#define VK_LAUNCH_APP2         0xb7
#define VK_OEM_1               0xba
#define VK_OEM_PLUS            0xbb
#define VK_OEM_COMMA           0xbc
#define VK_OEM_MINUS           0xbd
#define VK_OEM_PERIOD          0xbe
#define VK_OEM_2               0xbf
#define VK_OEM_3               0xc0
#define VK_OEM_4               0xcb
#define VK_OEM_5               0xcc
#define VK_OEM_6               0xcd
#define VK_OEM_7               0xce
#define VK_OEM_8               0xcf
#define VK_OEM_AX              0xe1
#define VK_OEM_102             0xe2
#define VK_ICO_HELP            0xe3
#define VK_ICO_00              0xe4
#define VK_PROCESSKEY          0xe5
#define VK_ATTN                0xf6
#define VK_CRSEL               0xf7
#define VK_EXSEL               0xf8
#define VK_EREOF               0xf9
#define VK_PLAY                0xfa
#define VK_ZOOM                0xfb
#define VK_NONAME              0xfc
#define VK_PA1                 0xfd
#define VK_OEM_CLEAR           0xfe

/* ja:クリップボード */
#define CF_TEXT        1
#define CF_BITMAP      2
#define CF_OEMTEXT     7
#define CF_DIB         8
#define CF_UNICODETEXT 13

/* ja:メッセージ */
#define PM_NOREMOVE 0x0000
#define PM_REMOVE   0x0001
#include "pshpack1.h"
typedef struct _MSG
{
  HWND hwnd;
  UINT message;
  WPARAM wParam;
  LPARAM lParam;
  DWORD time;
  POINT pt;
} MSG, *PMSG, *LPMSG;
#include "poppack.h"

/* ja:ウインドウ */
#include "w32wnd.h"
#define CreateWindowA(a,b,c,d,e,f,g,h,i,j,k) CreateWindowExA(0,a,b,c,d,e,f,g,h,i,j,k)
#define CreateWindowW(a,b,c,d,e,f,g,h,i,j,k) CreateWindowExW(0,a,b,c,d,e,f,g,h,i,j,k)
#ifdef UNICODE
# define CreateWindow(a,b,c,d,e,f,g,h,i,j,k) CreateWindowExW(0,a,b,c,d,e,f,g,h,i,j,k)
#else /* not UNICODE */
# define CreateWindow(a,b,c,d,e,f,g,h,i,j,k) CreateWindowExA(0,a,b,c,d,e,f,g,h,i,j,k)
#endif /* not UNICODE */
#define HWND_DESKTOP NULL
#define CW_USEDEFAULT ((INT)0x80000000)
#define GWL_USERDATA   (-21)
#define GWL_EXSTYLE    (-20)
#define GWL_STYLE      (-16)
#define GWL_ID         (-12)
#define GWL_HWNDPARENT (-8)
#define GWL_HINSTANCE  (-6)
#define GWL_WNDPROC    (-4)
#define DWL_MSGRESULT 0
#define DWL_DLGPROC   4
#define DWL_USER      8
#define GW_HWNDFIRST 0
#define GW_HWNDLAST  1
#define GW_HWNDNEXT  2
#define GW_HWNDPREV  3
#define GW_OWNER     4
#define GW_CHILD     5
#define SW_HIDE            0
#define SW_SHOWNORMAL      1
#define SW_NORMAL          1
#define SW_SHOWMINIMIZED   2
#define SW_SHOWMAXIMIZED   3
#define SW_MAXIMIZE        3
#define SW_SHOWNOACTIVATE  4
#define SW_SHOW            5
#define SW_MINIMIZE        6
#define SW_SHOWMINNOACTIVE 7
#define SW_SHOWNA          8
#define SW_RESTORE         9
#define SW_SHOWDEFAULT     10
#define SW_FORCEMINIMIZE   11
#define SW_MAX             11
#define SW_NORMALNA        0xcc
#define HWND_TOP       ((HWND)0)
#define HWND_BOTTOM    ((HWND)1)
#define HWND_TOPMOST   ((HWND)-1)
#define HWND_NOTOPMOST ((HWND)-2)
#define HWND_MESSAGE   ((HWND)-3)
#define SWP_NOSIZE         0x0001
#define SWP_NOMOVE         0x0002
#define SWP_NOZORDER       0x0004
#define SWP_NOREDRAW       0x0008
#define SWP_NOACTIVATE     0x0010
#define SWP_FRAMECHANGED   0x0020
#define SWP_SHOWWINDOW     0x0040
#define SWP_HIDEWINDOW     0x0080
#define SWP_NOCOPYBITS     0x0100
#define SWP_NOOWNERZORDER  0x0200
#define SWP_DRAWFRAME      SWP_FRAMECHANGED
#define SWP_NOREPOSITION   SWP_NOOWNERZORDER
#define SWP_NOSENDCHANGING 0x0400
#define SWP_DEFERERASE     0x2000
#define SWP_ASYNCWINDOWPOS 0x4000
typedef LRESULT (CALLBACK *WNDPROC)(HWND, UINT, WPARAM, LPARAM);
typedef INT_PTR (CALLBACK *DLGPROC)(HWND, UINT, WPARAM, LPARAM);
typedef BOOL CALLBACK (*WNDENUMPROC)(HWND, LPARAM);
#include "pshpack1.h"
typedef struct _WNDCLASSA
{
  UINT style;
  WNDPROC lpfnWndProc;
  INT cbClsExtra;
  INT cbWndExtra;
  HINSTANCE hInstance;
  HICON hIcon;
  HCURSOR hCursor;
  HBRUSH hbrBackground;
  LPCSTR lpszMenuName;
  LPCSTR lpszClassName;
} WNDCLASSA, *PWNDCLASSA, *LPWNDCLASSA;
typedef struct _WNDCLASSW
{
  UINT style;
  WNDPROC lpfnWndProc;
  INT cbClsExtra;
  INT cbWndExtra;
  HINSTANCE hInstance;
  HICON hIcon;
  HCURSOR hCursor;
  HBRUSH hbrBackground;
  LPCWSTR lpszMenuName;
  LPCWSTR lpszClassName;
} WNDCLASSW, *PWNDCLASSW, *LPWNDCLASSW;
#include "poppack.h"
#ifdef UNICODE
# define WNDCLASS WNDCLASSW
# define PWNDCLASS PWNDCLASSW
# define LPWNDCLASS LPWNDCLASSW
#else /* not UNICODE */
# define WNDCLASS WNDCLASSA
# define PWNDCLASS PWNDCLASSA
# define LPWNDCLASS LPWNDCLASSA
#endif /* not UNICODE */
#include "pshpack1.h"
typedef struct _WNDCLASSEXA
{
  UINT cbSize;
  UINT style;
  WNDPROC lpfnWndProc;
  INT cbClsExtra;
  INT cbWndExtra;
  HINSTANCE hInstance;
  HICON hIcon;
  HCURSOR hCursor;
  HBRUSH hbrBackground;
  LPCSTR lpszMenuName;
  LPCSTR lpszClassName;
  HICON hIconSm;
} WNDCLASSEXA, *PWNDCLASSEXA, *LPWNDCLASSEXA;
typedef struct _WNDCLASSEXW
{
  UINT cbSize;
  UINT style;
  WNDPROC lpfnWndProc;
  INT cbClsExtra;
  INT cbWndExtra;
  HINSTANCE hInstance;
  HICON hIcon;
  HCURSOR hCursor;
  HBRUSH hbrBackground;
  LPCWSTR lpszMenuName;
  LPCWSTR lpszClassName;
  HICON hIconSm;
} WNDCLASSEXW, *PWNDCLASSEXW, *LPWNDCLASSEXW;
#include "poppack.h"
#ifdef UNICODE
# define WNDCLASSEX WNDCLASSEXW
# define PWNDCLASSEX PWNDCLASSEXW
# define LPWNDCLASSEX LPWNDCLASSEXW
#else /* not UNICODE */
# define WNDCLASSEX WNDCLASSEXA
# define PWNDCLASSEX PWNDCLASSEXA
# define LPWNDCLASSEX LPWNDCLASSEXA
#endif /* not UNICODE */
#include "pshpack1.h"
typedef struct _CREATESTRUCTA
{
  LPVOID lpCreateParams;
  HINSTANCE hInstance;
  HMENU hMenu;
  HWND hwndParent;
  INT cy;
  INT cx;
  INT y;
  INT x;
  LONG style;
  LPCSTR lpszName;
  LPCSTR lpszClass;
  DWORD dwExStyle;
} CREATESTRUCTA, *PCREATESTRUCTA, *LPCREATESTRUCTA;
typedef struct _CREATESTRUCTW
{
  LPVOID lpCreateParams;
  HINSTANCE hInstance;
  HMENU hMenu;
  HWND hwndParent;
  INT cy;
  INT cx;
  INT y;
  INT x;
  LONG style;
  LPCWSTR lpszName;
  LPCWSTR lpszClass;
  DWORD dwExStyle;
} CREATESTRUCTW, *PCREATESTRUCTW, *LPCREATESTRUCTW;
#include "poppack.h"
#ifdef UNICODE
# define CREATESTRUCT CREATESTRUCTW
# define PCREATESTRUCT PCREATESTRUCTW
# define LPCREATESTRUCT LPCREATESTRUCTW
#else /* not UNICODE */
# define CREATESTRUCT CREATESTRUCTA
# define PCREATESTRUCT PCREATESTRUCTA
# define LPCREATESTRUCT LPCREATESTRUCTA
#endif /* not UNICODE */

/* ja:ダイアログ */
#include "w32dlg.h"
#define CreateDialogA(a,b,c,d) CreateDialogParamA(a,b,c,d,0)
#define CreateDialogW(a,b,c,d) CreateDialogParamW(a,b,c,d,0)
#define DialogBoxA(a,b,c,d) DialogBoxParamA(a,b,c,d,0)
#define DialogBoxW(a,b,c,d) DialogBoxParamW(a,b,c,d,0)
#ifdef UNICODE
# define CreateDialog(a,b,c,d) CreateDialogParamW(a,b,c,d,0)
# define DialogBox(a,b,c,d) DialogBoxParamW(a,b,c,d,0)
#else /* not UNICODE */
# define CreateDialog(a,b,c,d) CreateDialogParamA(a,b,c,d,0)
# define DialogBox(a,b,c,d) DialogBoxParamA(a,b,c,d,0)
#endif /* not UNICODE */
#include "pshpack2.h"
typedef struct _DLGTEMPLATE
{
  DWORD style;
  DWORD exstyle;
  WORD cdit;
  SHORT x;
  SHORT y;
  SHORT cx;
  SHORT cy;
} DLGTEMPLATE, *PDLGTEMPLATE, *LPDLGTEMPLATE;
typedef CONST DLGTEMPLATE *PCDLGTEMPLATE, *LPCDLGTEMPLATE;
typedef struct _DLGITEMTEMPLATE
{
  DWORD style;
  DWORD exstyle;
  SHORT x;
  SHORT y;
  SHORT cx;
  SHORT cy;
  WORD ID;
} DLGITEMTEMPLATE, *PDLGITEMTEMPLATE, *LPDLGITEMTEMPLATE;
typedef CONST DLGITEMTEMPLATE *PCDLGITEMTEMPLATE, *LCPDLGITEMTEMPLATE;
typedef struct _DLGTEMPLATEEX
{
  WORD dlgVer;
  WORD signature;
  DWORD helpID;
  DWORD exstyle;
  DWORD style;
  WORD cdit;
  SHORT x;
  SHORT y;
  SHORT cx;
  SHORT cy;
} DLGTEMPLATEEX, *PDLGTEMPLATEEX, *LPDLGTEMPLATEEX;
typedef CONST DLGTEMPLATEEX *PCDLGTEMPLATEEX, *LPCDLGTEMPLATEEX;
typedef struct _DLGITEMTEMPLATEEX
{
  DWORD helpID;
  DWORD exstyle;
  DWORD style;
  SHORT x;
  SHORT y;
  SHORT cx;
  SHORT cy;
  WORD ID;
} DLGITEMTEMPLATEEX, *PDLGITEMTEMPLATEEX, *LPDLGITEMTEMPLATEEX;
typedef CONST DLGITEMTEMPLATEEX *PCDLGITEMTEMPLATEEX, *LPCDLGITEMTEMPLATEEX;
#include "poppack.h"

/* ja:メッセージボックス */
#define MB_OK                   0x00000000
#define MB_OKCANCEL             0x00000001
#define MB_ABORTRETRYIGNORE     0x00000002
#define MB_YESNOCANCEL          0x00000003
#define MB_YESNO                0x00000004
#define MB_RETRYCANCEL          0x00000005
#define MB_TYPEMASK             0x0000000F
#define MB_ICONHAND             0x00000010
#define MB_ICONQUESTION         0x00000020
#define MB_ICONEXCLAMATION      0x00000030
#define MB_ICONASTERISK         0x00000040
#define MB_USERICON             0x00000080
#define MB_ICONMASK             0x000000F0
#define MB_ICONINFORMATION      MB_ICONASTERISK
#define MB_ICONSTOP             MB_ICONHAND
#define MB_ICONWARNING          MB_ICONEXCLAMATION
#define MB_ICONERROR            MB_ICONHAND
#define MB_DEFBUTTON1           0x00000000
#define MB_DEFBUTTON2           0x00000100
#define MB_DEFBUTTON3           0x00000200
#define MB_DEFBUTTON4           0x00000300
#define MB_DEFMASK              0x00000f00
#define MB_APPLMODAL            0x00000000
#define MB_SYSTEMMODAL          0x00001000
#define MB_TASKMODAL            0x00002000
#define MB_MODEMASK             0x00003000
#define MB_HELP                 0x00004000
#define MB_NOFOCUS              0x00008000
#define MB_MISCMASK             0x0000C000
#define MB_SETFOREGROUND        0x00010000
#define MB_DEFAULT_DESKTOP_ONLY 0x00020000
#define MB_SERVICE_NOTIFICATION 0x00040000
#define MB_TOPMOST              0x00040000
#define MB_RIGHT                0x00080000
#define MB_RTLREADING           0x00100000
/* en:ID */
#define IDOK       1
#define IDCANCEL   2
#define IDABORT    3
#define IDRETRY    4
#define IDIGNORE   5
#define IDYES      6
#define IDNO       7
#define IDCLOSE    8
#define IDHELP     9
#define IDTRYAGAIN 10
#define IDCONTINUE 11
#endif /* W32LDR_HEADER */


typedef struct _W32LdrMessage
{
  HWND hWnd;
  UINT Msg;
  WPARAM wParam;
  LPARAM lParam;
} W32LdrMessage;
typedef struct _W32LdrTimer
{
  HWND hWnd;
  UINT uIDEvent;
  TIMERPROC lpTimerFunc;
  guint id;
} W32LdrTimer;


static GList *glist_message = NULL;
static GList *glist_timer = NULL;


/* ActivateKeyboardLayout ordinal = 1, argument = 8 temporary */
static DWORD WINAPI ActivateKeyboardLayout (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* AdjustWindowRect ordinal = 2, argument = 12 temporary */
static DWORD WINAPI AdjustWindowRect (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* AdjustWindowRectEx ordinal = 3, argument = 16 temporary */
static DWORD WINAPI AdjustWindowRectEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* AlignRects ordinal = 4, argument = 0 temporary */
static DWORD WINAPI AlignRects (VOID)
{
  return 0;
}


/* AllowSetForegroundWindow ordinal = 5, argument = 4 temporary */
static DWORD WINAPI AllowSetForegroundWindow (DWORD dwArgv0)
{
  return 0;
}


/* AnimateWindow ordinal = 6, argument = 12 temporary */
static DWORD WINAPI AnimateWindow (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* AnyPopup ordinal = 7, argument = 0 temporary */
static DWORD WINAPI AnyPopup (VOID)
{
  return 0;
}


/* AppendMenuA ordinal = 8, argument = 16 */
BOOL WINAPI AppendMenuA (HMENU hMenu, UINT uFlags, UINT_PTR uItem, LPCSTR lpszItem)
{
  gchar *utf8str;
  gunichar2 *utf16str;
  BOOL fResult;

  if (uFlags & (MFT_BITMAP | MFT_OWNERDRAW))
    return AppendMenuW (hMenu, uFlags, uItem, (LPCWSTR)lpszItem);

  utf8str = w32ldr_utf8_from_mb (lpszItem);
  utf16str = utf8str ? g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL)
                     : g_malloc0 (sizeof (gunichar2));
  g_free (utf8str);
  fResult = AppendMenuW (hMenu, uFlags, uItem, utf16str);
  g_free (utf16str);
  return fResult;
}


/* AppendMenuW ordinal = 9, argument = 16 */
BOOL WINAPI AppendMenuW (HMENU hMenu, UINT uFlags, UINT_PTR uItem, LPCWSTR lpszItem)
{
  return InsertMenuW (hMenu, (UINT)-1, uFlags | MF_BYPOSITION,
                                                            uItem, lpszItem);
}


/* ArrangeIconicWindows ordinal = 10, argument = 4 temporary */
static DWORD WINAPI ArrangeIconicWindows (DWORD dwArgv0)
{
  return 0;
}


/* AttachThreadInput ordinal = 11, argument = 12 temporary */
static DWORD WINAPI AttachThreadInput (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* BeginDeferWindowPos ordinal = 12, argument = 4 temporary */
static DWORD WINAPI BeginDeferWindowPos (DWORD dwArgv0)
{
  return 0;
}


/* BeginPaint ordinal = 13, argument = 8 */
HDC WINAPI BeginPaint (HWND hWnd, LPPAINTSTRUCT lpPaint)
{
  if (!lpPaint)
    return NULL;
  g_memset (lpPaint, 0, sizeof (PAINTSTRUCT));
  if (!hWnd || !GetClientRect (hWnd, &lpPaint->rcPaint))
    return NULL;
  lpPaint->hdc = GetDC (hWnd);
  lpPaint->fErase = TRUE;
  return lpPaint->hdc;
}


/* BlockInput ordinal = 14, argument = 4 temporary */
static DWORD WINAPI BlockInput (DWORD dwArgv0)
{
  return 0;
}


/* BringWindowToTop ordinal = 15, argument = 4 temporary */
static DWORD WINAPI BringWindowToTop (DWORD dwArgv0)
{
  return 0;
}


/* BroadcastSystemMessage ordinal = 16, argument = 20 temporary */
/* DWORD WINAPI BroadcastSystemMessage (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4) */


/* BroadcastSystemMessageA ordinal = 17, argument = 20 temporary */
static DWORD WINAPI BroadcastSystemMessageA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* BroadcastSystemMessageW ordinal = 18, argument = 20 temporary */
static DWORD WINAPI BroadcastSystemMessageW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* CallMsgFilter ordinal = 19, argument = 8 temporary */
/* DWORD WINAPI CallMsgFilter (DWORD dwArgv0, DWORD dwArgv1) */


/* CallMsgFilterA ordinal = 20, argument = 8 temporary */
static DWORD WINAPI CallMsgFilterA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CallMsgFilterW ordinal = 21, argument = 8 temporary */
static DWORD WINAPI CallMsgFilterW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CallNextHookEx ordinal = 22, argument = 16 temporary */
static DWORD WINAPI CallNextHookEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CallWindowProcA ordinal = 23, argument = 20 */
LRESULT WINAPI CallWindowProcA (WNDPROC lpPrevWndFunc, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return lpPrevWndFunc (hWnd, Msg, wParam, lParam);
}


/* CallWindowProcW ordinal = 24, argument = 20 */
LRESULT WINAPI CallWindowProcW (WNDPROC lpPrevWndFunc, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return lpPrevWndFunc (hWnd, Msg, wParam, lParam);
}


/* CascadeChildWindows ordinal = 25, argument = 8 temporary */
static DWORD WINAPI CascadeChildWindows (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CascadeWindows ordinal = 26, argument = 20 temporary */
static DWORD WINAPI CascadeWindows (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ChangeClipboardChain ordinal = 27, argument = 8 temporary */
static DWORD WINAPI ChangeClipboardChain (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ChangeDisplaySettingsA ordinal = 28, argument = 8 temporary */
static DWORD WINAPI ChangeDisplaySettingsA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ChangeDisplaySettingsExA ordinal = 29, argument = 20 temporary */
static DWORD WINAPI ChangeDisplaySettingsExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ChangeDisplaySettingsExW ordinal = 30, argument = 20 temporary */
static DWORD WINAPI ChangeDisplaySettingsExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ChangeDisplaySettingsW ordinal = 31, argument = 8 temporary */
static DWORD WINAPI ChangeDisplaySettingsW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ChangeMenuA ordinal = 32, argument = 20 temporary */
static DWORD WINAPI ChangeMenuA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ChangeMenuW ordinal = 33, argument = 20 temporary */
static DWORD WINAPI ChangeMenuW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* CharLowerA ordinal = 34, argument = 4 */
LPSTR WINAPI CharLowerA (LPSTR lpsz)
{
  gchar *utf8str, *str, *mb;

  if (GPOINTER_TO_UINT (lpsz) <= G_MAXUINT16)
    return GINT_TO_POINTER (g_unichar_toupper (GPOINTER_TO_UINT (lpsz)));
  utf8str = w32ldr_utf8_from_mb (lpsz);
  str = utf8str ? g_utf8_strdown (utf8str, -1) : NULL;
  g_free (utf8str);
  mb = w32ldr_utf8_to_mb (str);
  g_free (str);
  g_strcpy (lpsz, mb);
  g_free (mb);
  return lpsz;
}


/* CharLowerBuffA ordinal = 35, argument = 8 temporary */
static DWORD WINAPI CharLowerBuffA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharLowerBuffW ordinal = 36, argument = 8 temporary */
static DWORD WINAPI CharLowerBuffW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharLowerW ordinal = 37, argument = 4 */
LPWSTR WINAPI CharLowerW (LPWSTR lpsz)
{
  gchar *utf8str, *str;
  gunichar2 *wc;

  if (GPOINTER_TO_UINT (lpsz) <= G_MAXUINT16)
    return GINT_TO_POINTER (g_unichar_toupper (GPOINTER_TO_UINT (lpsz)));
  utf8str = g_utf16_to_utf8 (lpsz, -1, NULL, NULL, NULL);
  str = utf8str ? g_utf8_strdown (utf8str, -1) : NULL;
  g_free (utf8str);
  wc = str ? g_utf8_to_utf16 (str, -1, NULL, NULL, NULL) : NULL;
  g_free (str);
  lstrcpyW (lpsz, wc);
  g_free (wc);
  return lpsz;
}


/* CharNextA ordinal = 38, argument = 4 temporary */
static DWORD WINAPI CharNextA (DWORD dwArgv0)
{
  return 0;
}


/* CharNextExA ordinal = 39, argument = 12 temporary */
static DWORD WINAPI CharNextExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* CharNextW ordinal = 40, argument = 4 temporary */
static DWORD WINAPI CharNextW (DWORD dwArgv0)
{
  return 0;
}


/* CharPrevA ordinal = 41, argument = 8 temporary */
static DWORD WINAPI CharPrevA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharPrevExA ordinal = 42, argument = 16 temporary */
static DWORD WINAPI CharPrevExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CharPrevW ordinal = 43, argument = 8 temporary */
static DWORD WINAPI CharPrevW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharToOemA ordinal = 44, argument = 8 temporary */
static DWORD WINAPI CharToOemA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharToOemBuffA ordinal = 45, argument = 12 temporary */
static DWORD WINAPI CharToOemBuffA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* CharToOemBuffW ordinal = 46, argument = 12 temporary */
static DWORD WINAPI CharToOemBuffW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* CharToOemW ordinal = 47, argument = 8 temporary */
static DWORD WINAPI CharToOemW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharUpperA ordinal = 48, argument = 4 */
LPSTR WINAPI CharUpperA (LPSTR lpsz)
{
  gchar *utf8str, *str, *mb;

  if (GPOINTER_TO_UINT (lpsz) <= G_MAXUINT16)
    return GINT_TO_POINTER (g_unichar_toupper (GPOINTER_TO_UINT (lpsz)));
  utf8str = w32ldr_utf8_from_mb (lpsz);
  str = utf8str ? g_utf8_strup (utf8str, -1) : NULL;
  g_free (utf8str);
  mb = w32ldr_utf8_to_mb (utf8str);
  g_free (str);
  g_strcpy (lpsz, mb);
  g_free (mb);
  return lpsz;
}


/* CharUpperBuffA ordinal = 49, argument = 8 temporary */
static DWORD WINAPI CharUpperBuffA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharUpperBuffW ordinal = 50, argument = 8 temporary */
static DWORD WINAPI CharUpperBuffW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CharUpperW ordinal = 51, argument = 4 */
LPWSTR WINAPI CharUpperW (LPWSTR lpsz)
{
  gchar *utf8str, *str;
  gunichar2 *wc;

  if (GPOINTER_TO_UINT (lpsz) <= G_MAXUINT16)
    return GINT_TO_POINTER (g_unichar_toupper (GPOINTER_TO_UINT (lpsz)));
  utf8str = g_utf16_to_utf8 (lpsz, -1, NULL, NULL, NULL);
  str = utf8str ? g_utf8_strup (utf8str, -1) : NULL;
  g_free (utf8str);
  wc = str ? g_utf8_to_utf16 (str, -1, NULL, NULL, NULL) : NULL;
  g_free (str);
  lstrcpyW (lpsz, wc);
  g_free (wc);
  return lpsz;
}


/* CheckDlgButton ordinal = 52, argument = 12 */
BOOL WINAPI CheckDlgButton (HWND hDlg, INT nIDButton, UINT uCheck)
{
  GtkWidget *widget;

  if (uCheck != BST_INDETERMINATE && (widget = GetDlgItem (hDlg, nIDButton)))
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
                                                        uCheck == BST_CHECKED);
      return TRUE;
    }
  return FALSE;
}


/* CheckMenuItem ordinal = 53, argument = 12 */
DWORD WINAPI CheckMenuItem (HMENU hMenu, UINT uItem, UINT uFlags)
{
  DWORD dwResult = (DWORD)-1;

  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item && GTK_IS_IMAGE_MENU_ITEM (menu_item))
        {
          GtkWidget *image = NULL;

          if (uFlags & MFS_CHECKED)
            {
              GdkPixbuf *pixbuf;

              pixbuf = w32ldr_menu_get_pixbuf_check ();
              if (pixbuf)
                image = gtk_image_new_from_pixbuf (pixbuf);
            }
          if (!(uFlags & MFS_CHECKED) || image)
            {
              dwResult = gtk_image_menu_item_get_image
                                            (GTK_IMAGE_MENU_ITEM (menu_item))
                                                ? MFS_CHECKED : MFS_UNCHECKED;
              gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                                                                        image);
            }
        }
    }
  return dwResult;
}


/* CheckMenuRadioItem ordinal = 54, argument = 20 */
BOOL WINAPI CheckMenuRadioItem (HMENU hMenu, UINT uFirst, UINT uLast, UINT uCheck, UINT uFlags)
{
  guint st, ed;
  DWORD fResult = TRUE;

  st = MIN (uFirst, uLast);
  ed = MAX (uFirst, uLast);
  if (hMenu && st <= uCheck && uCheck <= ed)
    {
      guint i;

      for (i = st; i <= ed; i++)
        {
          GtkWidget *menu_item;

          menu_item = w32ldr_menu_get_item (hMenu, i, uFlags);
          if (menu_item && GTK_IS_IMAGE_MENU_ITEM (menu_item))
            {
              GtkWidget *image = NULL;

              if (i == uCheck)
                {
                  GdkPixbuf *pixbuf;

                  pixbuf = w32ldr_menu_get_pixbuf_radio ();
                  if (pixbuf)
                    image = gtk_image_new_from_pixbuf (pixbuf);
                }
              if (i == uCheck && !image)
                fResult = FALSE;
              gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                                                                        image);
            }
          else
            {
              fResult = FALSE;
            }
        }
    }
  else
    {
      fResult = FALSE;
    }
  return fResult;
}


/* CheckRadioButton ordinal = 55, argument = 16 */
BOOL WINAPI CheckRadioButton (HWND hDlg, INT nIDFirstButton, INT nIDLastButton, INT nIDCheckButton)
{
  gint i;

  for (i = nIDFirstButton; i <= nIDLastButton; i++)
    {
      GtkWidget *widget;

      widget = GetDlgItem (hDlg, i);
      if (widget)
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
                                                        i == nIDCheckButton);
    }
  return TRUE;
}


/* ChildWindowFromPoint ordinal = 56, argument = 12 temporary */
static DWORD WINAPI ChildWindowFromPoint (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* ChildWindowFromPointEx ordinal = 57, argument = 16 temporary */
static DWORD WINAPI ChildWindowFromPointEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CliImmSetHotKey ordinal = 58, argument = 0 temporary */
static DWORD WINAPI CliImmSetHotKey (VOID)
{
  return 0;
}


/* ClientThreadSetup ordinal = 59, argument = 0 temporary */
static DWORD WINAPI ClientThreadSetup (VOID)
{
  return 0;
}


/* ClientToScreen ordinal = 60, argument = 8 */
BOOL WINAPI ClientToScreen (HWND hWnd, LPPOINT lpPoint)
{
  GdkWindow *window;
  GtkWidget *widget = NULL;

  if (!hWnd || !lpPoint)
    return FALSE;
  if (GTK_IS_WINDOW (hWnd))
    {
      W32LdrOverlapData *od;

      od = g_object_get_data (G_OBJECT (hWnd), "user_data");
      widget = od->fixed;
    }
  if (!widget)
    widget = hWnd;
  lpPoint->x += widget->allocation.x;
  lpPoint->y += widget->allocation.y;
  for (window = widget->window; window;
                                    window = gdk_window_get_parent (window))
    {
      gint x, y;

      gdk_window_get_position (window, &x, &y);
      lpPoint->x += x;
      lpPoint->y += y;
    }
  return TRUE;
}


/* ClipCursor ordinal = 61, argument = 4 */
BOOL WINAPI ClipCursor (CONST LPRECT lpRect)
{
  return TRUE;
}


/* CloseClipboard ordinal = 62, argument = 0 */
BOOL WINAPI CloseClipboard (VOID)
{
  return TRUE;
}


/* CloseDesktop ordinal = 63, argument = 4 temporary */
static DWORD WINAPI CloseDesktop (DWORD dwArgv0)
{
  return 0;
}


/* CloseWindow ordinal = 64, argument = 4 */
BOOL WINAPI CloseWindow (HWND hWnd)
{
  if (!hWnd || !GTK_IS_WINDOW (hWnd))
    return FALSE;
  gtk_window_iconify (GTK_WINDOW (hWnd));
  return TRUE;
}


/* CloseWindowStation ordinal = 65, argument = 4 temporary */
static DWORD WINAPI CloseWindowStation (DWORD dwArgv0)
{
  return 0;
}


/* CopyAcceleratorTableA ordinal = 66, argument = 12 temporary */
static DWORD WINAPI CopyAcceleratorTableA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* CopyAcceleratorTableW ordinal = 67, argument = 12 temporary */
static DWORD WINAPI CopyAcceleratorTableW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* CopyIcon ordinal = 68, argument = 4 */
HICON WINAPI CopyIcon (HICON hIcon)
{
  return hIcon && GDK_IS_PIXBUF (hIcon) ? gdk_pixbuf_copy (hIcon) : NULL;
}


/* CopyImage ordinal = 69, argument = 20 */
HANDLE WINAPI CopyImage (HANDLE hImage, UINT uType, INT cxDesired, INT cyDesired, UINT fuFlags)
{
  HANDLE hCopy = NULL;

  if (GDK_IS_PIXBUF (hImage))
    {
      if (cxDesired > 0 && cyDesired > 0
                && (gdk_pixbuf_get_width (GDK_PIXBUF (hImage)) != cxDesired
                || gdk_pixbuf_get_height (GDK_PIXBUF (hImage)) != cyDesired))
        {
          hCopy = gdk_pixbuf_scale_simple (GDK_PIXBUF (hImage),
                                    cxDesired, cyDesired, GDK_INTERP_HYPER);
        }
      else if (fuFlags & LR_COPYRETURNORG)
        {
          g_object_ref (G_OBJECT (hImage));
          hCopy = hImage;
        }
      else
        {
          hCopy = gdk_pixbuf_copy (GDK_PIXBUF (hImage));
        }
      if (fuFlags & LR_COPYDELETEORG)
        g_object_unref (G_OBJECT (hImage));
    }
  else
    {
#if GLIB_CHECK_VERSION(2,8,0)
      const gchar *str;
      gint x = 0, y = 0;
      GdkPixbuf *src, *pixbuf = NULL;

      src = gdk_cursor_get_image ((GdkCursor *)hImage);
      str = gdk_pixbuf_get_option (src, "x_hot");
      if (str)
        misc_str_to_val (&x, str, 10, TRUE);
      str = gdk_pixbuf_get_option (src, "y_hot");
      if (str)
        misc_str_to_val (&y, str, 10, TRUE);
      if (src)
        {
          if (cxDesired > 0 && cyDesired > 0
                                && (gdk_pixbuf_get_width (src) != cxDesired
                                || gdk_pixbuf_get_height (src) != cyDesired))
            {
              x = cxDesired / gdk_pixbuf_get_width (src);
              y = cyDesired / gdk_pixbuf_get_height (src);
              pixbuf = gdk_pixbuf_scale_simple (src, cxDesired, cyDesired,
                                                            GDK_INTERP_HYPER);
            }
          else if (!(fuFlags & LR_COPYRETURNORG))
            {
              pixbuf = gdk_pixbuf_copy (src);
            }
        }
      if (pixbuf)
        {
          hCopy = gdk_cursor_new_from_pixbuf
                                    (gdk_display_get_default (), pixbuf, x, y);
          g_object_unref (G_OBJECT (pixbuf));
        }
      else
        {
#endif /* GLIB_CHECK_VERSION(2,8,0) */
          gdk_cursor_ref ((GdkCursor *)hImage);
          hCopy = hImage;
#if GLIB_CHECK_VERSION(2,8,0)
        }
#endif /* GLIB_CHECK_VERSION(2,8,0) */
      if (fuFlags & LR_COPYDELETEORG)
        gdk_cursor_unref ((GdkCursor *)hImage);
    }
  return hCopy;
}


/* CopyRect ordinal = 70, argument = 8 */
BOOL WINAPI CopyRect (LPRECT lprcDst, CONST LPRECT lprcSrc)
{
  if (!lprcDst || !lprcSrc)
    return FALSE;
  *lprcDst = *lprcSrc;
  return TRUE;
}


/* CountClipboardFormats ordinal = 71, argument = 0 temporary */
static DWORD WINAPI CountClipboardFormats (VOID)
{
  return 0;
}


/* CreateAcceleratorTableA ordinal = 72, argument = 8 temporary */
static DWORD WINAPI CreateAcceleratorTableA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CreateAcceleratorTableW ordinal = 73, argument = 8 temporary */
static DWORD WINAPI CreateAcceleratorTableW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* CreateCaret ordinal = 74, argument = 16 temporary */
static DWORD WINAPI CreateCaret (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CreateCursor ordinal = 75, argument = 28 temporary */
static DWORD WINAPI CreateCursor (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* CreateDesktopA ordinal = 76, argument = 24 temporary */
static DWORD WINAPI CreateDesktopA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* CreateDesktopW ordinal = 77, argument = 24 temporary */
static DWORD WINAPI CreateDesktopW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* CreateDialogIndirectParamA ordinal = 78, argument = 20 */
HWND WINAPI CreateDialogIndirectParamA (HINSTANCE hInstance, LPCDLGTEMPLATE lpDialogTemplate, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  return w32ldr_CreateDialogIndirectParam (hInstance,
                                           lpDialogTemplate,
                                           hWndParent,
                                           lpDialogFunc,
                                           dwInitParam,
                                           FALSE,
                                           FALSE);
}


/* CreateDialogIndirectParamAorW ordinal = 79, argument = 0 temporary */
static DWORD WINAPI CreateDialogIndirectParamAorW (VOID)
{
  return 0;
}


/* CreateDialogIndirectParamW ordinal = 80, argument = 20 */
HWND WINAPI CreateDialogIndirectParamW (HINSTANCE hInstance, LPCDLGTEMPLATE lpDialogTemplate, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  return w32ldr_CreateDialogIndirectParam (hInstance,
                                           lpDialogTemplate,
                                           hWndParent,
                                           lpDialogFunc,
                                           dwInitParam,
                                           FALSE,
                                           TRUE);
}


/* CreateDialogParamA ordinal = 81, argument = 20 */
HWND WINAPI CreateDialogParamA (HINSTANCE hInstance, LPCSTR lpTemplateName, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  LPCDLGTEMPLATE lpDialogTemplate;
  HRSRC hRsrc;

  return (hRsrc = FindResourceA (hInstance, lpTemplateName,
                                                MAKEINTRESOURCEA (RT_DIALOG)))
                        && (lpDialogTemplate = LoadResource (hInstance, hRsrc))
                    ? CreateDialogIndirectParamA (hInstance, lpDialogTemplate,
                                hWndParent, lpDialogFunc, dwInitParam) : NULL;
}


/* CreateDialogParamW ordinal = 82, argument = 20 */
HWND WINAPI CreateDialogParamW (HINSTANCE hInstance, LPCWSTR lpTemplateName, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  LPCDLGTEMPLATE lpDialogTemplate;
  HRSRC hRsrc;

  return (hRsrc = FindResourceW (hInstance, lpTemplateName,
                                                MAKEINTRESOURCEW (RT_DIALOG)))
                        && (lpDialogTemplate = LoadResource (hInstance, hRsrc))
                    ? CreateDialogIndirectParamW (hInstance, lpDialogTemplate,
                                hWndParent, lpDialogFunc, dwInitParam) : NULL;
}


/* CreateIcon ordinal = 83, argument = 28 temporary */
static DWORD WINAPI CreateIcon (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* CreateIconFromResource ordinal = 84, argument = 16 temporary */
static DWORD WINAPI CreateIconFromResource (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CreateIconFromResourceEx ordinal = 85, argument = 28 temporary */
static DWORD WINAPI CreateIconFromResourceEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* CreateIconIndirect ordinal = 86, argument = 4 temporary */
static DWORD WINAPI CreateIconIndirect (DWORD dwArgv0)
{
  return 0;
}


/* CreateMDIWindowA ordinal = 87, argument = 40 temporary */
static DWORD WINAPI CreateMDIWindowA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8, DWORD dwArgv9)
{
  return 0;
}


/* CreateMDIWindowW ordinal = 88, argument = 40 temporary */
static DWORD WINAPI CreateMDIWindowW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8, DWORD dwArgv9)
{
  return 0;
}


/* CreateMenu ordinal = 89, argument = 0 */
HMENU WINAPI CreateMenu (VOID)
{
  return gtk_menu_bar_new ();
}


/* CreatePopupMenu ordinal = 90, argument = 0 */
HMENU WINAPI CreatePopupMenu (VOID)
{
  return gtk_menu_new ();
}


/* CreateWindowExA ordinal = 91, argument = 48 */
HWND WINAPI CreateWindowExA (DWORD dwExStyle, LPCSTR lpClassName, LPCSTR lpWindowName, DWORD dwStyle, INT x, INT y, INT nWidth, INT nHeight, HWND hWndParent, HMENU hMenu, HINSTANCE hInstance, LPVOID lpParam)
{
  gchar *classname, *windowname;
  HWND hWnd;

  classname = GPOINTER_TO_UINT (lpClassName) > G_MAXUINT16
                ? w32ldr_utf8_from_mb (lpClassName) : (gchar *)lpClassName;
  windowname = GPOINTER_TO_UINT (lpWindowName) > G_MAXUINT16
                ? w32ldr_utf8_from_mb (lpWindowName) : (gchar *)lpWindowName;
  hWnd = w32ldr_CreateWindowEx (dwExStyle,
                                classname,
                                windowname,
                                dwStyle & ~DS_MODALFRAME,
                                x,
                                y,
                                nWidth,
                                nHeight,
                                hWndParent,
                                hMenu,
                                hInstance,
                                lpParam,
                                FALSE);
  if (GPOINTER_TO_UINT (classname) > G_MAXUINT16)
    g_free (classname);
  if (GPOINTER_TO_UINT (windowname) > G_MAXUINT16)
    g_free (windowname);
  return hWnd;
}


/* CreateWindowExW ordinal = 92, argument = 48 */
HWND WINAPI CreateWindowExW (DWORD dwExStyle, LPCWSTR lpClassName, LPCWSTR lpWindowName, DWORD dwStyle, INT x, INT y, INT nWidth, INT nHeight, HWND hWndParent, HMENU hMenu, HINSTANCE hInstance, LPVOID lpParam)
{
  gchar *classname, *windowname;
  HWND hWnd;

  classname = GPOINTER_TO_UINT (lpClassName) > G_MAXUINT16
                        ? g_utf16_to_utf8 (lpClassName, -1, NULL, NULL, NULL)
                        : (gchar *)lpClassName;
  windowname = GPOINTER_TO_UINT (lpWindowName) > G_MAXUINT16
                        ? g_utf16_to_utf8 (lpWindowName, -1, NULL, NULL, NULL)
                        : (gchar *)lpWindowName;
  hWnd = w32ldr_CreateWindowEx (dwExStyle,
                                classname,
                                windowname,
                                dwStyle & ~DS_MODALFRAME,
                                x,
                                y,
                                nWidth,
                                nHeight,
                                hWndParent,
                                hMenu,
                                hInstance,
                                lpParam,
                                TRUE);
  if (GPOINTER_TO_UINT (classname) > G_MAXUINT16)
    g_free (classname);
  if (GPOINTER_TO_UINT (windowname) > G_MAXUINT16)
    g_free (windowname);
  return hWnd;
}


/* CreateWindowStationA ordinal = 93, argument = 16 temporary */
static DWORD WINAPI CreateWindowStationA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CreateWindowStationW ordinal = 94, argument = 16 temporary */
static DWORD WINAPI CreateWindowStationW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* CtxInitUser32 ordinal = 95, argument = 0 temporary */
static DWORD WINAPI CtxInitUser32 (VOID)
{
  return 0;
}


/* DdeAbandonTransaction ordinal = 96, argument = 12 temporary */
static DWORD WINAPI DdeAbandonTransaction (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeAccessData ordinal = 97, argument = 8 temporary */
static DWORD WINAPI DdeAccessData (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* DdeAddData ordinal = 98, argument = 16 temporary */
static DWORD WINAPI DdeAddData (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DdeClientTransaction ordinal = 99, argument = 32 temporary */
static DWORD WINAPI DdeClientTransaction (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* DdeCmpStringHandles ordinal = 100, argument = 8 temporary */
static DWORD WINAPI DdeCmpStringHandles (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* DdeConnect ordinal = 101, argument = 16 temporary */
static DWORD WINAPI DdeConnect (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DdeConnectList ordinal = 102, argument = 20 temporary */
static DWORD WINAPI DdeConnectList (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DdeCreateDataHandle ordinal = 103, argument = 28 temporary */
static DWORD WINAPI DdeCreateDataHandle (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* DdeCreateStringHandleA ordinal = 104, argument = 12 temporary */
static DWORD WINAPI DdeCreateStringHandleA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeCreateStringHandleW ordinal = 105, argument = 12 temporary */
static DWORD WINAPI DdeCreateStringHandleW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeDisconnect ordinal = 106, argument = 4 temporary */
static DWORD WINAPI DdeDisconnect (DWORD dwArgv0)
{
  return 0;
}


/* DdeDisconnectList ordinal = 107, argument = 4 temporary */
static DWORD WINAPI DdeDisconnectList (DWORD dwArgv0)
{
  return 0;
}


/* DdeEnableCallback ordinal = 108, argument = 12 temporary */
static DWORD WINAPI DdeEnableCallback (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeFreeDataHandle ordinal = 109, argument = 4 temporary */
static DWORD WINAPI DdeFreeDataHandle (DWORD dwArgv0)
{
  return 0;
}


/* DdeFreeStringHandle ordinal = 110, argument = 8 temporary */
static DWORD WINAPI DdeFreeStringHandle (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* DdeGetData ordinal = 111, argument = 16 temporary */
static DWORD WINAPI DdeGetData (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DdeGetLastError ordinal = 112, argument = 4 temporary */
static DWORD WINAPI DdeGetLastError (DWORD dwArgv0)
{
  return 0;
}


/* DdeGetQualityOfService ordinal = 113, argument = 0 temporary */
static DWORD WINAPI DdeGetQualityOfService (VOID)
{
  return 0;
}


/* DdeImpersonateClient ordinal = 114, argument = 4 temporary */
static DWORD WINAPI DdeImpersonateClient (DWORD dwArgv0)
{
  return 0;
}


/* DdeInitializeA ordinal = 115, argument = 16 temporary */
static DWORD WINAPI DdeInitializeA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DdeInitializeW ordinal = 116, argument = 16 temporary */
static DWORD WINAPI DdeInitializeW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DdeKeepStringHandle ordinal = 117, argument = 8 temporary */
static DWORD WINAPI DdeKeepStringHandle (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* DdeNameService ordinal = 118, argument = 16 temporary */
static DWORD WINAPI DdeNameService (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DdePostAdvise ordinal = 119, argument = 12 temporary */
static DWORD WINAPI DdePostAdvise (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeQueryConvInfo ordinal = 120, argument = 12 temporary */
static DWORD WINAPI DdeQueryConvInfo (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeQueryNextServer ordinal = 121, argument = 8 temporary */
static DWORD WINAPI DdeQueryNextServer (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* DdeQueryStringA ordinal = 122, argument = 20 temporary */
static DWORD WINAPI DdeQueryStringA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DdeQueryStringW ordinal = 123, argument = 20 temporary */
static DWORD WINAPI DdeQueryStringW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DdeReconnect ordinal = 124, argument = 4 temporary */
static DWORD WINAPI DdeReconnect (DWORD dwArgv0)
{
  return 0;
}


/* DdeSetQualityOfService ordinal = 125, argument = 12 temporary */
static DWORD WINAPI DdeSetQualityOfService (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeSetUserHandle ordinal = 126, argument = 12 temporary */
static DWORD WINAPI DdeSetUserHandle (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DdeUnaccessData ordinal = 127, argument = 4 temporary */
static DWORD WINAPI DdeUnaccessData (DWORD dwArgv0)
{
  return 0;
}


/* DdeUninitialize ordinal = 128, argument = 4 temporary */
static DWORD WINAPI DdeUninitialize (DWORD dwArgv0)
{
  return 0;
}


/* DefDlgProcA ordinal = 129, argument = 16 temporary */
static DWORD WINAPI DefDlgProcA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DefDlgProcW ordinal = 130, argument = 16 temporary */
static DWORD WINAPI DefDlgProcW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DefFrameProcA ordinal = 131, argument = 20 temporary */
static DWORD WINAPI DefFrameProcA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DefFrameProcW ordinal = 132, argument = 20 temporary */
static DWORD WINAPI DefFrameProcW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DefMDIChildProcA ordinal = 133, argument = 16 temporary */
static DWORD WINAPI DefMDIChildProcA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DefMDIChildProcW ordinal = 134, argument = 16 temporary */
static DWORD WINAPI DefMDIChildProcW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DefWindowProcA ordinal = 135, argument = 16 */
LRESULT WINAPI DefWindowProcA (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return DefWindowProcW (hWnd, Msg, wParam, lParam);
}


/* DefWindowProcW ordinal = 136, argument = 16 */
LRESULT WINAPI DefWindowProcW (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  W32LdrWindowData *wd;

  if (!hWnd)
    return 0;
  wd = g_object_get_data (G_OBJECT (hWnd), "user_data");
  if (!wd)
    return 0;
  switch (Msg)
    {
      case WM_CLOSE:
        if (GTK_IS_WINDOW (hWnd))
          gtk_widget_destroy (hWnd);
        return 0;
      case WM_ENABLE:
        gtk_widget_set_sensitive (hWnd, wParam);
        return 0;
      case WM_SHOWWINDOW:
        if (wParam)
          gtk_widget_show (hWnd);
        else
          gtk_widget_hide (hWnd);
        return 0;
      case WM_GETTEXT:
        if (GTK_IS_WINDOW (hWnd))
          {
            const gchar *utf8str = NULL;

            utf8str = gtk_window_get_title (GTK_WINDOW (hWnd));
            if (wParam > 0 && lParam && utf8str)
              {
                if (wd->widechar)
                  {
                    gunichar2 *wc;

                    wc = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
                    lstrcpynW (GINT_TO_POINTER (lParam), wc, wParam);
                    g_free (wc);
                    return lstrlenW (GINT_TO_POINTER (lParam));
                  }
                else
                  {
                    gchar *mb;

                    mb = w32ldr_utf8_to_mb (utf8str);
                    g_strncpy (GINT_TO_POINTER (lParam), mb, wParam);
                    g_free (mb);
                    return g_strlen (GINT_TO_POINTER (lParam));
                  }
              }
          }
        return 0;
      case WM_GETTEXTLENGTH:
        if (GTK_IS_WINDOW (hWnd))
          {
            const gchar *utf8str = NULL;

            utf8str = gtk_window_get_title (GTK_WINDOW (hWnd));
            if (utf8str)
              {
                LRESULT lResult;

                if (wd->widechar)
                  {
                    gunichar2 *wc;

                    wc = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
                    lResult = lstrlenW (wc);
                    g_free (wc);
                  }
                else
                  {
                    gchar *mb;

                    mb = w32ldr_utf8_to_mb (utf8str);
                    lResult = g_strlen (mb);
                    g_free (mb);
                  }
                return lResult;
              }
          }
        return 0;
      case WM_SETTEXT:
        if (GTK_IS_WINDOW (hWnd))
          {
            gchar *utf8str;

            utf8str = lParam ? wd->widechar
                             ? g_utf16_to_utf8 (GINT_TO_POINTER (lParam),
                                                        -1, NULL, NULL, NULL)
                             : w32ldr_utf8_from_mb (GINT_TO_POINTER (lParam))
                             : g_strdup ("");
            gtk_window_set_title (GTK_WINDOW (hWnd), utf8str);
            g_free (utf8str);
          }
        return TRUE;
    }
  return 0;
}


/* DeferWindowPos ordinal = 137, argument = 32 temporary */
static DWORD WINAPI DeferWindowPos (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* DeleteMenu ordinal = 138, argument = 12 */
BOOL WINAPI DeleteMenu (HMENU hMenu, UINT uItem, UINT uFlags)
{
  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item)
        {
          gtk_container_remove (GTK_CONTAINER (hMenu), menu_item);
          return TRUE;
        }
    }
  return FALSE;
}


/* DeregisterShellHookWindow ordinal = 139, argument = 4 temporary */
static DWORD WINAPI DeregisterShellHookWindow (DWORD dwArgv0)
{
  return 0;
}


/* DestroyAcceleratorTable ordinal = 140, argument = 4 temporary */
static DWORD WINAPI DestroyAcceleratorTable (DWORD dwArgv0)
{
  return 0;
}


/* DestroyCaret ordinal = 141, argument = 0 temporary */
static DWORD WINAPI DestroyCaret (VOID)
{
  return 0;
}


/* DestroyCursor ordinal = 142, argument = 4 */
BOOL WINAPI DestroyCursor (HCURSOR hCursor)
{
  if (!hCursor)
    return FALSE;
  gdk_cursor_unref (hCursor);
  return TRUE;
}


/* DestroyIcon ordinal = 143, argument = 4 */
BOOL WINAPI DestroyIcon (HICON hIcon)
{
  if (!hIcon || !GDK_IS_PIXBUF (hIcon))
    return FALSE;
  g_object_unref (G_OBJECT (hIcon));
  return TRUE;
}


/* DestroyMenu ordinal = 144, argument = 4 */
BOOL WINAPI DestroyMenu (HMENU hMenu)
{
  if (!hMenu || !GTK_IS_MENU_SHELL (hMenu))
    return FALSE;
  gtk_widget_destroy (hMenu);
  return TRUE;
}


/* DestroyWindow ordinal = 145, argument = 4 */
BOOL WINAPI DestroyWindow (HWND hWnd)
{
  if (!hWnd || !GTK_IS_WIDGET (hWnd))
    return FALSE;
  gtk_widget_destroy (hWnd);
  return TRUE;
}


/* DeviceEventWorker ordinal = 146, argument = 0 temporary */
static DWORD WINAPI DeviceEventWorker (VOID)
{
  return 0;
}


/* DialogBoxIndirectParamA ordinal = 147, argument = 20 */
INT WINAPI DialogBoxIndirectParamA (HINSTANCE hInstance, LPCDLGTEMPLATE lpDialogTemplate, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  gint result;
  GtkWidget *dialog;

  dialog = w32ldr_CreateDialogIndirectParam (hInstance,
                                             lpDialogTemplate,
                                             hWndParent,
                                             lpDialogFunc,
                                             dwInitParam,
                                             TRUE,
                                             FALSE);
  result = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
  return result;
}


/* DialogBoxIndirectParamAorW ordinal = 148, argument = 0 temporary */
static DWORD WINAPI DialogBoxIndirectParamAorW (VOID)
{
  return 0;
}


/* DialogBoxIndirectParamW ordinal = 149, argument = 20 */
INT WINAPI DialogBoxIndirectParamW (HINSTANCE hInstance, LPCDLGTEMPLATE lpDialogTemplate, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  gint result;
  GtkWidget *dialog;

  dialog = w32ldr_CreateDialogIndirectParam (hInstance,
                                             lpDialogTemplate,
                                             hWndParent,
                                             lpDialogFunc,
                                             dwInitParam,
                                             TRUE,
                                             TRUE);
  result = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
  return result;
}


/* DialogBoxParamA ordinal = 150, argument = 20 */
INT WINAPI DialogBoxParamA (HINSTANCE hInstance, LPCSTR lpTemplateName, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  LPCDLGTEMPLATE lpDialogTemplate;
  HRSRC hRsrc;

  return (hRsrc = FindResourceA (hInstance, lpTemplateName,
                                                MAKEINTRESOURCEA (RT_DIALOG)))
                        && (lpDialogTemplate = LoadResource (hInstance, hRsrc))
        ? DialogBoxIndirectParamA (hInstance, lpDialogTemplate,
                                hWndParent, lpDialogFunc, dwInitParam) : -1;
}


/* DialogBoxParamW ordinal = 151, argument = 20 */
INT WINAPI DialogBoxParamW (HINSTANCE hInstance, LPCWSTR lpTemplateName, HWND hWndParent, DLGPROC lpDialogFunc, LPARAM dwInitParam)
{
  LPCDLGTEMPLATE lpDialogTemplate;
  HRSRC hRsrc;

  return (hRsrc = FindResourceW (hInstance, lpTemplateName,
                                                MAKEINTRESOURCEW (RT_DIALOG)))
                        && (lpDialogTemplate = LoadResource (hInstance, hRsrc))
        ? DialogBoxIndirectParamW (hInstance, lpDialogTemplate,
                                hWndParent, lpDialogFunc, dwInitParam) : -1;
}


/* DispatchMessageA ordinal = 152, argument = 4 */
LONG WINAPI DispatchMessageA (LPMSG lpMsg)
{
  return SendMessageA (lpMsg->hwnd, lpMsg->message,
                                                lpMsg->wParam, lpMsg->lParam);
}


/* DispatchMessageW ordinal = 153, argument = 4 */
LONG WINAPI DispatchMessageW (LPMSG lpMsg)
{
  return SendMessageW (lpMsg->hwnd, lpMsg->message,
                                                lpMsg->wParam, lpMsg->lParam);
}


/* DlgDirListA ordinal = 154, argument = 20 temporary */
static DWORD WINAPI DlgDirListA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DlgDirListComboBoxA ordinal = 155, argument = 20 temporary */
static DWORD WINAPI DlgDirListComboBoxA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DlgDirListComboBoxW ordinal = 156, argument = 20 temporary */
static DWORD WINAPI DlgDirListComboBoxW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DlgDirListW ordinal = 157, argument = 20 temporary */
static DWORD WINAPI DlgDirListW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DlgDirSelectComboBoxExA ordinal = 158, argument = 16 temporary */
static DWORD WINAPI DlgDirSelectComboBoxExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DlgDirSelectComboBoxExW ordinal = 159, argument = 16 temporary */
static DWORD WINAPI DlgDirSelectComboBoxExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DlgDirSelectExA ordinal = 160, argument = 16 temporary */
static DWORD WINAPI DlgDirSelectExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DlgDirSelectExW ordinal = 161, argument = 16 temporary */
static DWORD WINAPI DlgDirSelectExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DragDetect ordinal = 162, argument = 12 temporary */
static DWORD WINAPI DragDetect (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* DragObject ordinal = 163, argument = 20 temporary */
static DWORD WINAPI DragObject (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DrawAnimatedRects ordinal = 164, argument = 16 temporary */
static DWORD WINAPI DrawAnimatedRects (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DrawCaption ordinal = 165, argument = 16 temporary */
static DWORD WINAPI DrawCaption (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DrawCaptionTempA ordinal = 166, argument = 0 temporary */
static DWORD WINAPI DrawCaptionTempA (VOID)
{
  return 0;
}


/* DrawCaptionTempW ordinal = 167, argument = 0 temporary */
static DWORD WINAPI DrawCaptionTempW (VOID)
{
  return 0;
}


/* DrawEdge ordinal = 168, argument = 16 temporary */
static DWORD WINAPI DrawEdge (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DrawFocusRect ordinal = 169, argument = 8 temporary */
static DWORD WINAPI DrawFocusRect (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* DrawFrame ordinal = 170, argument = 16 temporary */
static DWORD WINAPI DrawFrame (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DrawFrameControl ordinal = 171, argument = 16 temporary */
static DWORD WINAPI DrawFrameControl (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DrawIcon ordinal = 172, argument = 16 temporary */
static DWORD WINAPI DrawIcon (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* DrawIconEx ordinal = 173, argument = 36 temporary */
static DWORD WINAPI DrawIconEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8)
{
  return 0;
}


/* DrawMenuBar ordinal = 174, argument = 4 */
BOOL WINAPI DrawMenuBar (HWND hWnd)
{
  return TRUE;
}


/* DrawMenuBarTemp ordinal = 175, argument = 0 temporary */
static DWORD WINAPI DrawMenuBarTemp (VOID)
{
  return 0;
}


/* DrawStateA ordinal = 176, argument = 40 temporary */
static DWORD WINAPI DrawStateA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8, DWORD dwArgv9)
{
  return 0;
}


/* DrawStateW ordinal = 177, argument = 40 temporary */
static DWORD WINAPI DrawStateW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8, DWORD dwArgv9)
{
  return 0;
}


/* DrawTextA ordinal = 178, argument = 20 temporary */
static DWORD WINAPI DrawTextA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* DrawTextExA ordinal = 179, argument = 24 temporary */
static DWORD WINAPI DrawTextExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* DrawTextExW ordinal = 180, argument = 24 temporary */
static DWORD WINAPI DrawTextExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* DrawTextW ordinal = 181, argument = 20 temporary */
static DWORD WINAPI DrawTextW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* EditWndProc ordinal = 182, argument = 16 temporary */
static DWORD WINAPI EditWndProc (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* EmptyClipboard ordinal = 183, argument = 0 */
BOOL WINAPI EmptyClipboard (VOID)
{
  GtkClipboard *clipboard;

  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (clipboard)
    gtk_clipboard_clear (clipboard);
  return TRUE;
}


/* EnableMenuItem ordinal = 184, argument = 12 */
BOOL EnableMenuItem (HMENU hMenu, UINT uItem, UINT uFlags)
{
  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item)
        {
          BOOL fResult;

          fResult = GTK_WIDGET_SENSITIVE (menu_item) ? MF_ENABLED : MFS_GRAYED;
          gtk_widget_set_sensitive (menu_item, !(uFlags & MFS_GRAYED));
          return fResult;
        }
    }
  return (BOOL)-1;
}


/* EnableScrollBar ordinal = 185, argument = 12 temporary */
static DWORD WINAPI EnableScrollBar (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnableWindow ordinal = 186, argument = 8 */
BOOL WINAPI EnableWindow (HWND hWnd, BOOL bEnable)
{
  if (!hWnd)
    return FALSE;
  if (IsWindowUnicode (hWnd))
    SendMessageW (hWnd, WM_ENABLE, bEnable, 0);
  else
    SendMessageA (hWnd, WM_ENABLE, bEnable, 0);
  return TRUE;
}


/* EndDeferWindowPos ordinal = 187, argument = 4 temporary */
static DWORD WINAPI EndDeferWindowPos (DWORD dwArgv0)
{
  return 0;
}


/* EndDialog ordinal = 188, argument = 8 */
BOOL WINAPI EndDialog (HWND hDlg, INT nResult)
{
  if (hDlg && GTK_IS_DIALOG (hDlg))
    {
      gtk_dialog_response (GTK_DIALOG (hDlg), nResult);
      return TRUE;
    }
  return FALSE;
}


/* EndMenu ordinal = 189, argument = 0 */
BOOL WINAPI EndMenu (VOID)
{
  return TRUE;
}


/* EndPaint ordinal = 190, argument = 8 */
BOOL WINAPI EndPaint (HWND hWnd, LPPAINTSTRUCT lpPaint)
{
  return lpPaint && ReleaseDC (hWnd, lpPaint->hdc);
}


/* EndTask ordinal = 191, argument = 12 temporary */
static DWORD WINAPI EndTask (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumChildWindows ordinal = 192, argument = 12 temporary */
static DWORD WINAPI EnumChildWindows (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumClipboardFormats ordinal = 193, argument = 4 temporary */
static DWORD WINAPI EnumClipboardFormats (DWORD dwArgv0)
{
  return 0;
}


/* EnumDesktopWindows ordinal = 194, argument = 12 temporary */
static DWORD WINAPI EnumDesktopWindows (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumDesktopsA ordinal = 195, argument = 12 temporary */
static DWORD WINAPI EnumDesktopsA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumDesktopsW ordinal = 196, argument = 12 temporary */
static DWORD WINAPI EnumDesktopsW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumDisplayDevicesA ordinal = 197, argument = 16 temporary */
static DWORD WINAPI EnumDisplayDevicesA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* EnumDisplayDevicesW ordinal = 198, argument = 16 temporary */
static DWORD WINAPI EnumDisplayDevicesW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* EnumDisplayMonitors ordinal = 199, argument = 16 temporary */
static DWORD WINAPI EnumDisplayMonitors (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* EnumDisplaySettingsA ordinal = 200, argument = 12 temporary */
static DWORD WINAPI EnumDisplaySettingsA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumDisplaySettingsExA ordinal = 201, argument = 16 temporary */
static DWORD WINAPI EnumDisplaySettingsExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* EnumDisplaySettingsExW ordinal = 202, argument = 16 temporary */
static DWORD WINAPI EnumDisplaySettingsExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* EnumDisplaySettingsW ordinal = 203, argument = 12 temporary */
static DWORD WINAPI EnumDisplaySettingsW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumPropsA ordinal = 204, argument = 8 temporary */
static DWORD WINAPI EnumPropsA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* EnumPropsExA ordinal = 205, argument = 12 temporary */
static DWORD WINAPI EnumPropsExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumPropsExW ordinal = 206, argument = 12 temporary */
static DWORD WINAPI EnumPropsExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* EnumPropsW ordinal = 207, argument = 8 temporary */
static DWORD WINAPI EnumPropsW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* EnumThreadWindows ordinal = 208, argument = 12 */
BOOL WINAPI EnumThreadWindows (DWORD dwThreadId, WNDENUMPROC lpEnumFunc, LPARAM lParam)
{
  return EnumWindows (lpEnumFunc, lParam);
}


/* EnumWindowStationsA ordinal = 209, argument = 8 temporary */
static DWORD WINAPI EnumWindowStationsA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* EnumWindowStationsW ordinal = 210, argument = 8 temporary */
static DWORD WINAPI EnumWindowStationsW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* EnumWindows ordinal = 211, argument = 8 */
BOOL WINAPI EnumWindows (WNDENUMPROC lpEnumFunc, LPARAM lParam)
{
  GList *glist_toplevels;

  glist_toplevels = gtk_window_list_toplevels ();
  if (glist_toplevels)
    {
      GList *glist;

      for (glist = g_list_first (glist_toplevels); glist;
                                                glist = g_list_next (glist))
        if (!lpEnumFunc (glist->data, lParam))
          break;
      g_list_foreach (glist_toplevels, (GFunc)g_object_ref, NULL);
      g_list_free (glist_toplevels);
    }
  return TRUE;
}


/* EqualRect ordinal = 212, argument = 8 */
BOOL WINAPI EqualRect (CONST LPRECT lprc1, CONST LPRECT lprc2)
{
  if (!lprc1 || !lprc2)
    return FALSE;
  return (lprc1->left == lprc2->left && lprc1->top == lprc2->top
            && lprc1->right == lprc2->right && lprc1->bottom == lprc2->bottom);
}


/* ExcludeUpdateRgn ordinal = 213, argument = 8 temporary */
static DWORD WINAPI ExcludeUpdateRgn (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ExitWindowsEx ordinal = 214, argument = 8 temporary */
static DWORD WINAPI ExitWindowsEx (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* FillRect ordinal = 215, argument = 12 temporary */
static DWORD WINAPI FillRect (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* FindWindowA ordinal = 216, argument = 8 temporary */
static DWORD WINAPI FindWindowA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* FindWindowExA ordinal = 217, argument = 16 temporary */
static DWORD WINAPI FindWindowExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* FindWindowExW ordinal = 218, argument = 16 temporary */
static DWORD WINAPI FindWindowExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* FindWindowW ordinal = 219, argument = 8 temporary */
static DWORD WINAPI FindWindowW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* FlashWindow ordinal = 220, argument = 8 temporary */
static DWORD WINAPI FlashWindow (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* FlashWindowEx ordinal = 221, argument = 4 temporary */
static DWORD WINAPI FlashWindowEx (DWORD dwArgv0)
{
  return 0;
}


/* FrameRect ordinal = 222, argument = 12 temporary */
static DWORD WINAPI FrameRect (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* FreeDDElParam ordinal = 223, argument = 8 temporary */
static DWORD WINAPI FreeDDElParam (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetActiveWindow ordinal = 224, argument = 0 temporary */
static DWORD WINAPI GetActiveWindow (VOID)
{
  return 0;
}


/* GetAltTabInfo ordinal = 225, argument = 20 temporary */
/* DWORD WINAPI GetAltTabInfo (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4) */


/* GetAltTabInfoA ordinal = 226, argument = 20 temporary */
static DWORD WINAPI GetAltTabInfoA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetAltTabInfoW ordinal = 227, argument = 20 temporary */
static DWORD WINAPI GetAltTabInfoW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetAncestor ordinal = 228, argument = 8 temporary */
static DWORD WINAPI GetAncestor (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetAppCompatFlags ordinal = 230, argument = 0 temporary */
static DWORD WINAPI GetAppCompatFlags (VOID)
{
  return 0;
}


/* GetAppCompatFlags2 ordinal = 229, argument = 0 temporary */
static DWORD WINAPI GetAppCompatFlags2 (VOID)
{
  return 0;
}


/* GetAsyncKeyState ordinal = 231, argument = 4 temporary */
static DWORD WINAPI GetAsyncKeyState (DWORD dwArgv0)
{
  return 0;
}


/* GetCapture ordinal = 232, argument = 0 temporary */
static DWORD WINAPI GetCapture (VOID)
{
  return 0;
}


/* GetCaretBlinkTime ordinal = 233, argument = 0 temporary */
static DWORD WINAPI GetCaretBlinkTime (VOID)
{
  return 0;
}


/* GetCaretPos ordinal = 234, argument = 4 temporary */
static DWORD WINAPI GetCaretPos (DWORD dwArgv0)
{
  return 0;
}


/* GetClassInfoA ordinal = 235, argument = 12 temporary */
static DWORD WINAPI GetClassInfoA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetClassInfoExA ordinal = 236, argument = 12 temporary */
static DWORD WINAPI GetClassInfoExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetClassInfoExW ordinal = 237, argument = 12 temporary */
static DWORD WINAPI GetClassInfoExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetClassInfoW ordinal = 238, argument = 12 temporary */
static DWORD WINAPI GetClassInfoW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetClassLongA ordinal = 239, argument = 8 temporary */
static DWORD WINAPI GetClassLongA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetClassLongW ordinal = 240, argument = 8 temporary */
static DWORD WINAPI GetClassLongW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetClassNameA ordinal = 241, argument = 12 */
INT WINAPI GetClassNameA (HWND hWnd, LPSTR lpClassName, INT nMaxCount)
{
  INT nResult = 0;
  W32LdrWindowData *wd;

  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  if (wd && lpClassName)
    {
      if (wd->classname)
        {
          gchar *mb;

          mb = w32ldr_utf8_to_mb (wd->classname);
          nResult = g_strlen (mb);
          if (nResult < nMaxCount)
            g_strcpy (lpClassName, mb);
          else
            nResult = 0;
          g_free (mb);
        }
      else if (nMaxCount > 0)
        {
          lpClassName[0] = '\0';
        }
    }
  return nResult;
}


/* GetClassNameW ordinal = 242, argument = 12 */
INT WINAPI GetClassNameW (HWND hWnd, LPWSTR lpClassName, INT nMaxCount)
{
  INT nResult = 0;
  W32LdrWindowData *wd;

  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  if (wd && lpClassName)
    {
      if (wd->classname)
        {
          gunichar2 *wc;

          wc = g_utf8_to_utf16 (wd->classname, -1, NULL, NULL, NULL);
          nResult = lstrlenW (wc);
          if (nResult < nMaxCount)
            lstrcpyW (lpClassName, wc);
          else
            nResult = 0;
          g_free (wc);
        }
      else if (nMaxCount > 0)
        {
          lpClassName[0] = '\0';
        }
    }
  return nResult;
}


/* GetClassWord ordinal = 243, argument = 8 temporary */
static DWORD WINAPI GetClassWord (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetClientRect ordinal = 244, argument = 8 */
BOOL WINAPI GetClientRect (HWND hWnd, LPRECT lpRect)
{
  GtkRequisition requisition;
  GtkWidget *widget = NULL;

  if (!hWnd || !lpRect)
    return FALSE;
  if (GTK_IS_WINDOW (hWnd))
    {
      W32LdrOverlapData *od;

      od = g_object_get_data (G_OBJECT (hWnd), "user_data");
      widget = od->fixed;
    }
  if (!widget)
    widget = hWnd;
  gtk_widget_size_request (widget, &requisition);
  lpRect->left = 0;
  lpRect->top = 0;
  lpRect->right = MAX (widget->allocation.width, requisition.width);
  lpRect->bottom = MAX (widget->allocation.height, requisition.height);
  return TRUE;
}


/* GetClipCursor ordinal = 245, argument = 4 temporary */
static DWORD WINAPI GetClipCursor (DWORD dwArgv0)
{
  return 0;
}


/* GetClipboardData ordinal = 246, argument = 4 temporary */
static DWORD WINAPI GetClipboardData (DWORD dwArgv0)
{
  return 0;
}


/* GetClipboardFormatNameA ordinal = 247, argument = 12 temporary */
static DWORD WINAPI GetClipboardFormatNameA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetClipboardFormatNameW ordinal = 248, argument = 12 temporary */
static DWORD WINAPI GetClipboardFormatNameW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetClipboardOwner ordinal = 249, argument = 0 temporary */
static DWORD WINAPI GetClipboardOwner (VOID)
{
  return 0;
}


/* GetClipboardSequenceNumber ordinal = 250, argument = 0 temporary */
static DWORD WINAPI GetClipboardSequenceNumber (VOID)
{
  return 0;
}


/* GetClipboardViewer ordinal = 251, argument = 0 temporary */
static DWORD WINAPI GetClipboardViewer (VOID)
{
  return 0;
}


/* GetComboBoxInfo ordinal = 252, argument = 8 temporary */
static DWORD WINAPI GetComboBoxInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetCursor ordinal = 253, argument = 0 */
HCURSOR WINAPI GetCursor (VOID)
{
  return NULL;
}


/* GetCursorFrameInfo ordinal = 254, argument = 0 temporary */
static DWORD WINAPI GetCursorFrameInfo (VOID)
{
  return 0;
}


/* GetCursorInfo ordinal = 255, argument = 4 temporary */
static DWORD WINAPI GetCursorInfo (DWORD dwArgv0)
{
  return 0;
}


/* GetCursorPos ordinal = 256, argument = 4 */
BOOL WINAPI GetCursorPos (LPPOINT lpPoint)
{
#if GTK_CHECK_VERSION(2,2,0)
  if (lpPoint)
    {
      gint x, y;

      gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
      lpPoint->x = x;
      lpPoint->y = y;
      return TRUE;
    }
#endif /* GTK_CHECK_VERSION(2,2,0) */
  return FALSE;
}


/* GetDC ordinal = 257, argument = 4 */
HDC WINAPI GetDC (HWND hWnd)
{
  W32LdrDC *w32dc = NULL;

  if (hWnd)
    {
      w32dc = g_malloc0 (sizeof (W32LdrDC));
      w32dc->widget = hWnd;
      w32dc->bgcolor.red = G_MAXUINT16;
      w32dc->bgcolor.green = G_MAXUINT16;
      w32dc->bgcolor.blue = G_MAXUINT16;
      w32dc->pixbuf = gdk_pixbuf_get_from_drawable (NULL,
            w32dc->widget->window, gdk_colormap_get_system (),
            w32dc->widget->allocation.x, w32dc->widget->allocation.y, 0, 0,
            w32dc->widget->allocation.width, w32dc->widget->allocation.height);
      if (!w32dc->pixbuf)
        w32dc->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 1, 1);
      w32dc->brush = w32ldr_brush_new ();
      w32dc->font = w32ldr_font_new ();
      w32dc->pen = w32ldr_pen_new ();
    }
  return w32dc;
}


/* GetDCEx ordinal = 258, argument = 12 temporary */
static DWORD WINAPI GetDCEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetDesktopWindow ordinal = 259, argument = 0 */
HWND WINAPI GetDesktopWindow (VOID)
{
  return NULL;
}


/* GetDialogBaseUnits ordinal = 260, argument = 0 temporary */
static DWORD WINAPI GetDialogBaseUnits (VOID)
{
  return 0;
}


/* GetDlgCtrlID ordinal = 261, argument = 4 */
INT WINAPI GetDlgCtrlID (HWND hWndCtl)
{
  W32LdrWindowData *wd;

  wd = hWndCtl ? g_object_get_data (G_OBJECT (hWndCtl), "user_data") : NULL;
  return wd ? wd->wID : 0;
}


/* GetDlgItem ordinal = 262, argument = 8 */
HWND WINAPI GetDlgItem (HWND hDlg, INT nIDDlgItem)
{
  gint i;
  W32LdrWindowData *dlg;

  dlg = g_object_get_data (G_OBJECT (hDlg), "user_data");
  if (dlg)
    for (i = 0; dlg->child[i]; i++)
      {
        W32LdrWindowData *wd;

        wd = g_object_get_data (G_OBJECT (dlg->child[i]), "user_data");
        if (wd && wd->wID == (WORD)nIDDlgItem){
          return dlg->child[i];}
      }
  return NULL;
}


/* GetDlgItemInt ordinal = 263, argument = 16 */
UINT WINAPI GetDlgItemInt (HWND hDlg, INT nIDDlgItem, BOOL *lpTranslated, BOOL bSigned)
{
  gboolean result;
  const gchar *text;
  gint value = 0;
  GtkWidget *widget;

  text = (widget = GetDlgItem (hDlg, nIDDlgItem))
                ? gtk_label_get_text (GTK_LABEL (GTK_IS_BUTTON (widget)
                || GTK_IS_CHECK_BUTTON (widget) || GTK_IS_RADIO_BUTTON (widget)
                    ? gtk_bin_get_child (GTK_BIN (widget)) : widget)) : NULL;
  result = text ? misc_str_to_val (&value, text, 10, bSigned) : FALSE;
  if (lpTranslated)
    *lpTranslated = result;
  return value;
}


/* GetDlgItemTextA ordinal = 264, argument = 16 */
UINT WINAPI GetDlgItemTextA (HWND hDlg, INT nIDDlgItem, LPSTR lpString, INT nMaxCount)
{
  return GetWindowTextA (GetDlgItem (hDlg, nIDDlgItem), lpString, nMaxCount);
}


/* GetDlgItemTextW ordinal = 265, argument = 16 */
UINT WINAPI GetDlgItemTextW (HWND hDlg, INT nIDDlgItem, LPWSTR lpString, INT nMaxCount)
{
  return GetWindowTextW (GetDlgItem (hDlg, nIDDlgItem), lpString, nMaxCount);
}


/* GetDoubleClickTime ordinal = 266, argument = 0 temporary */
static DWORD WINAPI GetDoubleClickTime (VOID)
{
  return 0;
}


/* GetFocus ordinal = 267, argument = 0 */
HWND WINAPI GetFocus (VOID)
{
  return NULL;
}


/* GetForegroundWindow ordinal = 268, argument = 0 temporary */
static DWORD WINAPI GetForegroundWindow (VOID)
{
  return 0;
}


/* GetGUIThreadInfo ordinal = 269, argument = 8 temporary */
static DWORD WINAPI GetGUIThreadInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetGuiResources ordinal = 270, argument = 8 temporary */
static DWORD WINAPI GetGuiResources (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetIconInfo ordinal = 271, argument = 8 temporary */
static DWORD WINAPI GetIconInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetInputDesktop ordinal = 272, argument = 0 temporary */
static DWORD WINAPI GetInputDesktop (VOID)
{
  return 0;
}


/* GetInputState ordinal = 273, argument = 0 temporary */
static DWORD WINAPI GetInputState (VOID)
{
  return 0;
}


/* GetInternalWindowPos ordinal = 274, argument = 0 temporary */
static DWORD WINAPI GetInternalWindowPos (VOID)
{
  return 0;
}


/* GetKBCodePage ordinal = 275, argument = 0 temporary */
static DWORD WINAPI GetKBCodePage (VOID)
{
  return 0;
}


/* GetKeyNameTextA ordinal = 276, argument = 12 temporary */
static DWORD WINAPI GetKeyNameTextA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetKeyNameTextW ordinal = 277, argument = 12 temporary */
static DWORD WINAPI GetKeyNameTextW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetKeyState ordinal = 278, argument = 4 temporary */
static DWORD WINAPI GetKeyState (DWORD dwArgv0)
{
  return 0;
}


/* GetKeyboardLayout ordinal = 279, argument = 4 temporary */
static DWORD WINAPI GetKeyboardLayout (DWORD dwArgv0)
{
  return 0;
}


/* GetKeyboardLayoutList ordinal = 280, argument = 8 temporary */
static DWORD WINAPI GetKeyboardLayoutList (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetKeyboardLayoutNameA ordinal = 281, argument = 4 temporary */
static DWORD WINAPI GetKeyboardLayoutNameA (DWORD dwArgv0)
{
  return 0;
}


/* GetKeyboardLayoutNameW ordinal = 282, argument = 4 temporary */
static DWORD WINAPI GetKeyboardLayoutNameW (DWORD dwArgv0)
{
  return 0;
}


/* GetKeyboardState ordinal = 283, argument = 4 temporary */
static DWORD WINAPI GetKeyboardState (DWORD dwArgv0)
{
  return 0;
}


/* GetKeyboardType ordinal = 284, argument = 4 */
INT WINAPI GetKeyboardType (INT nTypeFlag)
{
  return 0;
}


/* GetLastActivePopup ordinal = 285, argument = 4 temporary */
static DWORD WINAPI GetLastActivePopup (DWORD dwArgv0)
{
  return 0;
}


/* GetLastInputInfo ordinal = 286, argument = 4 temporary */
static DWORD WINAPI GetLastInputInfo (DWORD dwArgv0)
{
  return 0;
}


/* GetListBoxInfo ordinal = 287, argument = 4 temporary */
static DWORD WINAPI GetListBoxInfo (DWORD dwArgv0)
{
  return 0;
}


/* GetMenu ordinal = 288, argument = 4 */
HMENU WINAPI GetMenu (HWND hWnd)
{
  HMENU hMenu = NULL;

  if (hWnd && GTK_IS_WINDOW (hWnd))
    {
      W32LdrOverlapData *od;

      od = g_object_get_data (G_OBJECT (hWnd), "user_data");
      if (od)
        hMenu = od->menu;
    }
  return hMenu;
}


/* GetMenuBarInfo ordinal = 289, argument = 16 temporary */
static DWORD WINAPI GetMenuBarInfo (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* GetMenuCheckMarkDimensions ordinal = 290, argument = 0 */
LONG WINAPI GetMenuCheckMarkDimensions (VOID)
{
  gint width = 16, height = 16;
#if GLIB_CHECK_VERSION(2,2,0)
  GtkSettings *settings;

  settings = gtk_settings_get_default ();
  if (settings)
    {
      gint w, h;

      if (gtk_icon_size_lookup_for_settings (settings,
                                                GTK_ICON_SIZE_MENU, &w, &h))
        {
          width = w;
          height = h;
        }
    }
#endif /* GLIB_CHECK_VERSION(2,2,0) */
  return MAKELONG (width, height);
}


/* GetMenuContextHelpId ordinal = 291, argument = 4 temporary */
static DWORD WINAPI GetMenuContextHelpId (DWORD dwArgv0)
{
  return 0;
}


/* GetMenuDefaultItem ordinal = 292, argument = 12 */
UINT WINAPI GetMenuDefaultItem (HMENU hMenu, UINT uByPos, UINT gmdiFlags)
{
  return (UINT)-1;
}


/* GetMenuInfo ordinal = 293, argument = 8 temporary */
static DWORD WINAPI GetMenuInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetMenuItemCount ordinal = 294, argument = 4 */
INT WINAPI GetMenuItemCount (HMENU hMenu)
{
  gint length = 0;

  if (hMenu)
    {
      GList *glist;

      glist = gtk_container_get_children (GTK_CONTAINER (hMenu));
      length = g_list_length (glist);
      g_list_free (glist);
    }
  return length;
}


/* GetMenuItemID ordinal = 295, argument = 8 */
UINT WINAPI GetMenuItemID (HMENU hMenu, INT nPos)
{
  GtkWidget *menu_item;

  menu_item = w32ldr_menu_get_item (hMenu, nPos, MF_BYPOSITION);
  return menu_item ? GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (menu_item),
                                                    "user_data")) : (UINT)-1;
}


/* GetMenuItemInfoA ordinal = 296, argument = 16 */
BOOL WINAPI GetMenuItemInfoA (HMENU hMenu, UINT uItem, BOOL fByPosition, LPMENUITEMINFOA lpmii)
{
  if (hMenu && lpmii
                && lpmii->cbSize >= sizeof (MENUITEMINFOA) - sizeof (HBITMAP))
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem,
                                fByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
      if (menu_item)
        {
          if (lpmii->fMask & MIIM_STATE)
            {
              lpmii->fState = (GTK_WIDGET_SENSITIVE (menu_item)
                                                    ? MF_ENABLED : MFS_GRAYED);
              if (GTK_IS_IMAGE_MENU_ITEM (menu_item))
                lpmii->fState |= gtk_image_menu_item_get_image
                                            (GTK_IMAGE_MENU_ITEM (menu_item))
                                                ? MFS_CHECKED : MFS_UNCHECKED;
            }
          if (lpmii->fMask & MIIM_ID)
            lpmii->wID = GPOINTER_TO_UINT
                    (g_object_get_data (G_OBJECT (menu_item), "user_data"));
          if (lpmii->fMask & MIIM_SUBMENU)
            lpmii->hSubMenu = gtk_menu_item_get_submenu
                                                (GTK_MENU_ITEM (menu_item));
          if (lpmii->fMask & MIIM_CHECKMARKS)
            lpmii->hbmpChecked = lpmii->hbmpUnchecked = NULL;
          if (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE))
            lpmii->fType = GTK_IS_IMAGE_MENU_ITEM (menu_item)
                                                ? MFT_STRING : MFT_SEPARATOR;
          if (lpmii->fMask & (MIIM_TYPE | MIIM_STRING))
            lpmii->cch = GetMenuStringA (hMenu, uItem,
                                lpmii->dwTypeData, lpmii->cch,
                                fByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
          if (lpmii->fMask & MIIM_DATA)
            lpmii->dwItemData = 0;
          if (lpmii->fMask & MIIM_BITMAP
                                    && lpmii->cbSize >= sizeof (MENUITEMINFOA))
            lpmii->hbmpItem = NULL;
          return TRUE;
        }
    }
  return FALSE;
}


/* GetMenuItemInfoW ordinal = 297, argument = 16 */
BOOL WINAPI GetMenuItemInfoW (HMENU hMenu, UINT uItem, BOOL fByPosition, LPMENUITEMINFOW lpmii)
{
  if (hMenu && lpmii
                && lpmii->cbSize >= sizeof (MENUITEMINFOW) - sizeof (HBITMAP))
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem,
                                fByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
      if (menu_item)
        {
          if (lpmii->fMask & MIIM_STATE)
            {
              lpmii->fState = (GTK_WIDGET_SENSITIVE (menu_item)
                                                    ? MF_ENABLED : MFS_GRAYED);
              if (GTK_IS_IMAGE_MENU_ITEM (menu_item))
                lpmii->fState |= gtk_image_menu_item_get_image
                                            (GTK_IMAGE_MENU_ITEM (menu_item))
                                                ? MFS_CHECKED : MFS_UNCHECKED;
            }
          if (lpmii->fMask & MIIM_ID)
            lpmii->wID = GPOINTER_TO_UINT
                    (g_object_get_data (G_OBJECT (menu_item), "user_data"));
          if (lpmii->fMask & MIIM_SUBMENU)
            lpmii->hSubMenu = gtk_menu_item_get_submenu
                                                (GTK_MENU_ITEM (menu_item));
          if (lpmii->fMask & MIIM_CHECKMARKS)
            lpmii->hbmpChecked = lpmii->hbmpUnchecked = NULL;
          if (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE))
            lpmii->fType = GTK_IS_IMAGE_MENU_ITEM (menu_item)
                                                ? MFT_STRING : MFT_SEPARATOR;
          if (lpmii->fMask & (MIIM_TYPE | MIIM_STRING))
            lpmii->cch = GetMenuStringW (hMenu, uItem,
                                lpmii->dwTypeData, lpmii->cch,
                                fByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
          if (lpmii->fMask & MIIM_DATA)
            lpmii->dwItemData = 0;
          if (lpmii->fMask & MIIM_BITMAP
                                    && lpmii->cbSize >= sizeof (MENUITEMINFOW))
            lpmii->hbmpItem = NULL;
          return TRUE;
        }
    }
  return FALSE;
}


/* GetMenuItemRect ordinal = 298, argument = 16 */
BOOL WINAPI GetMenuItemRect (HWND hWnd, HMENU hMenu, UINT uItem, LPRECT lprcItem)
{
  return FALSE;
}


/* GetMenuState ordinal = 299, argument = 12 */
UINT WINAPI GetMenuState (HMENU hMenu, UINT uItem, UINT uFlags)
{
  UINT uResult = (UINT)-1;

  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item)
        {
          GtkWidget *menu;

          uResult = GTK_WIDGET_SENSITIVE (menu_item) ? MF_ENABLED : MFS_GRAYED;
          uResult |= GTK_IS_IMAGE_MENU_ITEM (menu_item)
            ? gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item))
                                ? MFS_CHECKED : MFS_UNCHECKED : MFT_SEPARATOR;
          menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
          if (menu)
            uResult |= GetMenuItemCount (menu) << 16 | MF_POPUP;
        }
    }
  return uResult;
}


/* GetMenuStringA ordinal = 300, argument = 20 */
INT WINAPI GetMenuStringA (HMENU hMenu, UINT uItem, LPSTR lpszString, INT cchMax, UINT uFlags)
{
  GtkWidget *menu_item;

  menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
  if (menu_item)
    {
      GtkWidget *label;

      label = gtk_bin_get_child (GTK_BIN (menu_item));
      if (label && GTK_IS_LABEL (label))
        {
          const gchar *utf8str;

          utf8str = gtk_label_get_text (GTK_LABEL (label));
          if (utf8str)
            {
              gchar *text, *mb;
              gint i = 0, j = 0;
              INT nResult;

              text = g_malloc ((g_strlen (utf8str) + 1) * sizeof (gchar));
              while (utf8str[i] != '\0')
                if (utf8str[i] == '_')
                  {
                    if (utf8str[++i] == '_')
                      {
                        text[j++] = '_';
                        i++;
                      }
                    else
                      {
                        text[j++] = '&';
                      }
                  }
                else
                  {
                    text[j++] = utf8str[i++];
                  }
              text[j] = '\0';
              mb = w32ldr_utf8_to_mb (text);
              g_free (text);
              if (lpszString && cchMax > 0)
                {
                  g_strncpy (lpszString, mb, cchMax);
                  nResult = g_strlen (lpszString);
                }
              else
                {
                  nResult = g_strlen (mb);
                }
              g_free (mb);
              return nResult;
            }
        }
    }
  return 0;
}


/* GetMenuStringW ordinal = 301, argument = 20 */
INT WINAPI GetMenuStringW (HMENU hMenu, UINT uItem, LPWSTR lpszString, INT cchMax, UINT uFlags)
{
  GtkWidget *menu_item;

  menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
  if (menu_item)
    {
      GtkWidget *label;

      label = gtk_bin_get_child (GTK_BIN (menu_item));
      if (label && GTK_IS_LABEL (label))
        {
          const gchar *utf8str;

          utf8str = gtk_label_get_label (GTK_LABEL (label));
          if (utf8str)
            {
              gchar *text;
              gunichar2 *wc;
              gint i = 0, j = 0;
              INT nResult;

              text = g_malloc ((g_strlen (utf8str) + 1) * sizeof (gchar));
              while (utf8str[i] != '\0')
                if (utf8str[i] == '_')
                  {
                    if (utf8str[++i] == '_')
                      {
                        text[j++] = '_';
                        i++;
                      }
                    else
                      {
                        text[j++] = '&';
                      }
                  }
                else
                  {
                    text[j++] = utf8str[i++];
                  }
              text[j] = '\0';
              wc = g_utf8_to_utf16 (text, -1, NULL, NULL, NULL);
              g_free (text);
              if (lpszString && cchMax > 0)
                {
                  lstrcpynW (lpszString, wc, cchMax);
                  nResult = lstrlenW (lpszString);
                }
              else
                {
                  nResult = lstrlenW (wc);
                }
              g_free (wc);
              return nResult;
            }
        }
    }
  return 0;
}


/* GetMessageA ordinal = 302, argument = 16 */
BOOL WINAPI WINAPI GetMessageA (LPMSG lpMsg, HWND hWnd, UINT wMsgFilterMin, UINT wMsgFilterMax)
{
  return GetMessageW (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax);
}


/* GetMessageExtraInfo ordinal = 303, argument = 0 temporary */
static DWORD WINAPI GetMessageExtraInfo (VOID)
{
  return 0;
}


/* GetMessagePos ordinal = 304, argument = 0 temporary */
static DWORD WINAPI GetMessagePos (VOID)
{
  return 0;
}


/* GetMessageTime ordinal = 305, argument = 0 temporary */
static DWORD WINAPI GetMessageTime (VOID)
{
  return 0;
}


/* GetMessageW ordinal = 306, argument = 16 */
BOOL WINAPI GetMessageW (LPMSG lpMsg, HWND hWnd, UINT wMsgFilterMin, UINT wMsgFilterMax)
{
  while (!PeekMessageW (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, PM_REMOVE))
    while (gtk_events_pending ())
      gtk_main_iteration ();
  return !lpMsg || lpMsg->message != WM_QUIT;
}


/* GetMonitorInfoA ordinal = 307, argument = 8 temporary */
static DWORD WINAPI GetMonitorInfoA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetMonitorInfoW ordinal = 308, argument = 8 temporary */
static DWORD WINAPI GetMonitorInfoW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetMouseMovePointsEx ordinal = 309, argument = 20 temporary */
static DWORD WINAPI GetMouseMovePointsEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetNextDlgGroupItem ordinal = 310, argument = 12 temporary */
static DWORD WINAPI GetNextDlgGroupItem (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetNextDlgTabItem ordinal = 311, argument = 12 temporary */
static DWORD WINAPI GetNextDlgTabItem (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetOpenClipboardWindow ordinal = 312, argument = 0 temporary */
static DWORD WINAPI GetOpenClipboardWindow (VOID)
{
  return 0;
}


/* GetParent ordinal = 313, argument = 4 */
HWND WINAPI GetParent (HWND hWnd)
{
  W32LdrWindowData *wd;

  if (!hWnd)
    return NULL;
  wd = g_object_get_data (G_OBJECT (hWnd), "user_data");
  if (!wd)
    return NULL;
  return wd->hWndParent;
}


/* GetPriorityClipboardFormat ordinal = 314, argument = 8 temporary */
static DWORD WINAPI GetPriorityClipboardFormat (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetProcessDefaultLayout ordinal = 315, argument = 4 temporary */
static DWORD WINAPI GetProcessDefaultLayout (DWORD dwArgv0)
{
  return 0;
}


/* GetProcessWindowStation ordinal = 316, argument = 0 temporary */
static DWORD WINAPI GetProcessWindowStation (VOID)
{
  return 0;
}


/* GetProgmanWindow ordinal = 317, argument = 0 temporary */
static DWORD WINAPI GetProgmanWindow (VOID)
{
  return 0;
}


/* GetPropA ordinal = 318, argument = 8 temporary */
static DWORD WINAPI GetPropA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetPropW ordinal = 319, argument = 8 temporary */
static DWORD WINAPI GetPropW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetQueueStatus ordinal = 320, argument = 4 temporary */
static DWORD WINAPI GetQueueStatus (DWORD dwArgv0)
{
  return 0;
}


/* GetScrollBarInfo ordinal = 321, argument = 12 temporary */
static DWORD WINAPI GetScrollBarInfo (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetScrollInfo ordinal = 322, argument = 12 temporary */
static DWORD WINAPI GetScrollInfo (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetScrollPos ordinal = 323, argument = 8 */
INT WINAPI GetScrollPos (HWND hWnd, INT nBar)
{
  return hWnd && nBar == SB_CTL ? IsWindowUnicode (hWnd)
                                ? SendMessageW (hWnd, SBM_GETPOS, 0, 0)
                                : SendMessageA (hWnd, SBM_GETPOS, 0, 0) : 0;
}


/* GetScrollRange ordinal = 324, argument = 16 */
BOOL WINAPI GetScrollRange (HWND hWnd, INT nBar, LPINT lpMinPos, LPINT lpMaxPos)
{
  if (!hWnd || nBar != SB_CTL)
    return FALSE;
  if (IsWindowUnicode (hWnd))
    SendMessageW (hWnd, SBM_GETRANGE,
                    GPOINTER_TO_INT (lpMinPos), GPOINTER_TO_INT (lpMaxPos));
  else
    SendMessageA (hWnd, SBM_GETRANGE,
                    GPOINTER_TO_INT (lpMinPos), GPOINTER_TO_INT (lpMaxPos));
  return TRUE;
}


/* GetShellWindow ordinal = 325, argument = 0 temporary */
static DWORD WINAPI GetShellWindow (VOID)
{
  return 0;
}


/* GetSubMenu ordinal = 326, argument = 8 */
HMENU WINAPI GetSubMenu (HMENU hMenu, INT nPos)
{
  GtkWidget *menu_item;

  menu_item = w32ldr_menu_get_item (hMenu, nPos, MF_BYPOSITION);
  return menu_item ? gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item))
                   : NULL;
}


/* GetSysColor ordinal = 327, argument = 4 */
DWORD WINAPI GetSysColor (int nIndex)
{
  switch (nIndex)
    {
      case COLOR_SCROLLBAR:               return 0xccdddd;
      case COLOR_BACKGROUND:              return 0xaa6633;
      case COLOR_ACTIVECAPTION:           return 0x662200;
      case COLOR_INACTIVECAPTION:         return 0x888888;
      case COLOR_MENU:                    return 0xccdddd;
      case COLOR_WINDOW:                  return 0xffffff;
      case COLOR_WINDOWFRAME:             return 0x000000;
      case COLOR_MENUTEXT:                return 0x000000;
      case COLOR_WINDOWTEXT:              return 0x000000;
      case COLOR_CAPTIONTEXT:             return 0xffffff;
      case COLOR_ACTIVEBORDER:            return 0xccdddd;
      case COLOR_INACTIVEBORDER:          return 0xccdddd;
      case COLOR_APPWORKSPACE:            return 0x888888;
      case COLOR_HIGHLIGHT:               return 0x662200;
      case COLOR_HIGHLIGHTTEXT:           return 0xffffff;
      case COLOR_BTNFACE:                 return 0xccdddd;
      case COLOR_BTNSHADOW:               return 0x888888;
      case COLOR_GRAYTEXT:                return 0x888888;
      case COLOR_BTNTEXT:                 return 0x000000;
      case COLOR_INACTIVECAPTIONTEXT:     return 0xccdddd;
      case COLOR_BTNHIGHLIGHT:            return 0xffffff;
      case COLOR_3DDKSHADOW:              return 0x444444;
      case COLOR_3DLIGHT:                 return 0xccdddd;
      case COLOR_INFOTEXT:                return 0x000000;
      case COLOR_INFOBK:                  return 0xeeffff;
      case COLOR_ALTERNATEBTNFACE:        return 0xbbbbbb;
      case COLOR_HOTLIGHT:                return 0xff0000;
      case COLOR_GRADIENTACTIVECAPTION:   return 0xffccaa;
      case COLOR_GRADIENTINACTIVECAPTION: return 0xcccccc;
      case COLOR_MENUHILIGHT:             return 0x000000;
      case COLOR_MENUBAR:                 return 0x000000;
    }
  return 0;
}


/* GetSysColorBrush ordinal = 328, argument = 4 temporary */
static DWORD WINAPI GetSysColorBrush (DWORD dwArgv0)
{
  return 0;
}


/* GetSystemMenu ordinal = 329, argument = 8 */
HMENU WINAPI GetSystemMenu (HWND hWnd, BOOL bRevert)
{
  return NULL;
}


/* GetSystemMetrics ordinal = 330, argument = 4 */
INT WINAPI GetSystemMetrics (INT nIndex)
{
  switch (nIndex)
    {
      case SM_CXCURSOR: return 32;
      case SM_CYCURSOR: return 32;
      case SM_CXICON:   return 32;
      case SM_CYICON:   return 32;
    }
  return 0;
}


/* GetTabbedTextExtentA ordinal = 331, argument = 20 temporary */
static DWORD WINAPI GetTabbedTextExtentA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetTabbedTextExtentW ordinal = 332, argument = 20 temporary */
static DWORD WINAPI GetTabbedTextExtentW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetTaskmanWindow ordinal = 333, argument = 0 temporary */
static DWORD WINAPI GetTaskmanWindow (VOID)
{
  return 0;
}


/* GetThreadDesktop ordinal = 334, argument = 4 temporary */
static DWORD WINAPI GetThreadDesktop (DWORD dwArgv0)
{
  return 0;
}


/* GetTitleBarInfo ordinal = 335, argument = 8 temporary */
static DWORD WINAPI GetTitleBarInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetTopWindow ordinal = 336, argument = 4 temporary */
static DWORD WINAPI GetTopWindow (DWORD dwArgv0)
{
  return 0;
}


/* GetUpdateRect ordinal = 337, argument = 12 temporary */
static DWORD WINAPI GetUpdateRect (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetUpdateRgn ordinal = 338, argument = 12 temporary */
static DWORD WINAPI GetUpdateRgn (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetUserObjectInformationA ordinal = 339, argument = 20 temporary */
static DWORD WINAPI GetUserObjectInformationA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetUserObjectInformationW ordinal = 340, argument = 20 temporary */
static DWORD WINAPI GetUserObjectInformationW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetUserObjectSecurity ordinal = 341, argument = 20 temporary */
static DWORD WINAPI GetUserObjectSecurity (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* GetWinStationInfo ordinal = 342, argument = 0 temporary */
static DWORD WINAPI GetWinStationInfo (VOID)
{
  return 0;
}


/* GetWindow ordinal = 343, argument = 8 */
HWND WINAPI GetWindow (HWND hWnd, UINT uCmd)
{
  W32LdrWindowData *wd;

  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  if (wd)
    switch (uCmd)
      {
        case GW_HWNDFIRST:
          wd = wd->hWndParent ? g_object_get_data (G_OBJECT (wd->hWndParent),
                                                        "user_data") : NULL;
          if (wd && wd->child)
            return wd->child[0];
          break;
        case GW_HWNDLAST:
          wd = wd->hWndParent ? g_object_get_data (G_OBJECT (wd->hWndParent),
                                                        "user_data") : NULL;
          if (wd && wd->child)
            {
              gint i;

              for (i = 0; wd->child[i]; i++);
              if (i > 0)
                return wd->child[i - 1];
            }
          break;
        case GW_HWNDNEXT:
          wd = wd->hWndParent ? g_object_get_data (G_OBJECT (wd->hWndParent),
                                                        "user_data") : NULL;
          if (wd && wd->child)
            {
              gint i;

              for (i = 0; wd->child[i]; i++)
                if (wd->child[i] == hWnd)
                  return wd->child[i + 1];
            }
          break;
        case GW_HWNDPREV:
          wd = wd->hWndParent ? g_object_get_data (G_OBJECT (wd->hWndParent),
                                                        "user_data") : NULL;
          if (wd && wd->child && wd->child[0])
            {
              gint i;

              for (i = 1; wd->child[i]; i++)
                if (wd->child[i] == hWnd)
                  return wd->child[i - 1];
            }
          break;
        case GW_OWNER: return wd->hWndParent;
        case GW_CHILD:
          if (wd->child)
            return wd->child[0];
      }
  return NULL;
}


/* GetWindowContextHelpId ordinal = 344, argument = 4 temporary */
static DWORD WINAPI GetWindowContextHelpId (DWORD dwArgv0)
{
  return 0;
}


/* GetWindowDC ordinal = 345, argument = 4 temporary */
static DWORD WINAPI GetWindowDC (DWORD dwArgv0)
{
  return 0;
}


/* GetWindowInfo ordinal = 346, argument = 8 temporary */
static DWORD WINAPI GetWindowInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetWindowLongA ordinal = 347, argument = 8 */
LONG WINAPI GetWindowLongA (HWND hWnd, INT nIndex)
{
  return GetWindowLongW (hWnd, nIndex);
}


/* GetWindowLongW ordinal = 348, argument = 8 */
LONG WINAPI GetWindowLongW (HWND hWnd, INT nIndex)
{
  W32LdrWindowData *wd;

  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  if (wd)
    switch (nIndex)
      {
        case GWL_USERDATA:  return wd->dwUserData;
        case GWL_EXSTYLE:   return wd->dwExStyle;
        case GWL_STYLE:     return wd->dwStyle;
        case GWL_ID:        return wd->wID;
        case GWL_HWNDPARENT:return GPOINTER_TO_INT (wd->hWndParent);
        case GWL_HINSTANCE: return GPOINTER_TO_INT (wd->hInstance);
        case GWL_WNDPROC:   return GPOINTER_TO_INT (wd->lpfnWndProc);
        case DWL_MSGRESULT: return wd->lResult;
        case DWL_DLGPROC:   return GPOINTER_TO_INT (wd->lpDialogFunc);
        case DWL_USER:      return wd->dwUser;
      }
  return 0;
}


/* GetWindowModuleFileName ordinal = 349, argument = 12 temporary */
/* DWORD WINAPI GetWindowModuleFileName (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2) */


/* GetWindowModuleFileNameA ordinal = 350, argument = 12 temporary */
static DWORD WINAPI GetWindowModuleFileNameA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetWindowModuleFileNameW ordinal = 351, argument = 12 temporary */
static DWORD WINAPI GetWindowModuleFileNameW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* GetWindowPlacement ordinal = 352, argument = 8 temporary */
static DWORD WINAPI GetWindowPlacement (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetWindowRect ordinal = 353, argument = 8 */
BOOL WINAPI GetWindowRect (HWND hWnd, LPRECT lpRect)
{
  if (!lpRect)
    return FALSE;
  if (!hWnd)
    {
      gint x, y, width, height, depth;

      gdk_window_get_geometry (gdk_get_default_root_window (),
                                            &x, &y, &width, &height, &depth);
      lpRect->left = x;
      lpRect->top = y;
      lpRect->right = x + width;
      lpRect->bottom = y + height;
    }
  else if (GTK_IS_WINDOW (hWnd))
    {
      gint x, y, width, height;

      gtk_window_get_position (GTK_WINDOW (hWnd), &x, &y);
      gtk_window_get_size (GTK_WINDOW (hWnd), &width, &height);
      lpRect->left = x;
      lpRect->top = y;
      lpRect->right = x + width;
      lpRect->bottom = y + height;
    }
  else
    {
      GdkWindow *window;
      GtkRequisition requisition;
      W32LdrWindowData *wd;

      lpRect->left = hWnd->allocation.x;
      lpRect->top = hWnd->allocation.y;
      wd = g_object_get_data (G_OBJECT (hWnd), "user_data");
      if (wd->hWndParent && GTK_IS_WINDOW (wd->hWndParent))
        {
          W32LdrOverlapData *od;

          od = g_object_get_data (G_OBJECT (wd->hWndParent), "user_data");
          if (od->fixed)
            {
              GList *glist;

              for (glist = g_list_first (GTK_FIXED (od->fixed)->children);
                                            glist; glist = g_list_next (glist))
                {
                  GtkFixedChild *child;

                  child = glist->data;
                  if (hWnd == child->widget)
                    {
                      lpRect->left = od->fixed->allocation.x + child->x;
                      lpRect->top = od->fixed->allocation.y + child->y;
                      break;
                    }
                }
            }
        }
      for (window = hWnd->window; window;
                                    window = gdk_window_get_parent (window))
        {
          gint x, y;

          gdk_window_get_position (window, &x, &y);
          lpRect->left += x;
          lpRect->top += y;
        }
      gtk_widget_size_request (hWnd, &requisition);
      lpRect->right = lpRect->left
                        + MAX (hWnd->allocation.width, requisition.width);
      lpRect->bottom = lpRect->top
                        + MAX (hWnd->allocation.height, requisition.height);
    }
  return TRUE;
}


/* GetWindowRgn ordinal = 354, argument = 8 temporary */
static DWORD WINAPI GetWindowRgn (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetWindowTextA ordinal = 355, argument = 12 */
INT WINAPI GetWindowTextA (HWND hWnd, LPSTR lpString, INT nMaxCount)
{
  return SendMessageA (hWnd, WM_GETTEXT,
                                        nMaxCount, GPOINTER_TO_INT (lpString));
}


/* GetWindowTextLengthA ordinal = 356, argument = 4 */
INT WINAPI GetWindowTextLengthA (HWND hWnd)
{
  return SendMessageA (hWnd, WM_GETTEXTLENGTH, 0, 0);
}


/* GetWindowTextLengthW ordinal = 357, argument = 4 */
INT WINAPI GetWindowTextLengthW (HWND hWnd)
{
  return SendMessageW (hWnd, WM_GETTEXTLENGTH, 0, 0);
}


/* GetWindowTextW ordinal = 358, argument = 12 */
INT WINAPI GetWindowTextW (HWND hWnd, LPWSTR lpString, INT nMaxCount)
{
  return SendMessageW (hWnd, WM_GETTEXT,
                                        nMaxCount, GPOINTER_TO_INT (lpString));
}


/* GetWindowThreadProcessId ordinal = 359, argument = 8 temporary */
static DWORD WINAPI GetWindowThreadProcessId (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GetWindowWord ordinal = 360, argument = 8 temporary */
static DWORD WINAPI GetWindowWord (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* GrayStringA ordinal = 361, argument = 36 temporary */
static DWORD WINAPI GrayStringA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8)
{
  return 0;
}


/* GrayStringW ordinal = 362, argument = 36 temporary */
static DWORD WINAPI GrayStringW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8)
{
  return 0;
}


/* HideCaret ordinal = 363, argument = 4 temporary */
static DWORD WINAPI HideCaret (DWORD dwArgv0)
{
  return 0;
}


/* HiliteMenuItem ordinal = 364, argument = 16 */
BOOL WINAPI HiliteMenuItem (HWND hWnd, HMENU hMenu, UINT uItem, UINT uFlags)
{
  return FALSE;
}


/* IMPGetIMEA ordinal = 365, argument = 8 temporary */
static DWORD WINAPI IMPGetIMEA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* IMPGetIMEW ordinal = 366, argument = 8 temporary */
static DWORD WINAPI IMPGetIMEW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* IMPQueryIMEA ordinal = 367, argument = 4 temporary */
static DWORD WINAPI IMPQueryIMEA (DWORD dwArgv0)
{
  return 0;
}


/* IMPQueryIMEW ordinal = 368, argument = 4 temporary */
static DWORD WINAPI IMPQueryIMEW (DWORD dwArgv0)
{
  return 0;
}


/* IMPSetIMEA ordinal = 369, argument = 8 temporary */
static DWORD WINAPI IMPSetIMEA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* IMPSetIMEW ordinal = 370, argument = 8 temporary */
static DWORD WINAPI IMPSetIMEW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ImpersonateDdeClientWindow ordinal = 371, argument = 8 temporary */
static DWORD WINAPI ImpersonateDdeClientWindow (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* InSendMessage ordinal = 372, argument = 0 temporary */
static DWORD WINAPI InSendMessage (VOID)
{
  return 0;
}


/* InSendMessageEx ordinal = 373, argument = 4 temporary */
static DWORD WINAPI InSendMessageEx (DWORD dwArgv0)
{
  return 0;
}


/* InflateRect ordinal = 374, argument = 12 */
BOOL WINAPI InflateRect (LPRECT lprc, INT dx, INT dy)
{
  if (!lprc)
    return FALSE;
  lprc->left -= dx;
  lprc->top -= dy;
  lprc->right += dx;
  lprc->bottom += dy;
  return TRUE;
}


/* InitializeLpkHooks ordinal = 375, argument = 0 temporary */
static DWORD WINAPI InitializeLpkHooks (VOID)
{
  return 0;
}


/* InitializeWin32EntryTable ordinal = 376, argument = 0 temporary */
static DWORD WINAPI InitializeWin32EntryTable (VOID)
{
  return 0;
}


/* InsertMenuA ordinal = 377, argument = 20 */
BOOL WINAPI InsertMenuA (HMENU hMenu, UINT uItem, UINT uFlags, UINT_PTR uNewItem, LPCSTR lpszNewItem)
{
  gchar *utf8str;
  gunichar2 *utf16str;
  BOOL fResult;

  if (uFlags & (MFT_BITMAP | MFT_OWNERDRAW))
    return InsertMenuW (hMenu, uItem, uFlags, uNewItem, (LPCWSTR)lpszNewItem);

  utf8str = w32ldr_utf8_from_mb (lpszNewItem);
  utf16str = utf8str ? g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL)
                     : g_malloc0 (sizeof (gunichar2));
  g_free (utf8str);
  fResult = InsertMenuW (hMenu, uItem, uFlags, uNewItem, utf16str);
  g_free (utf16str);
  return fResult;
}


/* InsertMenuItemA ordinal = 378, argument = 16 */
BOOL WINAPI InsertMenuItemA (HMENU hMenu, UINT uItem, BOOL fByPosition, LPMENUITEMINFOA lpmii)
{
  return lpmii && lpmii->cbSize >= sizeof (MENUITEMINFOA)- sizeof (HBITMAP)
        && InsertMenuA (hMenu, uItem,
                (fByPosition ? MF_BYPOSITION : MF_BYCOMMAND)
                | (lpmii->fMask & MIIM_STATE ? lpmii->fState : 0)
                | (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE) ? lpmii->fType : 0),
        lpmii->wID,
        lpmii->fMask & MIIM_BITMAP && lpmii->cbSize >= sizeof (MENUITEMINFOA)
                                ? (LPCSTR)lpmii->hbmpItem : lpmii->dwTypeData);
}


/* InsertMenuItemW ordinal = 379, argument = 16 */
BOOL WINAPI InsertMenuItemW (HMENU hMenu, UINT uItem, BOOL fByPosition, LPMENUITEMINFOW lpmii)
{
  return lpmii && lpmii->cbSize >= sizeof (MENUITEMINFOW)- sizeof (HBITMAP)
        && InsertMenuW (hMenu, uItem,
                (fByPosition ? MF_BYPOSITION : MF_BYCOMMAND)
                | (lpmii->fMask & MIIM_STATE ? lpmii->fState : 0)
                | (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE) ? lpmii->fType : 0),
        lpmii->wID,
        lpmii->fMask & MIIM_BITMAP && lpmii->cbSize >= sizeof (MENUITEMINFOW)
                            ? (LPCWSTR)lpmii->hbmpItem : lpmii->dwTypeData);
}


/* InsertMenuW ordinal = 380, argument = 20 */
BOOL WINAPI InsertMenuW (HMENU hMenu, UINT uItem, UINT uFlags, UINT_PTR uNewItem, LPCWSTR lpszNewItem)
{
  GtkWidget *menu_item = NULL;

  if (!hMenu)
    return FALSE;
  if (uFlags & MFT_SEPARATOR)
    {
      menu_item = gtk_separator_menu_item_new ();
    }
  else
    {
      menu_item = gtk_image_menu_item_new_with_label ("");
      if (!(uFlags & MFT_OWNERDRAW) && lpszNewItem)
        {
          if (uFlags & MFT_BITMAP)
            {
              gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                        gtk_image_new_from_pixbuf ((GdkPixbuf *)lpszNewItem));
              g_object_unref (G_OBJECT (lpszNewItem));
            }
          else
            {
              gchar *utf8str;

              utf8str = g_utf16_to_utf8 (lpszNewItem, -1, NULL, NULL, NULL);
              if (utf8str)
                {
                  gint i;
                  GList *glist;
                  GtkWidget *label = NULL;

                  glist = gtk_container_get_children
                                                (GTK_CONTAINER (menu_item));
                  for (i = 0; i < g_list_length (glist); i++)
                    if (GTK_IS_LABEL (g_list_nth_data (glist, i)))
                      {
                        label = g_list_nth_data (glist, i);
                        break;
                      }
                  g_list_free (glist);
                  if (label)
                    {
                      gchar *title;
                      gint j = 0;

                      title = g_malloc ((g_strlen (utf8str) * 2 + 1)
                                                            * sizeof (gchar));
                      for (i = 0; utf8str[i] != '\0'; i++)
                        switch (utf8str[i])
                          {
                            case '_': title[j++] = '_';
                            case '&': title[j++] = '_'; break;
                            default:  title[j++] = utf8str[i];
                          }
                      g_free (utf8str);
                      for (i = 0; i < j; i++)
                        if (title[i] == '\t')
                          break;
                      title[j] = '\0';
                      gtk_label_set_text_with_mnemonic (GTK_LABEL (label),
                                                                        title);
                      g_free (title);
                    }
                }
            }
        }
    }
  if (!menu_item)
    return FALSE;
  gtk_widget_set_sensitive (menu_item, !(uFlags & MFS_GRAYED));
  if (uFlags & MFS_CHECKED)
    {
      GdkPixbuf *pixbuf;

      pixbuf = uFlags & MFT_RADIOCHECK ? w32ldr_menu_get_pixbuf_radio ()
                                       : w32ldr_menu_get_pixbuf_check ();
      if (pixbuf)
        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                                        gtk_image_new_from_pixbuf (pixbuf));
    }
  if (uFlags & MF_POPUP)
    {
      gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
                                                GUINT_TO_POINTER (uNewItem));
      g_object_set_data (G_OBJECT (menu_item), "user_data",
                                                GUINT_TO_POINTER (G_MAXUINT));
    }
  else
    {
      g_object_set_data (G_OBJECT (menu_item), "user_data",
                                                GUINT_TO_POINTER (uNewItem));
    }
  gtk_widget_show (menu_item);
  gtk_menu_shell_insert (GTK_MENU_SHELL (hMenu), menu_item,
                                            uFlags & MF_BYPOSITION ? uItem
                                : w32ldr_menu_get_item_pos (hMenu, uItem));
  return TRUE;
}


/* InternalGetWindowText ordinal = 381, argument = 12 temporary */
static DWORD WINAPI InternalGetWindowText (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* IntersectRect ordinal = 382, argument = 12 */
BOOL WINAPI IntersectRect (LPRECT lprcDst, CONST LPRECT lprcSrc1, CONST LPRECT lprcSrc2)
{
  if (!lprcDst || !lprcSrc1 || !lprcSrc2
    || IsRectEmpty (lprcSrc1) || IsRectEmpty (lprcSrc2)
    || lprcSrc1->left >= lprcSrc2->right || lprcSrc1->right <=lprcSrc2->left
    || lprcSrc1->top >= lprcSrc2->bottom || lprcSrc1->bottom <= lprcSrc2->top)
    {
      SetRectEmpty (lprcDst);
      return FALSE;
    }
  lprcDst->left = MAX (lprcSrc1->left, lprcSrc2->left);
  lprcDst->top = MAX (lprcSrc1->top, lprcSrc2->top);
  lprcDst->right = MIN (lprcSrc1->right, lprcSrc2->right);
  lprcDst->bottom = MIN (lprcSrc1->bottom, lprcSrc2->bottom);
  return TRUE;
}


/* InvalidateRect ordinal = 383, argument = 12 */
BOOL WINAPI InvalidateRect (HWND hWnd, CONST RECT *lpRect, BOOL bErase)
{
  if (!hWnd)
    return FALSE;
  if (lpRect)
    gtk_widget_queue_draw_area (hWnd, lpRect->left, lpRect->top,
                lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
  else
    gtk_widget_queue_draw (hWnd);
  return TRUE;
}


/* InvalidateRgn ordinal = 384, argument = 12 temporary */
static DWORD WINAPI InvalidateRgn (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* InvertRect ordinal = 385, argument = 8 temporary */
static DWORD WINAPI InvertRect (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* IsCharAlphaA ordinal = 386, argument = 4 temporary */
static DWORD WINAPI IsCharAlphaA (DWORD dwArgv0)
{
  return 0;
}


/* IsCharAlphaNumericA ordinal = 387, argument = 4 temporary */
static DWORD WINAPI IsCharAlphaNumericA (DWORD dwArgv0)
{
  return 0;
}


/* IsCharAlphaNumericW ordinal = 388, argument = 4 temporary */
static DWORD WINAPI IsCharAlphaNumericW (DWORD dwArgv0)
{
  return 0;
}


/* IsCharAlphaW ordinal = 389, argument = 4 temporary */
static DWORD WINAPI IsCharAlphaW (DWORD dwArgv0)
{
  return 0;
}


/* IsCharLowerA ordinal = 390, argument = 4 temporary */
static DWORD WINAPI IsCharLowerA (DWORD dwArgv0)
{
  return 0;
}


/* IsCharLowerW ordinal = 391, argument = 4 temporary */
static DWORD WINAPI IsCharLowerW (DWORD dwArgv0)
{
  return 0;
}


/* IsCharUpperA ordinal = 392, argument = 4 temporary */
static DWORD WINAPI IsCharUpperA (DWORD dwArgv0)
{
  return 0;
}


/* IsCharUpperW ordinal = 393, argument = 4 temporary */
static DWORD WINAPI IsCharUpperW (DWORD dwArgv0)
{
  return 0;
}


/* IsChild ordinal = 394, argument = 8 */
BOOL WINAPI IsChild (HWND hWndParent, HWND hWndChild)
{
  HWND hWnd;

  hWnd = hWndChild;
  while ((hWnd = GetParent (hWnd)))
    if (hWndParent == hWnd)
      return TRUE;
  return FALSE;
}


/* IsClipboardFormatAvailable ordinal = 395, argument = 4 temporary */
static DWORD WINAPI IsClipboardFormatAvailable (DWORD dwArgv0)
{
  return 0;
}


/* IsDialogMessage ordinal = 396, argument = 8 temporary */
/* DWORD WINAPI IsDialogMessage (DWORD dwArgv0, DWORD dwArgv1) */


/* IsDialogMessageA ordinal = 397, argument = 8 temporary */
static DWORD WINAPI IsDialogMessageA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* IsDialogMessageW ordinal = 398, argument = 8 temporary */
static DWORD WINAPI IsDialogMessageW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* IsDlgButtonChecked ordinal = 399, argument = 8 */
UINT WINAPI IsDlgButtonChecked (HWND hDlg, INT nIDButton)
{
  return SendDlgItemMessage (hDlg, nIDButton, BM_GETCHECK, 0, 0);
}


/* IsHungAppWindow ordinal = 400, argument = 4 temporary */
static DWORD WINAPI IsHungAppWindow (DWORD dwArgv0)
{
  return 0;
}


/* IsIconic ordinal = 401, argument = 4 temporary */
static DWORD WINAPI IsIconic (DWORD dwArgv0)
{
  return 0;
}


/* IsMenu ordinal = 402, argument = 4 */
BOOL WINAPI IsMenu (HMENU hMenu)
{
  return hMenu && GTK_IS_MENU_SHELL (hMenu);
}


/* IsRectEmpty ordinal = 403, argument = 4 */
BOOL WINAPI IsRectEmpty (CONST LPRECT lprc)
{
  return lprc->left < lprc->right && lprc->top < lprc->bottom;
}


/* IsWindow ordinal = 404, argument = 4 temporary */
static DWORD WINAPI IsWindow (DWORD dwArgv0)
{
  return 0;
}


/* IsWindowEnabled ordinal = 405, argument = 4 */
BOOL WINAPI IsWindowEnabled (HWND hWnd)
{
  return hWnd && GTK_WIDGET_IS_SENSITIVE (hWnd);
}


/* IsWindowUnicode ordinal = 406, argument = 4 */
BOOL WINAPI IsWindowUnicode (HWND hWnd)
{
  W32LdrWindowData *wd;

  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  return wd && wd->widechar;
}


/* IsWindowVisible ordinal = 407, argument = 4 temporary */
static DWORD WINAPI IsWindowVisible (DWORD dwArgv0)
{
  return 0;
}


/* IsZoomed ordinal = 408, argument = 4 temporary */
static DWORD WINAPI IsZoomed (DWORD dwArgv0)
{
  return 0;
}


/* KillSystemTimer ordinal = 409, argument = 0 temporary */
static DWORD WINAPI KillSystemTimer (VOID)
{
  return 0;
}


/* KillTimer ordinal = 410, argument = 8 */
BOOL WINAPI KillTimer (HWND hWnd, UINT uIDEvent)
{
  gboolean result = FALSE;
  GList *glist;

  for (glist = g_list_first (glist_timer); glist;
                                            glist = g_list_next (glist_timer))
    {
      W32LdrTimer *timer;

      timer = glist->data;
      if ((hWnd && timer->hWnd == hWnd && timer->uIDEvent == uIDEvent)
                                        || (!hWnd && timer->id == uIDEvent))
        {
          glist_timer = g_list_remove (glist_timer, timer);
          result = g_source_remove (timer->id);
          g_free (timer);
          break;
        }
    }
  return result;
}


/* LoadAcceleratorsA ordinal = 411, argument = 8 */
HACCEL WINAPI LoadAcceleratorsA (HINSTANCE hInstance, LPCSTR lpTableName)
{
  return NULL;
}


/* LoadAcceleratorsW ordinal = 412, argument = 8 */
HACCEL WINAPI LoadAcceleratorsW (HINSTANCE hInstance, LPCWSTR lpTableName)
{
  return NULL;
}


/* LoadBitmapA ordinal = 413, argument = 8 */
HBITMAP WINAPI LoadBitmapA (HINSTANCE hInstance, LPCSTR lpBitmapName)
{
  return (HBITMAP)LoadImageA (hInstance, lpBitmapName, IMAGE_BITMAP, 0, 0, 0);
}


/* LoadBitmapW ordinal = 414, argument = 8 */
HBITMAP WINAPI LoadBitmapW (HINSTANCE hInstance, LPCWSTR lpBitmapName)
{
  return (HBITMAP)LoadImageW (hInstance, lpBitmapName, IMAGE_BITMAP, 0, 0, 0);
}


/* LoadCursorA ordinal = 415, argument = 8 */
HCURSOR WINAPI LoadCursorA (HINSTANCE hInstance, LPCSTR lpCursorName)
{
  return (HCURSOR)LoadImageA (hInstance, lpCursorName, IMAGE_CURSOR,
                                            0, 0, LR_DEFAULTSIZE | LR_SHARED);
}


/* LoadCursorFromFileA ordinal = 416, argument = 4 temporary */
static DWORD WINAPI LoadCursorFromFileA (DWORD dwArgv0)
{
  return 0;
}


/* LoadCursorFromFileW ordinal = 417, argument = 4 temporary */
static DWORD WINAPI LoadCursorFromFileW (DWORD dwArgv0)
{
  return 0;
}


/* LoadCursorW ordinal = 418, argument = 8 */
HCURSOR WINAPI LoadCursorW (HINSTANCE hInstance, LPCWSTR lpCursorName)
{
  return (HCURSOR)LoadImageW (hInstance, lpCursorName, IMAGE_CURSOR,
                                            0, 0, LR_DEFAULTSIZE | LR_SHARED);
}


/* LoadIconA ordinal = 419, argument = 8 */
HICON WINAPI LoadIconA (HINSTANCE hInstance, LPCSTR lpIconName)
{
  return (HICON)LoadImageA (hInstance, lpIconName, IMAGE_ICON,
                                            0, 0, LR_DEFAULTSIZE | LR_SHARED);
}


/* LoadIconW ordinal = 420, argument = 8 */
HICON WINAPI LoadIconW (HINSTANCE hInstance, LPCWSTR lpIconName)
{
  return (HICON)LoadImageW (hInstance, lpIconName, IMAGE_ICON,
                                            0, 0, LR_DEFAULTSIZE | LR_SHARED);
}


/* LoadImageA ordinal = 421, argument = 24 */
HANDLE WINAPI LoadImageA (HINSTANCE hInstance, LPCSTR lpszName, UINT uType, INT cxDesired, INT cyDesired, UINT fuLoad)
{
  HANDLE hImage = NULL;

  if (IS_INTRESOURCE (lpszName))
    {
      hImage = LoadImageW (hInstance, (LPCWSTR)lpszName, uType,
                                                cxDesired, cyDesired, fuLoad);
    }
  else
    {
      gchar *utf8str;

      utf8str = w32ldr_utf8_from_mb (lpszName);
      if (utf8str)
        {
          gunichar2 *utf16str;

          utf16str = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
          g_free (utf8str);
          if (utf16str)
            {
              hImage = LoadImageW (hInstance, utf16str, uType,
                                                cxDesired, cyDesired, fuLoad);
              g_free (utf16str);
            }
        }
    }
  return hImage;
}


#include "pshpack1.h"
typedef struct _ICONHEADER
{
  WORD idReserved;
  WORD idType;
  WORD idCount;
} ICONHEADER, *PICONHEADER, *LPICONHEADER;
typedef struct _GRPICONDIRENTRY
{
  BYTE bWidth;
  BYTE bHeight;
  BYTE bColorCount;
  BYTE bReserved;
  WORD wPlanes;
  WORD wBitCount;
  DWORD dwBytesInRes;
  WORD nID;
} GRPICONDIRENTRY, *PGRPICONDIRENTRY, *LPGRPICONDIRENTRY;
#include "poppack.h"


static GHashTable *ghash_image = NULL;


/* LoadImageW ordinal = 422, argument = 24 */
HANDLE WINAPI LoadImageW (HINSTANCE hInstance, LPCWSTR lpszName, UINT uType, INT cxDesired, INT cyDesired, UINT fuLoad)
{
  gchar *key = NULL;
  gint width, height;
  GdkPixbuf *pixbuf = NULL;
  HANDLE hImage = NULL;

  width = cxDesired;
  height = cyDesired;
  if (fuLoad & LR_DEFAULTSIZE)
    switch (uType)
      {
        case IMAGE_CURSOR:
          width = GetSystemMetrics (SM_CXCURSOR);
          height = GetSystemMetrics (SM_CYCURSOR);
          break;
        case IMAGE_ICON:
          width = GetSystemMetrics (SM_CXICON);
          height = GetSystemMetrics (SM_CYICON);
      }
  if (fuLoad & LR_LOADFROMFILE)
    {
      gchar *file;

      file = w32ldr_filename_from_wc (lpszName);
      if (file)
        {
#if GLIB_CHECK_VERSION(2,4,0)
          if (width > 0 && height > 0)
            pixbuf = gdk_pixbuf_new_from_file_at_size (file, width, height,
                                                                        NULL);
          else
#endif /* GLIB_CHECK_VERSION(2,4,0) */
            pixbuf = gdk_pixbuf_new_from_file (file, NULL);
          g_free (file);
        }
    }
  else
    {
      W32LdrModule *module;

      module = hInstance ? hInstance - 1 : NULL;
      if (IS_INTRESOURCE (lpszName))
        {
          key = g_strdup_printf ("%s"
                                G_SEARCHPATH_SEPARATOR_S "%u"
                                G_SEARCHPATH_SEPARATOR_S "%d"
                                G_SEARCHPATH_SEPARATOR_S "%d,%d",
                                    module && module->file ? module->file : "",
                                    uType,
                                    GPOINTER_TO_INT (lpszName),
                                    width, height);
        }
      else
        {
          gchar *utf8str;

          utf8str = g_utf16_to_utf8 (lpszName, -1, NULL, NULL, NULL);
          key = g_strdup_printf ("%s"
                                G_SEARCHPATH_SEPARATOR_S "%u"
                                G_SEARCHPATH_SEPARATOR_S "%s"
                                G_SEARCHPATH_SEPARATOR_S "%d,%d",
                                    module && module->file ? module->file : "",
                                    uType,
                                    utf8str,
                                    width, height);
          g_free (utf8str);
        }
      if (ghash_image)
        hImage = g_hash_table_lookup (ghash_image, key);
      if (hImage)
        {
          g_free (key);
          key = NULL;
        }
      else
        {
          switch (uType)
            {
              case IMAGE_BITMAP:
                if (hInstance)
                  {
                    BitmapInfoHeader *bmih;
                    HRSRC hRsrc;

                    pixbuf = (hRsrc = FindResourceW (hInstance, lpszName,
                                                MAKEINTRESOURCEW (RT_BITMAP)))
                                    && (bmih = LoadResource (hInstance, hRsrc))
                                            ? pixbuf_from_bitmap (bmih) : NULL;
                  }
                break;
              case IMAGE_CURSOR:
                if (!hInstance)
                  switch (GPOINTER_TO_INT (lpszName))
                    {
                      case OCR_NORMAL:
                        hImage = gdk_cursor_new (GDK_ARROW);
                        break;
                      case OCR_IBEAM:
                        hImage = gdk_cursor_new (GDK_XTERM);
                        break;
                      case OCR_WAIT:
                        hImage = gdk_cursor_new (GDK_WATCH);
                        break;
                      case OCR_CROSS:
                        hImage = gdk_cursor_new (GDK_CROSS);
                        break;
                      case OCR_UP:
                        hImage = gdk_cursor_new (GDK_CENTER_PTR);
                        break;
                      case OCR_SIZE:
                        hImage = gdk_cursor_new (GDK_SIZING);
                        break;
                      case OCR_ICON:
                        hImage = gdk_cursor_new (GDK_ICON);
                        break;
                      case OCR_SIZENWSE:
                        hImage = gdk_cursor_new (GDK_TOP_LEFT_CORNER);
                        break;
                      case OCR_SIZENESW:
                        hImage = gdk_cursor_new (GDK_TOP_RIGHT_CORNER);
                        break;
                      case OCR_SIZEWE:
                        hImage = gdk_cursor_new (GDK_RIGHT_SIDE);
                        break;
                      case OCR_SIZENS:
                        hImage = gdk_cursor_new (GDK_TOP_SIDE);
                        break;
                      case OCR_SIZEALL:
                        hImage = gdk_cursor_new (GDK_FLEUR);
                        break;
                      case OCR_NO:
                        hImage = gdk_cursor_new (GDK_PIRATE);
                        break;
                      case OCR_HAND:
                        hImage = gdk_cursor_new (GDK_HAND1);
                        break;
                      case OCR_APPSTARTING:
                        hImage = gdk_cursor_new (GDK_CLOCK);
                        break;
                      case OCR_HELP:
                        hImage = gdk_cursor_new (GDK_QUESTION_ARROW);
                    }
                break;
              case IMAGE_ICON:
                if (hInstance)
                  {
                    HRSRC hRsrc;
                    LPICONHEADER lpicoh;
                    LPGRPICONDIRENTRY lpicod;

                    lpicoh = (hRsrc = FindResourceW (hInstance, lpszName,
                                            MAKEINTRESOURCEW (RT_GROUP_ICON)))
                                    ? LoadResource (hInstance, hRsrc) : NULL;
                    lpicod = (LPGRPICONDIRENTRY)(lpicoh + 1);
                    if (lpicoh && lpicoh->idType == 1 && lpicoh->idCount > 0)
                      {
                        gint i, index = -1, color = 0, distance = G_MAXINT;
                        BitmapInfoHeader *bmih;

                        for (i = 0; i < lpicoh->idCount; i++)
                          if (lpicod[i].wPlanes == 1)
                            {
                              gint d;

                              d = width > 0 && height > 0
                                                ? (lpicod[i].bWidth - width)
                                                * (lpicod[i].bWidth - width)
                                                + (lpicod[i].bHeight - height)
                                                * (lpicod[i].bHeight - height)
                                                : 0;
                              if (d < distance || (d == distance
                                        && color < (lpicod[i].bColorCount > 0
                                        ? lpicod[i].bColorCount : G_MAXUINT8)))
                                {
                                  color = lpicod[i].bColorCount > 0
                                        ? lpicod[i].bColorCount : G_MAXUINT8;
                                  distance = d;
                                  index = i;
                                }
                            }
                        bmih = index >= 0 && (hRsrc = FindResourceW (hInstance,
                                        MAKEINTRESOURCEW (lpicod[index].nID),
                                                MAKEINTRESOURCEW (RT_ICON)))
                                    ? LoadResource (hInstance, hRsrc) : NULL;
                        if (bmih && bmih_get_size (bmih) == BMIH_SIZE
                            && bmih_get_width (bmih) == lpicod[index].bWidth
                            && bmih_get_planes (bmih) == 1
                            && bmih_get_compression (bmih) == BI_COMP_RGB
                            && bmih_get_x_pixels_per_meter (bmih) == 0
                            && bmih_get_y_pixels_per_meter (bmih) == 0
                            && bmih_get_color_used (bmih) == 0
                            && bmih_get_color_important (bmih) == 0)
                          {
                            if (bmih_get_height (bmih) == lpicod[index].bHeight
                                            && (bmih_get_size_image (bmih) == 0
                                                || bmih_get_size_image (bmih)
                                                    == bm_image_bytes (bmih)))
                              {
                                hImage = pixbuf_from_bitmap (bmih);
                              }
                            else if (bmih_get_height (bmih)
                                                == lpicod[index].bHeight * 2
                                && bmih_get_size_image (bmih)
                                == bx_image_bytes (lpicod[index].bWidth,
                                                   lpicod[index].bHeight,
                                                   bmih_get_bit_count (bmih),
                                                   BI_COMP_RGB, 0)
                                 + bx_image_bytes (lpicod[index].bWidth,
                                                   lpicod[index].bHeight,
                                                   1,
                                                   BI_COMP_RGB, 0))
                              {
                                gsize all_bytes, line_bytes;
                                guint8 *p, *q;
                                gint y;
                                BitmapInfoHeader *bmih0, *bmih1;

                                all_bytes = bx_all_bytes (lpicod[index].bWidth,
                                                    lpicod[index].bHeight,
                                                    bmih_get_bit_count (bmih),
                                                    BI_COMP_RGB, 0);
                                line_bytes = bx_line_bytes
                                                        (lpicod[index].bWidth,
                                                        lpicod[index].bHeight,
                                                        1, BI_COMP_RGB, 0);
                                p = (guint8 *)bmih + all_bytes;
                                bmih0 = g_memdup (bmih, all_bytes);
                                bmih_set_height (bmih0, lpicod[index].bHeight);
                                bmih1 = g_memdup (bmih,
                                        bx_all_bytes (lpicod[index].bWidth,
                                                        lpicod[index].bHeight,
                                                        32, BI_COMP_RGB, 0));
                                bitmap_convert_32 (bmih0, bmih1);
                                g_free (bmih0);
                                q = (guint8 *)bm_image_pointer (bmih1) + 3;
                                for (y = 0; y < lpicod[index].bHeight; y++)
                                  {
                                    gint x;

                                    for (x = 0; x < lpicod[index].bWidth; x++)
                                      {
                                        *q = (p[x / 8] << (x % 8 + 1) & 0x10)
                                                                        - 1;
                                        q += 4;
                                      }
                                    p += line_bytes;
                                  }
                                pixbuf = pixbuf_from_bitmap (bmih1);
                                g_free (bmih1);
                              }
                          }
                      }
                  }
            }
        }
    }
  if (!hImage && pixbuf)
    {
      if (width > 0 && height > 0 && (gdk_pixbuf_get_width (pixbuf) != width
                                || gdk_pixbuf_get_height (pixbuf) != height))
        {
          GdkPixbuf *src;

          src = pixbuf;
          pixbuf = gdk_pixbuf_scale_simple (src, width, height,
                                                            GDK_INTERP_HYPER);
          g_object_unref (G_OBJECT (src));
        }
      if (uType == IMAGE_CURSOR)
        {
#if GLIB_CHECK_VERSION(2,4,0)
          const gchar *str;
          gint x = 0, y = 0;

          str = gdk_pixbuf_get_option (pixbuf, "x_hot");
          if (str)
            misc_str_to_val (&x, str, 10, TRUE);
          str = gdk_pixbuf_get_option (pixbuf, "y_hot");
          if (str)
            misc_str_to_val (&y, str, 10, TRUE);
          hImage = gdk_cursor_new_from_pixbuf
                                    (gdk_display_get_default (), pixbuf, x, y);
#else /* not GLIB_CHECK_VERSION(2,4,0) */
          hImage = NULL;
#endif /* not GLIB_CHECK_VERSION(2,4,0) */
          g_object_unref (G_OBJECT (pixbuf));
        }
      else
        {
          hImage = pixbuf;
        }
    }
  if (key)
    {
      if (hImage)
        {
          if (!ghash_image)
            ghash_image = g_hash_table_new (g_str_hash, g_str_equal);
          g_hash_table_insert (ghash_image, key, hImage);
        }
      else
        {
          g_free (key);
        }
    }
  if (hImage && !(fuLoad & (LR_LOADFROMFILE | LR_SHARED)))
    hImage = GDK_IS_PIXBUF (hImage)
                                ? (HANDLE)gdk_pixbuf_copy (GDK_PIXBUF (hImage))
                                : (HANDLE)gdk_cursor_ref ((GdkCursor *)hImage);
  return hImage;
}


/* LoadKeyboardLayoutA ordinal = 423, argument = 8 temporary */
static DWORD WINAPI LoadKeyboardLayoutA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* LoadKeyboardLayoutEx ordinal = 424, argument = 0 temporary */
static DWORD WINAPI LoadKeyboardLayoutEx (VOID)
{
  return 0;
}


/* LoadKeyboardLayoutW ordinal = 425, argument = 8 temporary */
static DWORD WINAPI LoadKeyboardLayoutW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* LoadLocalFonts ordinal = 426, argument = 0 temporary */
static DWORD WINAPI LoadLocalFonts (VOID)
{
  return 0;
}


/* LoadMenuA ordinal = 427, argument = 8 */
HMENU WINAPI LoadMenuA (HINSTANCE hInstance, LPCSTR lpMenuName)
{
  LPMENUTEMPLATE lpMenuTemplate;
  HRSRC hRsrc;

  return (hRsrc = FindResourceA (hInstance, lpMenuName,
                                                MAKEINTRESOURCEA (RT_MENU)))
                        && (lpMenuTemplate = LoadResource (hInstance, hRsrc))
                ? w32ldr_LoadMenuIndirect (hInstance, lpMenuTemplate) : NULL;
}


/* LoadMenuIndirectA ordinal = 428, argument = 4 */
HMENU WINAPI LoadMenuIndirectA (LPCMENUTEMPLATE lpMenuTemplate)
{
  return w32ldr_LoadMenuIndirect (NULL, lpMenuTemplate);
}


/* LoadMenuIndirectW ordinal = 429, argument = 4 */
HMENU WINAPI LoadMenuIndirectW (LPCMENUTEMPLATE lpMenuTemplate)
{
  return w32ldr_LoadMenuIndirect (NULL, lpMenuTemplate);
}


/* LoadMenuW ordinal = 430, argument = 8 */
HMENU WINAPI LoadMenuW (HINSTANCE hInstance, LPCWSTR lpMenuName)
{
  LPMENUTEMPLATE lpMenuTemplate;
  HRSRC hRsrc;

  return (hRsrc = FindResourceW (hInstance, lpMenuName,
                                                MAKEINTRESOURCEW (RT_MENU)))
                        && (lpMenuTemplate = LoadResource (hInstance, hRsrc))
                ? w32ldr_LoadMenuIndirect (hInstance, lpMenuTemplate) : NULL;
}


/* LoadRemoteFonts ordinal = 431, argument = 0 temporary */
static DWORD WINAPI LoadRemoteFonts (VOID)
{
  return 0;
}


/* LoadStringA ordinal = 432, argument = 16 */
INT WINAPI LoadStringA (HINSTANCE hInstance, UINT uID, LPSTR lpBuffer, INT nBufferMax)
{
  gunichar2 *data;
  HRSRC hRsrc;

  if (!lpBuffer || nBufferMax <= 0)
    return 0;
  if ((hRsrc = FindResourceA (hInstance,
                                MAKEINTRESOURCEA (((uID >> 4) & 0xffff) + 1),
                                MAKEINTRESOURCEA (RT_STRING)))
                                && (data = LoadResource (hInstance, hRsrc)))
    {
      gsize leng;
      gint i;

      for (i = 0; i < (uID & 0x0f); i++)
        data += *data + 1;
      leng = WideCharToMultiByte (CP_ACP, 0, data + 1, *data,
                                        lpBuffer, nBufferMax - 1, NULL, NULL);
      lpBuffer[leng] = '\0';
    }

  return g_strlen (lpBuffer);
}


/* LoadStringW ordinal = 433, argument = 16 */
INT WINAPI LoadStringW (HINSTANCE hInstance, UINT uID, LPWSTR lpBuffer, INT nBufferMax)
{
  gunichar2 *data;
  HRSRC hRsrc;

  if (!lpBuffer || nBufferMax <= 0)
    return 0;
  if ((hRsrc = FindResourceW (hInstance,
                                MAKEINTRESOURCEW (((uID >> 4) & 0xffff) + 1),
                                MAKEINTRESOURCEW (RT_STRING)))
                                && (data = LoadResource (hInstance, hRsrc)))
    {
      gsize leng;
      gint i;

      for (i = 0; i < (uID & 0x0f); i++)
        data += *data + 1;
      leng = MIN (nBufferMax - 1, *data);
      g_memmove (lpBuffer, data + 1, leng * sizeof (gunichar2));
      lpBuffer[leng] = '\0';
    }

  return lstrlenW (lpBuffer);
}


/* LockSetForegroundWindow ordinal = 434, argument = 4 temporary */
static DWORD WINAPI LockSetForegroundWindow (DWORD dwArgv0)
{
  return 0;
}


/* LockWindowStation ordinal = 435, argument = 0 temporary */
static DWORD WINAPI LockWindowStation (VOID)
{
  return 0;
}


/* LockWindowUpdate ordinal = 436, argument = 4 temporary */
static DWORD WINAPI LockWindowUpdate (DWORD dwArgv0)
{
  return 0;
}


/* LockWorkStation ordinal = 437, argument = 0 temporary */
static DWORD WINAPI LockWorkStation (VOID)
{
  return 0;
}


/* LookupIconIdFromDirectory ordinal = 438, argument = 8 temporary */
static DWORD WINAPI LookupIconIdFromDirectory (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* LookupIconIdFromDirectoryEx ordinal = 439, argument = 20 temporary */
static DWORD WINAPI LookupIconIdFromDirectoryEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* MBToWCSEx ordinal = 440, argument = 0 temporary */
static DWORD WINAPI MBToWCSEx (VOID)
{
  return 0;
}


/* MB_GetString ordinal = 441, argument = 0 temporary */
static DWORD WINAPI MB_GetString (VOID)
{
  return 0;
}


/* MapDialogRect ordinal = 442, argument = 8 temporary */
static DWORD WINAPI MapDialogRect (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* MapVirtualKeyA ordinal = 443, argument = 8 temporary */
static DWORD WINAPI MapVirtualKeyA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* MapVirtualKeyExA ordinal = 444, argument = 12 temporary */
static DWORD WINAPI MapVirtualKeyExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* MapVirtualKeyExW ordinal = 445, argument = 12 temporary */
static DWORD WINAPI MapVirtualKeyExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* MapVirtualKeyW ordinal = 446, argument = 8 temporary */
static DWORD WINAPI MapVirtualKeyW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* MapWindowPoints ordinal = 447, argument = 16 */
INT WINAPI MapWindowPoints (HWND hWndFrom, HWND hWndTo, LPPOINT lpPoints, UINT cPoints)
{
  gint i;
  POINT ptMove = {0, 0};
  
  for (i = 0; i < cPoints; i++)
    {
      ClientToScreen (hWndFrom, lpPoints + i);
      ScreenToClient (hWndTo, lpPoints + i);
    }
  ClientToScreen (hWndFrom, &ptMove);
  ScreenToClient (hWndTo, &ptMove);
  return MAKELONG (ptMove.x, ptMove.y);
}


/* MenuItemFromPoint ordinal = 448, argument = 16 */
UINT WINAPI MenuItemFromPoint (HWND hWnd, HMENU hMenu, POINT ptScreen)
{
  return (UINT)-1;
}


/* MenuWindowProcA ordinal = 449, argument = 0 temporary */
static DWORD WINAPI MenuWindowProcA (VOID)
{
  return 0;
}


/* MenuWindowProcW ordinal = 450, argument = 0 temporary */
static DWORD WINAPI MenuWindowProcW (VOID)
{
  return 0;
}


/* MessageBeep ordinal = 451, argument = 4 */
BOOL WINAPI MessageBeep (UINT uType)
{
  return TRUE;
}


/* MessageBoxA ordinal = 452, argument = 16 */
INT WINAPI MessageBoxA (HWND hWnd, LPCSTR lpText, LPCSTR lpCaption, UINT uType)
{
  gchar *utf8str;
  gunichar2 *text, *title;
  INT nResult;

  utf8str = w32ldr_utf8_from_mb (lpText);
  text = utf8str ? g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL) : NULL;
  g_free (utf8str);
  utf8str = w32ldr_utf8_from_mb (lpCaption);
  title = utf8str ? g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL) : NULL;
  g_free (utf8str);
  nResult = MessageBoxW (hWnd, text, title, uType);
  g_free (text);
  g_free (title);
  return nResult;
}


/* MessageBoxExA ordinal = 453, argument = 20 */
INT WINAPI MessageBoxExA (HWND hWnd, LPCSTR lpText, LPCSTR lpCaption, UINT uType, WORD wLanguageId)
{
  return MessageBoxA (hWnd, lpText, lpCaption, uType);
}


/* MessageBoxExW ordinal = 454, argument = 20 */
INT WINAPI MessageBoxExW (HWND hWnd, LPCWSTR lpText, LPCWSTR lpCaption, UINT uType, WORD wLanguageId)
{
  return MessageBoxW (hWnd, lpText, lpCaption, uType);
}


/* MessageBoxIndirectA ordinal = 455, argument = 4 temporary */
static DWORD WINAPI MessageBoxIndirectA (DWORD dwArgv0)
{
  return 0;
}


/* MessageBoxIndirectW ordinal = 456, argument = 4 temporary */
static DWORD WINAPI MessageBoxIndirectW (DWORD dwArgv0)
{
  return 0;
}


/* MessageBoxW ordinal = 457, argument = 16 */
INT WINAPI MessageBoxW (HWND hWnd, LPCWSTR lpText, LPCWSTR lpCaption, UINT uType)
{
  gchar *text, *title;
  gint result;
  GtkMessageType mssage;
  GtkWidget *dialog;

  text = lpText ? g_utf16_to_utf8 (lpText, -1, NULL, NULL, NULL) : NULL;
  title = lpCaption ? g_utf16_to_utf8 (lpCaption, -1, NULL, NULL, NULL) : NULL;
  switch (uType & MB_ICONMASK)
    {
      case MB_ICONHAND:        mssage = GTK_MESSAGE_WARNING; break;
      case MB_ICONQUESTION:    mssage = GTK_MESSAGE_QUESTION; break;
      case MB_ICONEXCLAMATION: mssage = GTK_MESSAGE_WARNING; break;
      default:                 mssage = GTK_MESSAGE_INFO;
    }
  dialog = gtk_message_dialog_new (hWnd ? GTK_WINDOW (hWnd) : NULL,
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            mssage, GTK_BUTTONS_NONE, text);
  gtk_window_set_title (GTK_WINDOW (dialog), title);
  g_free (text);
  g_free (title);

  switch (uType & MB_TYPEMASK)
    {
      case MB_ABORTRETRYIGNORE:
        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                        "_Retry", GTK_RESPONSE_ACCEPT,
                                        "_Ignore", GTK_RESPONSE_REJECT,
                                        "_Abort", GTK_RESPONSE_CANCEL,
                                        NULL);
        break;
      case MB_OK:
        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                        GTK_STOCK_OK, GTK_RESPONSE_OK,
                                        NULL);
        break;
      case MB_OKCANCEL:
        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_OK, GTK_RESPONSE_OK,
                                        NULL);
        break;
      case MB_RETRYCANCEL:
        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        "_Retry", GTK_RESPONSE_ACCEPT,
                                        NULL);
        break;
      case MB_YESNO:
        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                        GTK_STOCK_NO, GTK_RESPONSE_NO,
                                        GTK_STOCK_YES, GTK_RESPONSE_YES,
                                        NULL);
        break;
      case MB_YESNOCANCEL:
        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                                        GTK_STOCK_NO, GTK_RESPONSE_NO,
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_YES, GTK_RESPONSE_YES,
                                        NULL);
    }
  result = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
  switch (uType & MB_TYPEMASK)
    {
      case MB_ABORTRETRYIGNORE:
        switch (result)
          {
            case  GTK_RESPONSE_ACCEPT: return IDRETRY;
            case  GTK_RESPONSE_REJECT: return IDIGNORE;
          }
        return IDABORT;
      case MB_OK:
        return IDOK;
      case MB_OKCANCEL:
        return result == GTK_RESPONSE_OK ? IDOK : IDCANCEL;
      case MB_RETRYCANCEL:
        return result == GTK_RESPONSE_ACCEPT ? IDRETRY : IDCANCEL;
      case MB_YESNO:
        return result == GTK_RESPONSE_YES ? IDYES : IDNO;
      case MB_YESNOCANCEL:
        switch (result)
          {
            case  GTK_RESPONSE_YES: return IDYES;
            case  GTK_RESPONSE_NO:  return IDNO;
          }
    }
  return IDCANCEL;
}


/* ModifyMenuA ordinal = 458, argument = 20 */
BOOL WINAPI ModifyMenuA (HMENU hMenu, UINT uItem, UINT uFlags, UINT_PTR uNewItem, LPCSTR lpszNewItem)
{
  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item
                && InsertMenuA (hMenu, uItem, uFlags, uNewItem, lpszNewItem))
        {
          gtk_container_remove (GTK_CONTAINER (hMenu), menu_item);
          return TRUE;
        }
    }
  return FALSE;
}


/* ModifyMenuW ordinal = 459, argument = 20 */
BOOL WINAPI ModifyMenuW (HMENU hMenu, UINT uItem, UINT uFlags, UINT_PTR uNewItem, LPCWSTR lpszNewItem)
{
  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item
                && InsertMenuW (hMenu, uItem, uFlags, uNewItem, lpszNewItem))
        {
          gtk_container_remove (GTK_CONTAINER (hMenu), menu_item);
          return TRUE;
        }
    }
  return FALSE;
}


/* MonitorFromPoint ordinal = 460, argument = 12 temporary */
static DWORD WINAPI MonitorFromPoint (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* MonitorFromRect ordinal = 461, argument = 8 temporary */
static DWORD WINAPI MonitorFromRect (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* MonitorFromWindow ordinal = 462, argument = 8 temporary */
static DWORD WINAPI MonitorFromWindow (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* MoveWindow ordinal = 463, argument = 24 */
BOOL WINAPI MoveWindow (HWND hWnd, INT x, INT y, INT nWidth, INT nHeight, BOOL bRepaint)
{
  return SetWindowPos (hWnd, NULL, x, y, nWidth, nHeight,
                                (bRepaint ? 0 : SWP_NOREDRAW) | SWP_NOZORDER);
}


/* MsgWaitForMultipleObjects ordinal = 464, argument = 20 temporary */
static DWORD WINAPI MsgWaitForMultipleObjects (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* MsgWaitForMultipleObjectsEx ordinal = 465, argument = 20 temporary */
static DWORD WINAPI MsgWaitForMultipleObjectsEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* NotifyWinEvent ordinal = 466, argument = 16 temporary */
static DWORD WINAPI NotifyWinEvent (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* OemKeyScan ordinal = 467, argument = 4 temporary */
static DWORD WINAPI OemKeyScan (DWORD dwArgv0)
{
  return 0;
}


/* OemToCharA ordinal = 468, argument = 8 temporary */
static DWORD WINAPI OemToCharA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* OemToCharBuffA ordinal = 469, argument = 12 temporary */
static DWORD WINAPI OemToCharBuffA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* OemToCharBuffW ordinal = 470, argument = 12 temporary */
static DWORD WINAPI OemToCharBuffW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* OemToCharW ordinal = 471, argument = 8 temporary */
static DWORD WINAPI OemToCharW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* OffsetRect ordinal = 472, argument = 12 */
BOOL WINAPI OffsetRect (LPRECT lprc, INT dx, INT dy)
{
  if (!lprc)
    return FALSE;
  lprc->left += dx;
  lprc->top += dy;
  lprc->right += dx;
  lprc->bottom += dy;
  return TRUE;
}


/* OpenClipboard ordinal = 473, argument = 4 */
BOOL WINAPI OpenClipboard (HWND hWndNewOwner)
{
  return TRUE;
}


/* OpenDesktopA ordinal = 474, argument = 16 temporary */
static DWORD WINAPI OpenDesktopA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* OpenDesktopW ordinal = 475, argument = 16 temporary */
static DWORD WINAPI OpenDesktopW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* OpenIcon ordinal = 476, argument = 4 temporary */
static DWORD WINAPI OpenIcon (DWORD dwArgv0)
{
  return 0;
}


/* OpenInputDesktop ordinal = 477, argument = 12 temporary */
static DWORD WINAPI OpenInputDesktop (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* OpenWindowStationA ordinal = 478, argument = 12 temporary */
static DWORD WINAPI OpenWindowStationA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* OpenWindowStationW ordinal = 479, argument = 12 temporary */
static DWORD WINAPI OpenWindowStationW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* PackDDElParam ordinal = 480, argument = 12 temporary */
static DWORD WINAPI PackDDElParam (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* PaintDesktop ordinal = 481, argument = 4 temporary */
static DWORD WINAPI PaintDesktop (DWORD dwArgv0)
{
  return 0;
}


/* PeekMessageA ordinal = 482, argument = 20 */
BOOL WINAPI PeekMessageA (LPMSG lpMsg, HWND hWnd, UINT wMsgFilterMin, UINT wMsgFilterMax, UINT wRemoveMsg)
{
  return PeekMessageW (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, wRemoveMsg);
}


/* PeekMessageW ordinal = 483, argument = 20 */
BOOL WINAPI PeekMessageW (LPMSG lpMsg, HWND hWnd, UINT wMsgFilterMin, UINT wMsgFilterMax, UINT wRemoveMsg)
{
  if (lpMsg)
    {
      GList *glist;

      for (glist = g_list_first (glist_message);
                                            glist; glist = g_list_next (glist))
        {
          W32LdrMessage *w32msg;

          w32msg = glist->data;
          if (w32msg->Msg == WM_QUIT
                || ((!hWnd || w32msg->hWnd == hWnd
                                                || IsChild (hWnd, w32msg->hWnd)
                            || (GPOINTER_TO_INT (hWnd) == -1 && !w32msg->hWnd))
                && ((wMsgFilterMin == 0 && wMsgFilterMax == 0)
                || (wMsgFilterMin <= w32msg->Msg
                                            && w32msg->Msg <= wMsgFilterMax))))
            {
              lpMsg->hwnd = w32msg->hWnd;
              lpMsg->message = w32msg->Msg;
              lpMsg->wParam = w32msg->wParam;
              lpMsg->lParam = w32msg->lParam;
              lpMsg->time = 0;
              lpMsg->pt.x = 0;
              lpMsg->pt.y = 0;
              if (wRemoveMsg & PM_REMOVE)
                {
                  glist_message = g_list_remove (glist_message, w32msg);
                  g_free (w32msg);
                }
              return TRUE;
            }
        }
    }
  return FALSE;
}


/* PostMessageA ordinal = 484, argument = 16 */
BOOL WINAPI PostMessageA (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return PostMessageW (hWnd, Msg, wParam, lParam);
}


/* PostMessageW ordinal = 485, argument = 16 */
BOOL WINAPI PostMessageW (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  W32LdrMessage *w32msg;

  if (!hWnd)
    return FALSE;
  w32msg = g_malloc (sizeof (W32LdrMessage));
  w32msg->hWnd = hWnd;
  w32msg->Msg = Msg;
  w32msg->wParam = wParam;
  w32msg->lParam = lParam;
  glist_message = g_list_append (glist_message, w32msg);
  return TRUE;
}


/* PostQuitMessage ordinal = 486, argument = 4 */
VOID WINAPI PostQuitMessage (INT nExitCode)
{
  W32LdrMessage *w32msg;

  w32msg = g_malloc (sizeof (W32LdrMessage));
  w32msg->hWnd = NULL;
  w32msg->Msg = WM_QUIT;
  w32msg->wParam = nExitCode;
  w32msg->lParam = 0;
  glist_message = g_list_prepend (glist_message, w32msg);
}


/* PostThreadMessageA ordinal = 487, argument = 16 temporary */
static DWORD WINAPI PostThreadMessageA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* PostThreadMessageW ordinal = 488, argument = 16 temporary */
static DWORD WINAPI PostThreadMessageW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* PrivateExtractIconExA ordinal = 489, argument = 0 temporary */
static DWORD WINAPI PrivateExtractIconExA (VOID)
{
  return 0;
}


/* PrivateExtractIconExW ordinal = 490, argument = 0 temporary */
static DWORD WINAPI PrivateExtractIconExW (VOID)
{
  return 0;
}


/* PrivateExtractIconsA ordinal = 491, argument = 32 temporary */
static DWORD WINAPI PrivateExtractIconsA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* PrivateExtractIconsW ordinal = 492, argument = 32 temporary */
static DWORD WINAPI PrivateExtractIconsW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* PrivateSetDbgTag ordinal = 493, argument = 0 temporary */
static DWORD WINAPI PrivateSetDbgTag (VOID)
{
  return 0;
}


/* PrivateSetRipFlags ordinal = 494, argument = 0 temporary */
static DWORD WINAPI PrivateSetRipFlags (VOID)
{
  return 0;
}


/* PtInRect ordinal = 495, argument = 12 */
BOOL WINAPI PtInRect (CONST LPRECT lprc, POINT pt)
{
  if (!lprc)
    return FALSE;
  return pt.x >= lprc->left && pt.y >= lprc->top
                                && pt.x < lprc->right && pt.y < lprc->bottom;
}


/* QuerySendMessage ordinal = 496, argument = 0 temporary */
static DWORD WINAPI QuerySendMessage (VOID)
{
  return 0;
}


/* QueryUserCounters ordinal = 497, argument = 0 temporary */
static DWORD WINAPI QueryUserCounters (VOID)
{
  return 0;
}


/* RealChildWindowFromPoint ordinal = 498, argument = 12 temporary */
static DWORD WINAPI RealChildWindowFromPoint (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* RealGetWindowClass ordinal = 499, argument = 12 temporary */
/* DWORD WINAPI RealGetWindowClass (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2) */


/* RealGetWindowClassA ordinal = 500, argument = 12 temporary */
static DWORD WINAPI RealGetWindowClassA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* RealGetWindowClassW ordinal = 501, argument = 12 temporary */
static DWORD WINAPI RealGetWindowClassW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* RedrawWindow ordinal = 502, argument = 16 */
BOOL WINAPI RedrawWindow (HWND hWnd, LPCRECT lprcUpdate, HRGN hrgnUpdate, UINT flags)
{
  if (flags & RDW_INVALIDATE)
    {
      if (lprcUpdate)
        {
          gtk_widget_queue_draw_area (hWnd, lprcUpdate->left, lprcUpdate->top,
                                        lprcUpdate->right - lprcUpdate->left,
                                        lprcUpdate->bottom - lprcUpdate->top);
        }
      else if (hrgnUpdate)
        {
          gint i, n_rectangles;
          GdkRectangle *rectangles;

          gdk_region_get_rectangles (hrgnUpdate, &rectangles, &n_rectangles);
          for (i = 0; i < n_rectangles; i++)
            gtk_widget_queue_draw_area (hWnd, rectangles[i].x, rectangles[i].y,
                                    rectangles[i].width, rectangles[i].height);
          g_free (rectangles);
        }
      else
        {
          gtk_widget_queue_draw (hWnd);
        }
    }
  return 0;
}


/* RegisterClassA ordinal = 503, argument = 4 */
ATOM WINAPI RegisterClassA (CONST LPWNDCLASSA lpWndClass)
{
  WNDCLASSEXA wcex;

  if (!lpWndClass)
    return 0;
  wcex.cbSize = sizeof (WNDCLASSEXA);
  wcex.style = lpWndClass->style;
  wcex.lpfnWndProc = lpWndClass->lpfnWndProc;
  wcex.cbClsExtra = lpWndClass->cbClsExtra;
  wcex.cbWndExtra = lpWndClass->cbWndExtra;
  wcex.hInstance = lpWndClass->hInstance;
  wcex.hIcon = lpWndClass->hIcon;
  wcex.hCursor = lpWndClass->hCursor;
  wcex.hbrBackground = lpWndClass->hbrBackground;
  wcex.lpszMenuName = lpWndClass->lpszMenuName;
  wcex.lpszClassName = lpWndClass->lpszClassName;
  wcex.hIconSm = NULL;
  return RegisterClassExA (&wcex);
}


/* RegisterClassExA ordinal = 504, argument = 4 */
ATOM WINAPI RegisterClassExA (CONST LPWNDCLASSEXA lpWndClassEx)
{
  gchar *menuname, *classname;
  ATOM nAtom;

  if (!lpWndClassEx)
    return 0;
  menuname = GPOINTER_TO_UINT (lpWndClassEx->lpszMenuName) > G_MAXUINT16
                        ? w32ldr_utf8_from_mb (lpWndClassEx->lpszMenuName)
                        : (gchar *)lpWndClassEx->lpszMenuName;
  classname = GPOINTER_TO_UINT (lpWndClassEx->lpszClassName) > G_MAXUINT16
                        ? w32ldr_utf8_from_mb (lpWndClassEx->lpszClassName)
                        : (gchar *)lpWndClassEx->lpszClassName;
  nAtom = w32ldr_RegisterClassEx (lpWndClassEx->style,
                                  lpWndClassEx->lpfnWndProc,
                                  lpWndClassEx->cbClsExtra,
                                  lpWndClassEx->cbWndExtra,
                                  lpWndClassEx->hInstance,
                                  lpWndClassEx->hIcon,
                                  lpWndClassEx->hCursor,
                                  lpWndClassEx->hbrBackground,
                                  menuname,
                                  classname,
                                  lpWndClassEx->hIconSm,
                                  FALSE);
  if (GPOINTER_TO_UINT (menuname) > G_MAXUINT16)
    g_free (menuname);
  if (GPOINTER_TO_UINT (classname) > G_MAXUINT16)
    g_free (classname);
  return nAtom;
}


/* RegisterClassExW ordinal = 505, argument = 4 */
ATOM WINAPI RegisterClassExW (CONST LPWNDCLASSEXW lpWndClassEx)
{
  gchar *menuname, *classname;
  ATOM nAtom;

  if (!lpWndClassEx)
    return 0;
  menuname = GPOINTER_TO_UINT (lpWndClassEx->lpszMenuName) > G_MAXUINT16
        ? g_utf16_to_utf8 (lpWndClassEx->lpszMenuName, -1, NULL, NULL, NULL)
        : (gchar *)lpWndClassEx->lpszMenuName;
  classname = GPOINTER_TO_UINT (lpWndClassEx->lpszClassName) > G_MAXUINT16
        ? g_utf16_to_utf8 (lpWndClassEx->lpszClassName, -1, NULL, NULL, NULL)
        : (gchar *)lpWndClassEx->lpszClassName;
  nAtom = w32ldr_RegisterClassEx (lpWndClassEx->style,
                                  lpWndClassEx->lpfnWndProc,
                                  lpWndClassEx->cbClsExtra,
                                  lpWndClassEx->cbWndExtra,
                                  lpWndClassEx->hInstance,
                                  lpWndClassEx->hIcon,
                                  lpWndClassEx->hCursor,
                                  lpWndClassEx->hbrBackground,
                                  menuname,
                                  classname,
                                  lpWndClassEx->hIconSm,
                                  TRUE);
  if (GPOINTER_TO_UINT (menuname) > G_MAXUINT16)
    g_free (menuname);
  if (GPOINTER_TO_UINT (classname) > G_MAXUINT16)
    g_free (classname);
  return nAtom;
}


/* RegisterClassW ordinal = 506, argument = 4 */
ATOM WINAPI RegisterClassW (CONST LPWNDCLASSW lpWndClass)
{
  WNDCLASSEXW wcex;

  if (!lpWndClass)
    return 0;
  wcex.cbSize = sizeof (WNDCLASSEXW);
  wcex.style = lpWndClass->style;
  wcex.lpfnWndProc = lpWndClass->lpfnWndProc;
  wcex.cbClsExtra = lpWndClass->cbClsExtra;
  wcex.cbWndExtra = lpWndClass->cbWndExtra;
  wcex.hInstance = lpWndClass->hInstance;
  wcex.hIcon = lpWndClass->hIcon;
  wcex.hCursor = lpWndClass->hCursor;
  wcex.hbrBackground = lpWndClass->hbrBackground;
  wcex.lpszMenuName = lpWndClass->lpszMenuName;
  wcex.lpszClassName = lpWndClass->lpszClassName;
  wcex.hIconSm = NULL;
  return RegisterClassExW (&wcex);
}


/* RegisterClipboardFormatA ordinal = 507, argument = 4 temporary */
static DWORD WINAPI RegisterClipboardFormatA (DWORD dwArgv0)
{
  return 0;
}


/* RegisterClipboardFormatW ordinal = 508, argument = 4 temporary */
static DWORD WINAPI RegisterClipboardFormatW (DWORD dwArgv0)
{
  return 0;
}


/* RegisterDeviceNotificationA ordinal = 509, argument = 12 temporary */
static DWORD WINAPI RegisterDeviceNotificationA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* RegisterDeviceNotificationW ordinal = 510, argument = 12 temporary */
static DWORD WINAPI RegisterDeviceNotificationW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* RegisterHotKey ordinal = 511, argument = 16 temporary */
static DWORD WINAPI RegisterHotKey (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* RegisterLogonProcess ordinal = 512, argument = 0 temporary */
static DWORD WINAPI RegisterLogonProcess (VOID)
{
  return 0;
}


/* RegisterServicesProcess ordinal = 513, argument = 0 temporary */
static DWORD WINAPI RegisterServicesProcess (VOID)
{
  return 0;
}


/* RegisterShellHookWindow ordinal = 514, argument = 4 temporary */
static DWORD WINAPI RegisterShellHookWindow (DWORD dwArgv0)
{
  return 0;
}


/* RegisterSystemThread ordinal = 515, argument = 0 temporary */
static DWORD WINAPI RegisterSystemThread (VOID)
{
  return 0;
}


/* RegisterTasklist ordinal = 516, argument = 0 temporary */
static DWORD WINAPI RegisterTasklist (VOID)
{
  return 0;
}


/* RegisterWindowMessageA ordinal = 517, argument = 4 temporary */
static DWORD WINAPI RegisterWindowMessageA (DWORD dwArgv0)
{
  return 0;
}


/* RegisterWindowMessageW ordinal = 518, argument = 4 temporary */
static DWORD WINAPI RegisterWindowMessageW (DWORD dwArgv0)
{
  return 0;
}


/* ReleaseCapture ordinal = 519, argument = 0 */
BOOL WINAPI ReleaseCapture (VOID)
{
  return TRUE;
}


/* ReleaseDC ordinal = 520, argument = 8 */
INT WINAPI ReleaseDC (HWND hWnd, HDC hDC)
{
  if (!hDC || hDC->widget != hWnd)
    return FALSE;
  if (hDC->pixbuf)
    g_object_unref (G_OBJECT (hDC->pixbuf));
  if (hDC->brush)
    g_object_unref (G_OBJECT (hDC->brush));
  if (hDC->font)
    g_object_unref (G_OBJECT (hDC->font));
  if (hDC->pen)
    g_object_unref (G_OBJECT (hDC->pen));
  g_free (hDC);
  return TRUE;
}


/* RemoveMenu ordinal = 521, argument = 12 */
BOOL WINAPI RemoveMenu (HMENU hMenu, UINT uItem, UINT uFlags)
{
  if (hMenu)
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem, uFlags);
      if (menu_item)
        {
          GtkWidget *menu;

          menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
          if (menu)
            g_object_ref (G_OBJECT (menu));
          gtk_container_remove (GTK_CONTAINER (hMenu), menu_item);
          return TRUE;
        }
    }
  return FALSE;
}


/* RemovePropA ordinal = 522, argument = 8 temporary */
static DWORD WINAPI RemovePropA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* RemovePropW ordinal = 523, argument = 8 temporary */
static DWORD WINAPI RemovePropW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ReplyMessage ordinal = 524, argument = 4 temporary */
static DWORD WINAPI ReplyMessage (DWORD dwArgv0)
{
  return 0;
}


/* ResolveDesktopForWOW ordinal = 525, argument = 0 temporary */
static DWORD WINAPI ResolveDesktopForWOW (VOID)
{
  return 0;
}


/* ReuseDDElParam ordinal = 526, argument = 20 temporary */
static DWORD WINAPI ReuseDDElParam (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ScreenToClient ordinal = 527, argument = 8 */
BOOL WINAPI ScreenToClient (HWND hWnd, LPPOINT lpPoint)
{
  GdkWindow *window;
  GtkWidget *widget = NULL;

  if (!hWnd || !lpPoint)
    return FALSE;
  if (GTK_IS_WINDOW (hWnd))
    {
      W32LdrOverlapData *od;

      od = g_object_get_data (G_OBJECT (hWnd), "user_data");
      widget = od->fixed;
    }
  if (!widget)
    widget = hWnd;
  lpPoint->x -= widget->allocation.x;
  lpPoint->y -= widget->allocation.y;
  for (window = widget->window; window;
                                    window = gdk_window_get_parent (window))
    {
      gint x, y;

      gdk_window_get_position (window, &x, &y);
      lpPoint->x -= x;
      lpPoint->y -= y;
    }
  return TRUE;
}


/* ScrollChildren ordinal = 528, argument = 0 temporary */
static DWORD WINAPI ScrollChildren (VOID)
{
  return 0;
}


/* ScrollDC ordinal = 529, argument = 28 temporary */
static DWORD WINAPI ScrollDC (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* ScrollWindow ordinal = 530, argument = 20 temporary */
static DWORD WINAPI ScrollWindow (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ScrollWindowEx ordinal = 531, argument = 32 temporary */
static DWORD WINAPI ScrollWindowEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* SendDlgItemMessageA ordinal = 532, argument = 20 */
LONG WINAPI SendDlgItemMessageA (HWND hDlg, INT nIDDlgItem, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return SendMessageA (GetDlgItem (hDlg, nIDDlgItem), Msg, wParam, lParam);
}


/* SendDlgItemMessageW ordinal = 533, argument = 20 */
LONG WINAPI SendDlgItemMessageW (HWND hDlg, INT nIDDlgItem, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return SendMessageW (GetDlgItem (hDlg, nIDDlgItem), Msg, wParam, lParam);
}


/* SendIMEMessageExA ordinal = 534, argument = 8 temporary */
static DWORD WINAPI SendIMEMessageExA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SendIMEMessageExW ordinal = 535, argument = 8 temporary */
static DWORD WINAPI SendIMEMessageExW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SendInput ordinal = 536, argument = 12 temporary */
static DWORD WINAPI SendInput (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SendMessageA ordinal = 537, argument = 16 */
LRESULT WINAPI SendMessageA (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  return SendMessageW (hWnd, Msg, wParam, lParam);
}


/* SendMessageCallbackA ordinal = 538, argument = 24 temporary */
static DWORD WINAPI SendMessageCallbackA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* SendMessageCallbackW ordinal = 539, argument = 24 temporary */
static DWORD WINAPI SendMessageCallbackW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* SendMessageTimeoutA ordinal = 540, argument = 28 temporary */
static DWORD WINAPI SendMessageTimeoutA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* SendMessageTimeoutW ordinal = 541, argument = 28 temporary */
static DWORD WINAPI SendMessageTimeoutW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* SendMessageW ordinal = 542, argument = 16 */
LRESULT WINAPI SendMessageW (HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
  W32LdrWindowData *wd;

  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  if (wd)
    {
      if (wd->lpDialogFunc)
        {
          INT_PTR nResult;

          nResult = wd->lpDialogFunc (hWnd, Msg, wParam, lParam);
          if (nResult)
            return nResult;
        }
      if (wd->lpfnWndProc){
        return wd->lpfnWndProc (hWnd, Msg, wParam, lParam);}
    }
  return 0;
}


/* SendNotifyMessageA ordinal = 543, argument = 16 temporary */
static DWORD WINAPI SendNotifyMessageA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SendNotifyMessageW ordinal = 544, argument = 16 temporary */
static DWORD WINAPI SendNotifyMessageW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetActiveWindow ordinal = 545, argument = 4 temporary */
static DWORD WINAPI SetActiveWindow (DWORD dwArgv0)
{
  return 0;
}


/* SetCapture ordinal = 546, argument = 4 */
HWND WINAPI SetCapture (HWND hWnd)
{
  return NULL;
}


/* SetCaretBlinkTime ordinal = 547, argument = 4 temporary */
static DWORD WINAPI SetCaretBlinkTime (DWORD dwArgv0)
{
  return 0;
}


/* SetCaretPos ordinal = 548, argument = 8 temporary */
static DWORD WINAPI SetCaretPos (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetClassLongA ordinal = 549, argument = 12 temporary */
static DWORD WINAPI SetClassLongA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetClassLongW ordinal = 550, argument = 12 temporary */
static DWORD WINAPI SetClassLongW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetClassWord ordinal = 551, argument = 12 temporary */
static DWORD WINAPI SetClassWord (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetClipboardData ordinal = 552, argument = 8 */
HANDLE WINAPI SetClipboardData (UINT uFormat, HANDLE hMem)
{
  GtkClipboard *clipboard;

  clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (clipboard && hMem)
    {
      LPVOID lpData;

      lpData = GlobalLock (hMem);
      if (lpData)
        {
          switch (uFormat)
            {
              case CF_TEXT:
              case CF_OEMTEXT:
                {
                  gchar *utf8str;

                  utf8str = w32ldr_utf8_from_mb (lpData);
                  if (utf8str)
                    {
                      gtk_clipboard_set_text (clipboard, utf8str, -1);
                      g_free (utf8str);
                    }
                }
                break;
              case CF_UNICODETEXT:
                {
                  gchar *utf8str;

                  utf8str = g_utf16_to_utf8 (lpData, -1, NULL, NULL, NULL);
                  if (utf8str)
                    {
                      gtk_clipboard_set_text (clipboard, utf8str, -1);
                      g_free (utf8str);
                    }
                }
                break;
            }
          GlobalUnlock (hMem);
        }
      GlobalFree (hMem);
    }
  return NULL;
}


/* SetClipboardViewer ordinal = 553, argument = 4 temporary */
static DWORD WINAPI SetClipboardViewer (DWORD dwArgv0)
{
  return 0;
}


/* SetConsoleReserveKeys ordinal = 554, argument = 0 temporary */
static DWORD WINAPI SetConsoleReserveKeys (VOID)
{
  return 0;
}


/* SetCursor ordinal = 555, argument = 4 */
HCURSOR WINAPI SetCursor (HCURSOR hCursor)
{
  return NULL;
}


/* SetCursorContents ordinal = 556, argument = 0 temporary */
static DWORD WINAPI SetCursorContents (VOID)
{
  return 0;
}


/* SetCursorPos ordinal = 557, argument = 8 */
BOOL WINAPI SetCursorPos (INT x, INT y)
{
  return FALSE;
}


/* SetDebugErrorLevel ordinal = 558, argument = 4 temporary */
static DWORD WINAPI SetDebugErrorLevel (DWORD dwArgv0)
{
  return 0;
}


/* SetDeskWallpaper ordinal = 559, argument = 4 temporary */
static DWORD WINAPI SetDeskWallpaper (DWORD dwArgv0)
{
  return 0;
}


/* SetDlgItemInt ordinal = 560, argument = 16 */
BOOL WINAPI SetDlgItemInt (HWND hDlg, INT nIDDlgItem, UINT uValue, BOOL bSigned)
{
  gchar *text;
  BOOL fResult;

  text = misc_str_from_val (uValue, 10, 0, bSigned);
  if (IsWindowUnicode (GetDlgItem (hDlg, nIDDlgItem)))
    {
      gunichar2 *wc;

      wc = text ? g_utf8_to_utf16 (text, -1, NULL, NULL, NULL)
                : g_malloc0 (sizeof (gunichar2));
      fResult = SendDlgItemMessageW (hDlg, nIDDlgItem,
                                        WM_SETTEXT, 0, GPOINTER_TO_INT (wc));
      g_free (wc);
    }
  else
    {
      fResult = SendDlgItemMessageA (hDlg, nIDDlgItem,
                                        WM_SETTEXT, 0, GPOINTER_TO_INT (text));
    }
  g_free (text);
  return fResult;
}


/* SetDlgItemTextA ordinal = 561, argument = 12 */
BOOL WINAPI SetDlgItemTextA (HWND hDlg, INT nIDDlgItem, LPCSTR lpString)
{
  return SendDlgItemMessageA (hDlg, nIDDlgItem,
                                    WM_SETTEXT, 0, GPOINTER_TO_INT (lpString));
}


/* SetDlgItemTextW ordinal = 562, argument = 12 */
BOOL WINAPI SetDlgItemTextW (HWND hDlg, INT nIDDlgItem, LPCWSTR lpString)
{
  return SendDlgItemMessageW (hDlg, nIDDlgItem,
                                    WM_SETTEXT, 0, GPOINTER_TO_INT (lpString));
}


/* SetDoubleClickTime ordinal = 563, argument = 4 temporary */
static DWORD WINAPI SetDoubleClickTime (DWORD dwArgv0)
{
  return 0;
}


/* SetFocus ordinal = 564, argument = 4 */
HWND WINAPI SetFocus (HWND hWnd)
{
  if (hWnd)
    gtk_widget_grab_focus (hWnd);
  return hWnd;
}


/* SetForegroundWindow ordinal = 565, argument = 4 */
BOOL WINAPI SetForegroundWindow (HWND hWnd)
{
  return TRUE;
}


/* SetInternalWindowPos ordinal = 566, argument = 0 temporary */
static DWORD WINAPI SetInternalWindowPos (VOID)
{
  return 0;
}


/* SetKeyboardState ordinal = 567, argument = 4 temporary */
static DWORD WINAPI SetKeyboardState (DWORD dwArgv0)
{
  return 0;
}


/* SetLastErrorEx ordinal = 568, argument = 8 temporary */
static DWORD WINAPI SetLastErrorEx (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetLayeredWindowAttributes ordinal = 569, argument = 16 temporary */
static DWORD WINAPI SetLayeredWindowAttributes (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetLogonNotifyWindow ordinal = 570, argument = 0 temporary */
static DWORD WINAPI SetLogonNotifyWindow (VOID)
{
  return 0;
}


/* SetMenu ordinal = 571, argument = 8 */
BOOL WINAPI SetMenu (HWND hWnd, HMENU hMenu)
{
  W32LdrOverlapData *od;

  if (!hWnd || !GTK_IS_WINDOW (hWnd))
    return FALSE;
  od = g_object_get_data (G_OBJECT (hWnd), "user_data");
  if (!od)
    return FALSE;
  if (od->menu)
    gtk_container_remove (GTK_CONTAINER (od->table), od->menu);
  od->menu = hMenu;
  if (od->menu)
    gtk_table_attach (GTK_TABLE (od->table), od->menu, 0, 2, 0, 1,
                                            GTK_EXPAND | GTK_FILL, 0, 0, 0);
  return TRUE;
}


/* SetMenuContextHelpId ordinal = 572, argument = 8 temporary */
static DWORD WINAPI SetMenuContextHelpId (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetMenuDefaultItem ordinal = 573, argument = 12 */
BOOL WINAPI SetMenuDefaultItem (HMENU hMenu, UINT uItem, UINT uByPos)
{
  return FALSE;
}


/* SetMenuInfo ordinal = 574, argument = 8 temporary */
static DWORD WINAPI SetMenuInfo (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetMenuItemBitmaps ordinal = 575, argument = 20 */
BOOL WINAPI SetMenuItemBitmaps (HMENU hMenu, UINT uItem, UINT uFlags, HBITMAP hbmUnchecked, HBITMAP hbmChecked)
{
  return FALSE;
}


/* SetMenuItemInfoA ordinal = 576, argument = 16 */
BOOL WINAPI SetMenuItemInfoA (HMENU hMenu, UINT uItem, BOOL fByPosition, LPMENUITEMINFOA lpmii)
{
  if (hMenu && lpmii
                && lpmii->cbSize >= sizeof (MENUITEMINFOA) - sizeof (HBITMAP))
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem,
                                fByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
      if (menu_item)
        {
          GtkWidget *menu_item_new = NULL;

          if (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE)
                                            && lpmii->fType & MFT_SEPARATOR)
            {
              if (!GTK_IS_SEPARATOR_MENU_ITEM (menu_item))
                menu_item_new = gtk_separator_menu_item_new ();
            }
          else
            {
              if (!GTK_IS_IMAGE_MENU_ITEM (menu_item))
                menu_item_new = gtk_image_menu_item_new_with_label ("");
            }
          if (menu_item_new)
            {
              GtkWidget *menu;

              menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
              if (menu)
                g_object_ref (G_OBJECT (menu));
              g_object_set_data (G_OBJECT (menu_item_new), "user_data",
                        g_object_get_data (G_OBJECT (menu_item), "user_data"));
              gtk_menu_shell_insert (GTK_MENU_SHELL (hMenu), menu_item,
                fByPosition ? uItem : w32ldr_menu_get_item_pos (hMenu, uItem));
              gtk_container_remove (GTK_CONTAINER (hMenu), menu_item);
              if (menu)
                gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item_new),
                                                                        menu);
              menu_item = menu_item_new;
            }
          if (lpmii->fMask & MIIM_STATE)
            {
              gtk_widget_set_sensitive (menu_item, lpmii->fState & MFS_GRAYED);
              if (GTK_IS_IMAGE_MENU_ITEM (menu_item))
                {
                  GtkWidget *image = NULL;

                  if (lpmii->fState & MFS_CHECKED)
                    {
                      GdkPixbuf *pixbuf;

                      pixbuf = lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE)
                                            && lpmii->fType & MFT_RADIOCHECK
                                            ? w32ldr_menu_get_pixbuf_radio ()
                                            : w32ldr_menu_get_pixbuf_check ();
                      if (pixbuf)
                        image = gtk_image_new_from_pixbuf (pixbuf);
                    }
                  if (!(lpmii->fState & MFS_CHECKED) || image)
                    gtk_image_menu_item_set_image
                                    (GTK_IMAGE_MENU_ITEM (menu_item), image);
                }
            }
          if (lpmii->fMask & MIIM_ID)
            g_object_set_data (G_OBJECT (menu_item), "user_data",
                                                GUINT_TO_POINTER (lpmii->wID));
          if (lpmii->fMask & MIIM_SUBMENU)
            gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
                                                            lpmii->hSubMenu);
          if (GTK_IS_IMAGE_MENU_ITEM (menu_item))
            {
              gchar *utf8str = NULL;
              GtkWidget *image = NULL;

              if (lpmii->fMask & MIIM_BITMAP
                                    && lpmii->cbSize >= sizeof (MENUITEMINFOA))
                {
                  image = gtk_image_new_from_pixbuf (lpmii->hbmpItem);
                  g_object_unref (G_OBJECT (lpmii->hbmpItem));
                }
              else if (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE)
                                                && lpmii->fType & MFT_BITMAP)
                {
                  image = gtk_image_new_from_pixbuf
                                            ((GdkPixbuf *)lpmii->dwTypeData);
                  g_object_unref (G_OBJECT (lpmii->dwTypeData));
                }
              if (image)
                {
                  gtk_image_menu_item_set_image
                                    (GTK_IMAGE_MENU_ITEM (menu_item), image);
                  utf8str = g_malloc0 (sizeof (gchar));
                }
              else if (lpmii->fMask & MIIM_STRING || lpmii->fMask & MIIM_TYPE)
                {
                  utf8str = lpmii->dwTypeData
                                    ? w32ldr_utf8_from_mb (lpmii->dwTypeData)
                                    : g_malloc0 (sizeof (gchar));
                }
              if (utf8str)
                {
                  gint i;
                  GList *glist;
                  GtkWidget *label = NULL;

                  glist = gtk_container_get_children
                                                (GTK_CONTAINER (menu_item));
                  for (i = 0; i < g_list_length (glist); i++)
                    if (GTK_IS_LABEL (g_list_nth_data (glist, i)))
                      {
                        label = g_list_nth_data (glist, i);
                        break;
                      }
                  g_list_free (glist);
                  if (label)
                    {
                      gchar *title;
                      gint j = 0;

                      title = g_malloc ((g_strlen (utf8str) * 2 + 1)
                                                            * sizeof (gchar));
                      for (i = 0; utf8str[i] != '\0'; i++)
                        switch (utf8str[i])
                          {
                            case '_': title[j++] = '_';
                            case '&': title[j++] = '_'; break;
                            default:  title[j++] = utf8str[i];
                          }
                      g_free (utf8str);
                      for (i = 0; i < j; i++)
                        if (title[i] == '\t')
                          break;
                      title[j] = '\0';
                      gtk_label_set_text_with_mnemonic (GTK_LABEL (label),
                                                                        title);
                      g_free (title);
                    }
                }
            }
          return TRUE;
        }
    }
  return FALSE;
}


/* SetMenuItemInfoW ordinal = 577, argument = 16 */
BOOL WINAPI SetMenuItemInfoW (HMENU hMenu, UINT uItem, BOOL fByPosition, LPMENUITEMINFOW lpmii)
{
  if (hMenu && lpmii
                && lpmii->cbSize >= sizeof (MENUITEMINFOW) - sizeof (HBITMAP))
    {
      GtkWidget *menu_item;

      menu_item = w32ldr_menu_get_item (hMenu, uItem,
                                fByPosition ? MF_BYPOSITION : MF_BYCOMMAND);
      if (menu_item)
        {
          GtkWidget *menu_item_new = NULL;

          if (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE)
                                            && lpmii->fType & MFT_SEPARATOR)
            {
              if (!GTK_IS_SEPARATOR_MENU_ITEM (menu_item))
                menu_item_new = gtk_separator_menu_item_new ();
            }
          else
            {
              if (!GTK_IS_IMAGE_MENU_ITEM (menu_item))
                menu_item_new = gtk_image_menu_item_new_with_label ("");
            }
          if (menu_item_new)
            {
              GtkWidget *menu;

              menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu_item));
              if (menu)
                g_object_ref (G_OBJECT (menu));
              g_object_set_data (G_OBJECT (menu_item_new), "user_data",
                        g_object_get_data (G_OBJECT (menu_item), "user_data"));
              gtk_menu_shell_insert (GTK_MENU_SHELL (hMenu), menu_item,
                fByPosition ? uItem : w32ldr_menu_get_item_pos (hMenu, uItem));
              gtk_container_remove (GTK_CONTAINER (hMenu), menu_item);
              if (menu)
                gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item_new),
                                                                        menu);
              menu_item = menu_item_new;
            }
          if (lpmii->fMask & MIIM_STATE)
            {
              gtk_widget_set_sensitive (menu_item, lpmii->fState & MFS_GRAYED);
              if (GTK_IS_IMAGE_MENU_ITEM (menu_item))
                {
                  GtkWidget *image = NULL;

                  if (lpmii->fState & MFS_CHECKED)
                    {
                      GdkPixbuf *pixbuf;

                      pixbuf = lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE)
                                            && lpmii->fType & MFT_RADIOCHECK
                                            ? w32ldr_menu_get_pixbuf_radio ()
                                            : w32ldr_menu_get_pixbuf_check ();
                      if (pixbuf)
                        image = gtk_image_new_from_pixbuf (pixbuf);
                    }
                  if (!(lpmii->fState & MFS_CHECKED) || image)
                    gtk_image_menu_item_set_image
                                    (GTK_IMAGE_MENU_ITEM (menu_item), image);
                }
            }
          if (lpmii->fMask & MIIM_ID)
            g_object_set_data (G_OBJECT (menu_item), "user_data",
                                                GUINT_TO_POINTER (lpmii->wID));
          if (lpmii->fMask & MIIM_SUBMENU)
            gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
                                                            lpmii->hSubMenu);
          if (GTK_IS_IMAGE_MENU_ITEM (menu_item))
            {
              gchar *utf8str = NULL;
              GtkWidget *image = NULL;

              if (lpmii->fMask & MIIM_BITMAP
                                    && lpmii->cbSize >= sizeof (MENUITEMINFOW))
                {
                  image = gtk_image_new_from_pixbuf (lpmii->hbmpItem);
                  g_object_unref (G_OBJECT (lpmii->hbmpItem));
                }
              else if (lpmii->fMask & (MIIM_TYPE | MIIM_FTYPE)
                                                && lpmii->fType & MFT_BITMAP)
                {
                  image = gtk_image_new_from_pixbuf
                                            ((GdkPixbuf *)lpmii->dwTypeData);
                  g_object_unref (G_OBJECT (lpmii->dwTypeData));
                }
              if (image)
                {
                  gtk_image_menu_item_set_image
                                    (GTK_IMAGE_MENU_ITEM (menu_item), image);
                  utf8str = g_malloc0 (sizeof (gchar));
                }
              else if (lpmii->fMask & MIIM_STRING || lpmii->fMask & MIIM_TYPE)
                {
                  utf8str = lpmii->dwTypeData
                    ? g_utf16_to_utf8 (lpmii->dwTypeData, -1, NULL, NULL, NULL)
                                                            : g_strdup ("");
                }
              if (utf8str)
                {
                  gint i;
                  GList *glist;
                  GtkWidget *label = NULL;

                  glist = gtk_container_get_children
                                                (GTK_CONTAINER (menu_item));
                  for (i = 0; i < g_list_length (glist); i++)
                    if (GTK_IS_LABEL (g_list_nth_data (glist, i)))
                      {
                        label = g_list_nth_data (glist, i);
                        break;
                      }
                  g_list_free (glist);
                  if (label)
                    {
                      gchar *title;
                      gint j = 0;

                      title = g_malloc ((g_strlen (utf8str) * 2 + 1)
                                                            * sizeof (gchar));
                      for (i = 0; utf8str[i] != '\0'; i++)
                        switch (utf8str[i])
                          {
                            case '_': title[j++] = '_';
                            case '&': title[j++] = '_'; break;
                            default:  title[j++] = utf8str[i];
                          }
                      g_free (utf8str);
                      for (i = 0; i < j; i++)
                        if (title[i] == '\t')
                          break;
                      title[j] = '\0';
                      gtk_label_set_text_with_mnemonic (GTK_LABEL (label),
                                                                        title);
                      g_free (title);
                    }
                }
            }
          return TRUE;
        }
    }
  return FALSE;
}


/* SetMessageExtraInfo ordinal = 578, argument = 4 temporary */
static DWORD WINAPI SetMessageExtraInfo (DWORD dwArgv0)
{
  return 0;
}


/* SetMessageQueue ordinal = 579, argument = 4 temporary */
static DWORD WINAPI SetMessageQueue (DWORD dwArgv0)
{
  return 0;
}


/* SetParent ordinal = 580, argument = 8 temporary */
static DWORD WINAPI SetParent (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetProcessDefaultLayout ordinal = 581, argument = 4 temporary */
static DWORD WINAPI SetProcessDefaultLayout (DWORD dwArgv0)
{
  return 0;
}


/* SetProcessWindowStation ordinal = 582, argument = 4 temporary */
static DWORD WINAPI SetProcessWindowStation (DWORD dwArgv0)
{
  return 0;
}


/* SetProgmanWindow ordinal = 583, argument = 0 temporary */
static DWORD WINAPI SetProgmanWindow (VOID)
{
  return 0;
}


/* SetPropA ordinal = 584, argument = 12 temporary */
static DWORD WINAPI SetPropA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetPropW ordinal = 585, argument = 12 temporary */
static DWORD WINAPI SetPropW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetRect ordinal = 586, argument = 20 */
BOOL WINAPI SetRect (LPRECT lprc, INT xLeft, INT yTop, INT xRight, INT yBottom)
{
  if (!lprc)
    return FALSE;
  lprc->left = xLeft;
  lprc->top = yTop;
  lprc->right = xRight;
  lprc->bottom = yBottom;
  return TRUE;
}


/* SetRectEmpty ordinal = 587, argument = 4 */
BOOL WINAPI SetRectEmpty (LPRECT lprc)
{
  if (!lprc)
    return FALSE;
  lprc->left = lprc->top = lprc->right = lprc->bottom = 0;
  return TRUE;
}


/* SetScrollInfo ordinal = 588, argument = 16 temporary */
static DWORD WINAPI SetScrollInfo (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetScrollPos ordinal = 589, argument = 16 */
INT WINAPI SetScrollPos (HWND hWnd, INT nBar, INT nPos, BOOL bRedraw)
{
  return hWnd && nBar == SB_CTL ? IsWindowUnicode (hWnd)
                        ? SendMessageW (hWnd, SBM_SETPOS, nPos, bRedraw)
                        : SendMessageA (hWnd, SBM_SETPOS, nPos, bRedraw) : 0;
}


/* SetScrollRange ordinal = 590, argument = 20 */
BOOL WINAPI SetScrollRange (HWND hWnd, INT nBar, INT nMinPos, INT nMaxPos, BOOL bRedraw)
{
  if (!hWnd || nBar != SB_CTL)
    return FALSE;
  if (IsWindowUnicode (hWnd))
    SendMessageW (hWnd, bRedraw ? SBM_SETRANGEREDRAW : SBM_SETRANGE,
                                                            nMinPos, nMaxPos);
  else
    SendMessageA (hWnd, bRedraw ? SBM_SETRANGEREDRAW : SBM_SETRANGE,
                                                            nMinPos, nMaxPos);
  return TRUE;
}


/* SetShellWindow ordinal = 591, argument = 4 temporary */
static DWORD WINAPI SetShellWindow (DWORD dwArgv0)
{
  return 0;
}


/* SetShellWindowEx ordinal = 592, argument = 0 temporary */
static DWORD WINAPI SetShellWindowEx (VOID)
{
  return 0;
}


/* SetSysColors ordinal = 593, argument = 12 temporary */
static DWORD WINAPI SetSysColors (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetSysColorsTemp ordinal = 594, argument = 0 temporary */
static DWORD WINAPI SetSysColorsTemp (VOID)
{
  return 0;
}


/* SetSystemCursor ordinal = 595, argument = 8 temporary */
static DWORD WINAPI SetSystemCursor (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetSystemMenu ordinal = 596, argument = 0 temporary */
static DWORD WINAPI SetSystemMenu (VOID)
{
  return 0;
}


/* SetSystemTimer ordinal = 597, argument = 0 temporary */
static DWORD WINAPI SetSystemTimer (VOID)
{
  return 0;
}


/* SetTaskmanWindow ordinal = 598, argument = 0 temporary */
static DWORD WINAPI SetTaskmanWindow (VOID)
{
  return 0;
}


/* SetThreadDesktop ordinal = 599, argument = 4 temporary */
static DWORD WINAPI SetThreadDesktop (DWORD dwArgv0)
{
  return 0;
}


static gboolean
w32ldr_SetTimer (W32LdrTimer *timer)
{
  if (timer->lpTimerFunc)
    {
      timer->lpTimerFunc (timer->hWnd, WM_TIMER,
                                    GPOINTER_TO_UINT (timer), GetTickCount ());
    }
  else if (timer->hWnd)
    {
      if (IsWindowUnicode (timer->hWnd))
        SendMessageW (timer->hWnd, WM_TIMER, timer->uIDEvent, 0);
      else
        SendMessageA (timer->hWnd, WM_TIMER, timer->uIDEvent, 0);
    }
  return TRUE;
}


/* SetTimer ordinal = 600, argument = 16 */
UINT WINAPI SetTimer (HWND hWnd, UINT uIDEvent, UINT uElapse, TIMERPROC lpTimerFunc)
{
  W32LdrTimer *timer;

  if (!hWnd && !lpTimerFunc)
    return 0;
  timer = g_malloc (sizeof (W32LdrTimer));
  timer->hWnd = hWnd;
  timer->uIDEvent = uIDEvent;
  timer->lpTimerFunc = lpTimerFunc;
  timer->id = g_timeout_add (uElapse, (GSourceFunc)w32ldr_SetTimer, timer);
  if (timer->id == 0)
    {
      g_free (timer);
      timer = NULL;
    }
  if (timer)
    glist_timer = g_list_append (glist_timer, timer);
  return GPOINTER_TO_UINT (timer);
}


/* SetUserObjectInformationA ordinal = 601, argument = 16 temporary */
static DWORD WINAPI SetUserObjectInformationA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetUserObjectInformationW ordinal = 602, argument = 16 temporary */
static DWORD WINAPI SetUserObjectInformationW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetUserObjectSecurity ordinal = 603, argument = 12 temporary */
static DWORD WINAPI SetUserObjectSecurity (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetWinEventHook ordinal = 604, argument = 28 temporary */
static DWORD WINAPI SetWinEventHook (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* SetWindowContextHelpId ordinal = 605, argument = 8 temporary */
static DWORD WINAPI SetWindowContextHelpId (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetWindowLongA ordinal = 606, argument = 12 */
LONG WINAPI SetWindowLongA (HWND hWnd, INT nIndex, LONG dwNewLong)
{
  return SetWindowLongW (hWnd, nIndex, dwNewLong);
}


/* SetWindowLongW ordinal = 607, argument = 12 */
LONG WINAPI SetWindowLongW (HWND hWnd, INT nIndex, LONG dwNewLong)
{
  LONG lOld;
  W32LdrWindowData *wd;

  lOld = GetWindowLongA (hWnd, nIndex);
  wd = hWnd ? g_object_get_data (G_OBJECT (hWnd), "user_data") : NULL;
  if (wd)
    switch (nIndex)
      {
        case GWL_USERDATA:
          wd->dwUserData = dwNewLong;
          break;
        case GWL_EXSTYLE:
          wd->dwExStyle = dwNewLong;
          break;
        case GWL_STYLE:
          wd->dwStyle = dwNewLong;
          break;
        case GWL_ID:
          wd->wID = dwNewLong;
          break;
        case GWL_HWNDPARENT:
          wd->hWndParent = GINT_TO_POINTER (dwNewLong);
          break;
        case GWL_HINSTANCE:
          wd->hInstance = GINT_TO_POINTER (dwNewLong);
          break;
        case GWL_WNDPROC:
          wd->lpfnWndProc = GINT_TO_POINTER (dwNewLong);
          break;
        case DWL_MSGRESULT:
          wd->lResult = dwNewLong;
          break;
        case DWL_DLGPROC:
          wd->lpDialogFunc = GINT_TO_POINTER (dwNewLong);
          break;
        case DWL_USER:
          wd->dwUser = dwNewLong;
      }
  return lOld;
}


/* SetWindowPlacement ordinal = 608, argument = 8 temporary */
static DWORD WINAPI SetWindowPlacement (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetWindowPos ordinal = 609, argument = 28 */
BOOL WINAPI SetWindowPos (HWND hWnd, HWND hWndInsertAfter, INT x, INT y, INT cx, INT cy, UINT uFlags)
{
  if (!hWnd)
    return FALSE;
  if (GTK_IS_WINDOW (hWnd))
    {
      if (!(uFlags & SWP_NOSIZE))
        gtk_window_resize (GTK_WINDOW (hWnd), cx, cy);
      if (!(uFlags & SWP_NOMOVE))
        gtk_window_move (GTK_WINDOW (hWnd), x, y);
    }
  else
    {
      W32LdrWindowData *wd;

      wd = g_object_get_data (G_OBJECT (hWnd), "user_data");
      if (wd->hWndParent)
        {
          GtkWidget *fixed = NULL;

          if (GTK_IS_FIXED (wd->hWndParent))
            {
              fixed = wd->hWndParent;
            }
          else if (GTK_IS_WINDOW (wd->hWndParent))
            {
              W32LdrOverlapData *od;

              od = g_object_get_data (G_OBJECT (wd->hWndParent), "user_data");
              fixed = od->fixed;
            }
          if (fixed)
            {
              if (wd->resizable && !(uFlags & SWP_NOSIZE))
                {
                  gtk_widget_set_size_request (hWnd, cx, cy);
                  gtk_widget_queue_resize (hWnd);
                }
              if (!(uFlags & SWP_NOMOVE))
                gtk_fixed_move (GTK_FIXED (fixed), hWnd, x, y);
            }
        }
    }
  if (uFlags & SWP_SHOWWINDOW)
    ShowWindow (hWnd, SW_SHOW);
  else if (uFlags & SWP_HIDEWINDOW)
    ShowWindow (hWnd, SW_HIDE);
  if (!(uFlags & SWP_NOREDRAW))
    gtk_widget_queue_draw (hWnd);
  return TRUE;
}


/* SetWindowRgn ordinal = 610, argument = 12 temporary */
static DWORD WINAPI SetWindowRgn (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetWindowStationUser ordinal = 611, argument = 0 temporary */
static DWORD WINAPI SetWindowStationUser (VOID)
{
  return 0;
}


/* SetWindowTextA ordinal = 612, argument = 8 */
BOOL WINAPI SetWindowTextA (HWND hWnd, LPCSTR lpString)
{
  return SendMessageA (hWnd, WM_SETTEXT, 0, GPOINTER_TO_INT (lpString));
}


/* SetWindowTextW ordinal = 613, argument = 8 */
BOOL WINAPI SetWindowTextW (HWND hWnd, LPCWSTR lpString)
{
  return SendMessageW (hWnd, WM_SETTEXT, 0, GPOINTER_TO_INT (lpString));
}


/* SetWindowWord ordinal = 614, argument = 12 temporary */
static DWORD WINAPI SetWindowWord (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* SetWindowsHookA ordinal = 615, argument = 8 temporary */
static DWORD WINAPI SetWindowsHookA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SetWindowsHookExA ordinal = 616, argument = 16 temporary */
static DWORD WINAPI SetWindowsHookExA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetWindowsHookExW ordinal = 617, argument = 16 temporary */
static DWORD WINAPI SetWindowsHookExW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* SetWindowsHookW ordinal = 618, argument = 8 temporary */
static DWORD WINAPI SetWindowsHookW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ShowCaret ordinal = 619, argument = 4 temporary */
static DWORD WINAPI ShowCaret (DWORD dwArgv0)
{
  return 0;
}


/* ShowCursor ordinal = 620, argument = 4 temporary */
static DWORD WINAPI ShowCursor (DWORD dwArgv0)
{
  return 0;
}


/* ShowOwnedPopups ordinal = 621, argument = 8 temporary */
static DWORD WINAPI ShowOwnedPopups (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ShowScrollBar ordinal = 622, argument = 12 temporary */
static DWORD WINAPI ShowScrollBar (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* ShowStartGlass ordinal = 623, argument = 0 temporary */
static DWORD WINAPI ShowStartGlass (VOID)
{
  return 0;
}


/* ShowWindow ordinal = 624, argument = 8 */
BOOL WINAPI ShowWindow (HWND hWnd, INT nCmdShow)
{
  if (!hWnd)
    return FALSE;
  if (IsWindowUnicode (hWnd))
    SendMessageW (hWnd, WM_SHOWWINDOW, nCmdShow != SW_HIDE, 0);
  else
    SendMessageA (hWnd, WM_SHOWWINDOW, nCmdShow != SW_HIDE, 0);
  return TRUE;
}


/* ShowWindowAsync ordinal = 625, argument = 8 temporary */
static DWORD WINAPI ShowWindowAsync (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SoftModalMessageBox ordinal = 626, argument = 0 temporary */
static DWORD WINAPI SoftModalMessageBox (VOID)
{
  return 0;
}


/* SubtractRect ordinal = 627, argument = 12 */
BOOL WINAPI SubtractRect (LPRECT lprcDst, CONST LPRECT lprcSrc1, CONST LPRECT lprcSrc2)
{
  RECT rcTmp;

  if (!lprcDst)
    return FALSE;
  if (IsRectEmpty (lprcSrc1))
    {
      SetRectEmpty (lprcDst);
      return FALSE;
    }
  *lprcDst = *lprcSrc1;
  if (IntersectRect (&rcTmp, lprcSrc1, lprcSrc2))
    {
      if (EqualRect (lprcDst, &rcTmp))
        {
          SetRectEmpty (lprcDst);
          return FALSE;
        }
      if (lprcDst->top == rcTmp.top && lprcDst->bottom == rcTmp.bottom)
        {
          if (lprcDst->left == rcTmp.left)
            lprcDst->left = rcTmp.right;
          else if (lprcDst->right == rcTmp.right)
            lprcDst->right = rcTmp.left;
        }
      else if (lprcDst->left == rcTmp.left && lprcDst->right == rcTmp.right)
        {
          if (lprcDst->top == rcTmp.top)
            lprcDst->top = rcTmp.bottom;
          else if (lprcDst->bottom == rcTmp.bottom)
            lprcDst->bottom = rcTmp.top;
        }
    }
  return TRUE;
}


/* SwapMouseButton ordinal = 628, argument = 4 temporary */
static DWORD WINAPI SwapMouseButton (DWORD dwArgv0)
{
  return 0;
}


/* SwitchDesktop ordinal = 629, argument = 4 temporary */
static DWORD WINAPI SwitchDesktop (DWORD dwArgv0)
{
  return 0;
}


/* SwitchToThisWindow ordinal = 630, argument = 8 temporary */
static DWORD WINAPI SwitchToThisWindow (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* SystemParametersInfoA ordinal = 631, argument = 16 */
BOOL WINAPI SystemParametersInfoA (UINT uiAction, UINT uiParam, PVOID pvParam, UINT fWinIni)
{
  return SystemParametersInfoW (uiAction, uiParam, pvParam, fWinIni);
}


/* SystemParametersInfoW ordinal = 632, argument = 16 */
BOOL WINAPI SystemParametersInfoW (UINT uiAction, UINT uiParam, PVOID pvParam, UINT fWinIni)
{
  if (uiAction == SPI_GETWORKAREA && pvParam)
    {
      gint x, y, width, height, depth;
      LPRECT lprcWorkArea;

      gdk_window_get_geometry (gdk_get_default_root_window (),
                                            &x, &y, &width, &height, &depth);
      lprcWorkArea = pvParam;
      lprcWorkArea->left = x;
      lprcWorkArea->top = y;
      lprcWorkArea->right = x + width - 1;
      lprcWorkArea->bottom = y + height - 1;
      return TRUE;
    }
  return FALSE;
}


/* TabbedTextOutA ordinal = 633, argument = 32 temporary */
static DWORD WINAPI TabbedTextOutA (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* TabbedTextOutW ordinal = 634, argument = 32 temporary */
static DWORD WINAPI TabbedTextOutW (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7)
{
  return 0;
}


/* TileChildWindows ordinal = 635, argument = 8 temporary */
static DWORD WINAPI TileChildWindows (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* TileWindows ordinal = 636, argument = 20 temporary */
static DWORD WINAPI TileWindows (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ToAscii ordinal = 637, argument = 20 temporary */
static DWORD WINAPI ToAscii (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* ToAsciiEx ordinal = 638, argument = 24 temporary */
static DWORD WINAPI ToAsciiEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* ToUnicode ordinal = 639, argument = 24 temporary */
static DWORD WINAPI ToUnicode (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5)
{
  return 0;
}


/* ToUnicodeEx ordinal = 640, argument = 28 temporary */
static DWORD WINAPI ToUnicodeEx (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6)
{
  return 0;
}


/* TrackMouseEvent ordinal = 641, argument = 4 temporary */
static DWORD WINAPI TrackMouseEvent (DWORD dwArgv0)
{
  return 0;
}


/* TrackPopupMenu ordinal = 642, argument = 28 */
BOOL WINAPI TrackPopupMenu (HMENU hMenu, UINT uFlags, INT x, INT y, INT nReserved, HWND hWnd, LPCRECT lprcRect)
{
  gtk_menu_popup (GTK_MENU (hMenu), NULL, NULL, NULL, NULL,
                                        uFlags & TPM_RIGHTBUTTON ? 3 : 1, 0);
  return uFlags & TPM_RETURNCMD ? 0 : TRUE;
}


/* TrackPopupMenuEx ordinal = 643, argument = 24 */
BOOL WINAPI TrackPopupMenuEx (HMENU hMenu, UINT uFlags, INT x, INT y, HWND hWnd, LPTPMPARAMS lptpm)
{
  gtk_menu_popup (GTK_MENU (hMenu), NULL, NULL, NULL, NULL,
                                        uFlags & TPM_RIGHTBUTTON ? 3 : 1, 0);
  return uFlags & TPM_RETURNCMD ? 0 : TRUE;
}


/* TranslateAccelerator ordinal = 644, argument = 12 */
/* INT WINAPI TranslateAccelerator (HWND hWnd, HACCEL hAccel, LPMSG lpMsg) */


/* TranslateAcceleratorA ordinal = 645, argument = 12 */
INT WINAPI TranslateAcceleratorA (HWND hWnd, HACCEL hAccel, LPMSG lpMsg)
{
  return 0;
}


/* TranslateAcceleratorW ordinal = 646, argument = 12 */
INT WINAPI TranslateAcceleratorW (HWND hWnd, HACCEL hAccel, LPMSG lpMsg)
{
  return 0;
}


/* TranslateMDISysAccel ordinal = 647, argument = 8 temporary */
static DWORD WINAPI TranslateMDISysAccel (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* TranslateMessage ordinal = 648, argument = 4 */
BOOL WINAPI TranslateMessage (CONST MSG *lpMsg)
{
  return lpMsg && (lpMsg->message == WM_KEYUP || lpMsg->message == WM_KEYDOWN
        || lpMsg->message == WM_SYSKEYUP || lpMsg->message == WM_SYSKEYDOWN);
}


/* TranslateMessageEx ordinal = 649, argument = 0 temporary */
static DWORD WINAPI TranslateMessageEx (VOID)
{
  return 0;
}


/* UnhookWinEvent ordinal = 650, argument = 4 temporary */
static DWORD WINAPI UnhookWinEvent (DWORD dwArgv0)
{
  return 0;
}


/* UnhookWindowsHook ordinal = 651, argument = 8 temporary */
static DWORD WINAPI UnhookWindowsHook (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* UnhookWindowsHookEx ordinal = 652, argument = 4 temporary */
static DWORD WINAPI UnhookWindowsHookEx (DWORD dwArgv0)
{
  return 0;
}


/* UnionRect ordinal = 653, argument = 12 */
BOOL WINAPI UnionRect (LPRECT lprcDst, CONST LPRECT lprcSrc1, CONST LPRECT lprcSrc2)
{
  if (!lprcDst)
    return FALSE;
  if (IsRectEmpty (lprcSrc1))
    {
      if (IsRectEmpty (lprcSrc2))
        {
          SetRectEmpty (lprcDst);
          return FALSE;
        }
      *lprcDst = *lprcSrc2;
    }
  else if (IsRectEmpty (lprcSrc2))
    {
      *lprcDst = *lprcSrc1;
    }
  else
    {
      lprcDst->left = MIN (lprcSrc1->left, lprcSrc2->left);
      lprcDst->top = MIN (lprcSrc1->top, lprcSrc2->top);
      lprcDst->right = MAX (lprcSrc1->right, lprcSrc2->right);
      lprcDst->bottom = MAX (lprcSrc1->bottom, lprcSrc2->bottom);
    }
  return TRUE;
}


/* UnloadKeyboardLayout ordinal = 654, argument = 4 temporary */
static DWORD WINAPI UnloadKeyboardLayout (DWORD dwArgv0)
{
  return 0;
}


/* UnlockWindowStation ordinal = 655, argument = 0 temporary */
static DWORD WINAPI UnlockWindowStation (VOID)
{
  return 0;
}


/* UnpackDDElParam ordinal = 656, argument = 16 temporary */
static DWORD WINAPI UnpackDDElParam (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* UnregisterClassA ordinal = 657, argument = 8 */
BOOL WINAPI UnregisterClassA (LPCSTR lpClassName, HINSTANCE hInstance)
{
  guint16 atom;
  W32LdrClassData *w32class;

  atom = w32ldr_atom_find_name_mb (lpClassName);
  if (atom > 0 && ghash_class && (w32class = g_hash_table_lookup (ghash_class,
                                            GUINT_TO_POINTER ((guint)atom))))
    {
      g_hash_table_remove (ghash_class, GUINT_TO_POINTER ((guint)atom));
      if (g_hash_table_size (ghash_class) <= 0)
        {
          g_hash_table_destroy (ghash_class);
          ghash_class = NULL;
        }
      g_free (w32class->classname);
      if (GPOINTER_TO_UINT (w32class->menuname) > G_MAXUINT16)
        g_free (w32class->menuname);
      g_free (w32class);
      return w32ldr_atom_delete_atom (atom);
    }
  return FALSE;
}


/* UnregisterClassW ordinal = 658, argument = 8 */
BOOL WINAPI UnregisterClassW (LPCWSTR lpClassName, HINSTANCE hInstance)
{
  guint16 atom;
  W32LdrClassData *w32class;

  atom = w32ldr_atom_find_name_wc (lpClassName);
  if (atom > 0 && ghash_class && (w32class = g_hash_table_lookup (ghash_class,
                                            GUINT_TO_POINTER ((guint)atom))))
    {
      g_hash_table_remove (ghash_class, GUINT_TO_POINTER ((guint)atom));
      if (g_hash_table_size (ghash_class) <= 0)
        {
          g_hash_table_destroy (ghash_class);
          ghash_class = NULL;
        }
      g_free (w32class->classname);
      if (GPOINTER_TO_UINT (w32class->menuname) > G_MAXUINT16)
        g_free (w32class->menuname);
      g_free (w32class);
      return w32ldr_atom_delete_atom (atom);
    }
  return FALSE;
}


/* UnregisterDeviceNotification ordinal = 659, argument = 4 temporary */
static DWORD WINAPI UnregisterDeviceNotification (DWORD dwArgv0)
{
  return 0;
}


/* UnregisterHotKey ordinal = 660, argument = 8 temporary */
static DWORD WINAPI UnregisterHotKey (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* UpdateLayeredWindow ordinal = 661, argument = 36 temporary */
static DWORD WINAPI UpdateLayeredWindow (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4, DWORD dwArgv5, DWORD dwArgv6, DWORD dwArgv7, DWORD dwArgv8)
{
  return 0;
}


/* UpdatePerUserSystemParameters ordinal = 662, argument = 0 temporary */
static DWORD WINAPI UpdatePerUserSystemParameters (VOID)
{
  return 0;
}


/* UpdateWindow ordinal = 663, argument = 4 */
BOOL WINAPI UpdateWindow (HWND hWnd)
{
  if (!hWnd || !hWnd->window)
    return FALSE;
  gdk_window_process_updates (hWnd->window, TRUE);
  return TRUE;
}


/* User32InitializeImmEntryTable ordinal = 664, argument = 0 temporary */
static DWORD WINAPI User32InitializeImmEntryTable (VOID)
{
  return 0;
}


/* UserClientDllInitialize ordinal = 665, argument = 0 temporary */
static DWORD WINAPI UserClientDllInitialize (VOID)
{
  return 0;
}


/* UserHandleGrantAccess ordinal = 666, argument = 12 temporary */
static DWORD WINAPI UserHandleGrantAccess (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2)
{
  return 0;
}


/* UserLpkPSMTextOut ordinal = 667, argument = 0 temporary */
static DWORD WINAPI UserLpkPSMTextOut (VOID)
{
  return 0;
}


/* UserLpkTabbedTextOut ordinal = 668, argument = 0 temporary */
static DWORD WINAPI UserLpkTabbedTextOut (VOID)
{
  return 0;
}


/* UserRealizePalette ordinal = 669, argument = 0 temporary */
static DWORD WINAPI UserRealizePalette (VOID)
{
  return 0;
}


/* UserRegisterWowHandlers ordinal = 670, argument = 0 temporary */
static DWORD WINAPI UserRegisterWowHandlers (VOID)
{
  return 0;
}


/* VRipOutput ordinal = 671, argument = 0 temporary */
static DWORD WINAPI VRipOutput (VOID)
{
  return 0;
}


/* VTagOutput ordinal = 672, argument = 0 temporary */
static DWORD WINAPI VTagOutput (VOID)
{
  return 0;
}


/* ValidateRect ordinal = 673, argument = 8 temporary */
static DWORD WINAPI ValidateRect (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* ValidateRgn ordinal = 674, argument = 8 temporary */
static DWORD WINAPI ValidateRgn (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* VkKeyScanA ordinal = 675, argument = 4 temporary */
static DWORD WINAPI VkKeyScanA (DWORD dwArgv0)
{
  return 0;
}


/* VkKeyScanExA ordinal = 676, argument = 8 temporary */
static DWORD WINAPI VkKeyScanExA (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* VkKeyScanExW ordinal = 677, argument = 8 temporary */
static DWORD WINAPI VkKeyScanExW (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* VkKeyScanW ordinal = 678, argument = 4 temporary */
static DWORD WINAPI VkKeyScanW (DWORD dwArgv0)
{
  return 0;
}


/* WCSToMBEx ordinal = 679, argument = 0 temporary */
static DWORD WINAPI WCSToMBEx (VOID)
{
  return 0;
}


/* WINNLSEnableIME ordinal = 680, argument = 8 temporary */
static DWORD WINAPI WINNLSEnableIME (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* WINNLSGetEnableStatus ordinal = 681, argument = 4 temporary */
static DWORD WINAPI WINNLSGetEnableStatus (DWORD dwArgv0)
{
  return 0;
}


/* WINNLSGetIMEHotkey ordinal = 682, argument = 4 temporary */
static DWORD WINAPI WINNLSGetIMEHotkey (DWORD dwArgv0)
{
  return 0;
}


/* WaitForInputIdle ordinal = 683, argument = 8 temporary */
static DWORD WINAPI WaitForInputIdle (DWORD dwArgv0, DWORD dwArgv1)
{
  return 0;
}


/* WaitMessage ordinal = 684, argument = 0 temporary */
static DWORD WINAPI WaitMessage (VOID)
{
  return 0;
}


/* Win32PoolAllocationStats ordinal = 685, argument = 0 temporary */
static DWORD WINAPI Win32PoolAllocationStats (VOID)
{
  return 0;
}


/* WinHelpA ordinal = 686, argument = 16 */
BOOL WINAPI WinHelpA (HWND hWndMain, LPCSTR lpszHelp, UINT uCommand, DWORD dwData)
{
  return FALSE;
}


/* WinHelpW ordinal = 687, argument = 16 */
BOOL WINAPI WinHelpW (HWND hWndMain, LPCWSTR lpszHelp, UINT uCommand, DWORD dwData)
{
  return FALSE;
}


/* WindowFromDC ordinal = 688, argument = 4 temporary */
static DWORD WINAPI WindowFromDC (DWORD dwArgv0)
{
  return 0;
}


/* WindowFromPoint ordinal = 689, argument = 8 */
HWND WINAPI WindowFromPoint (POINT Point)
{
  return NULL;
}


/* keybd_event ordinal = 690, argument = 16 temporary */
static DWORD WINAPI keybd_event (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3)
{
  return 0;
}


/* mouse_event ordinal = 691, argument = 20 temporary */
static DWORD WINAPI mouse_event (DWORD dwArgv0, DWORD dwArgv1, DWORD dwArgv2, DWORD dwArgv3, DWORD dwArgv4)
{
  return 0;
}


/* wsprintfA ordinal = 692, argument = 0 */
INT WINAPIV wsprintfA (LPSTR lpOutput, LPCSTR lpFormat, ...)
{
#if GLIB_CHECK_VERSION(2,2,0) || defined (HAVE_VPRINTF)
  va_list arglist;

  va_start (arglist, lpFormat);
# if GLIB_CHECK_VERSION(2,2,0)
  g_vsprintf (lpOutput, lpFormat, arglist);
# else /* not GLIB_CHECK_VERSION(2,2,0) */
  vsprintf (lpOutput, lpFormat, arglist);
# endif /* not GLIB_CHECK_VERSION(2,2,0) */
  va_end (arglist);
#else /* not GLIB_CHECK_VERSION(2,2,0) || defined (HAVE_VPRINTF) */
  if (lpOutput)
    *lpOutput = '\0';
#endif /* not GLIB_CHECK_VERSION(2,2,0) || defined (HAVE_VPRINTF) */
  return g_strlen (lpOutput);
}


/* wsprintfW ordinal = 693, argument = 0 */
INT WINAPIV wsprintfW (LPWSTR lpOutput, LPCWSTR lpFormat, ...)
{
  if (lpOutput)
    *lpOutput = '\0';
  return 0;
}


/* wvsprintfA ordinal = 694, argument = 12 */
INT WINAPI wvsprintfA (LPSTR lpOutput, LPCSTR lpFormat, va_list arglist)
{
#if GLIB_CHECK_VERSION(2,2,0)
  return g_vsprintf (lpOutput, lpFormat, arglist);
#elif defined HAVE_VPRINTF
  return vsprintf (lpOutput, lpFormat, arglist);
#else /* not GLIB_CHECK_VERSION(2,2,0) || defined (HAVE_VPRINTF) */
  if (lpOutput)
    *lpOutput = '\0';
  return 0;
#endif /* not GLIB_CHECK_VERSION(2,2,0) || defined (HAVE_VPRINTF) */
}


/* wvsprintfW ordinal = 695, argument = 12 */
INT WINAPI wvsprintfW (LPSTR lpOutput, LPCSTR lpFormat, va_list arglist)
{
  if (lpOutput)
    *lpOutput = '\0';
  return 0;
}


W32LdrExport user32_exports[] = {
{FALSE,   1, "ActivateKeyboardLayout",        ActivateKeyboardLayout},
{FALSE,   2, "AdjustWindowRect",              AdjustWindowRect},
{FALSE,   3, "AdjustWindowRectEx",            AdjustWindowRectEx},
{FALSE,   4, "AlignRects",                    AlignRects},
{FALSE,   5, "AllowSetForegroundWindow",      AllowSetForegroundWindow},
{FALSE,   6, "AnimateWindow",                 AnimateWindow},
{FALSE,   7, "AnyPopup",                      AnyPopup},
{TRUE,    8, "AppendMenuA",                   AppendMenuA},
{TRUE,    9, "AppendMenuW",                   AppendMenuW},
{FALSE,  10, "ArrangeIconicWindows",          ArrangeIconicWindows},
{FALSE,  11, "AttachThreadInput",             AttachThreadInput},
{FALSE,  12, "BeginDeferWindowPos",           BeginDeferWindowPos},
{TRUE,   13, "BeginPaint",                    BeginPaint},
{FALSE,  14, "BlockInput",                    BlockInput},
{FALSE,  15, "BringWindowToTop",              BringWindowToTop},
{FALSE,  16, "BroadcastSystemMessage",        BroadcastSystemMessageA},
{FALSE,  17, "BroadcastSystemMessageA",       BroadcastSystemMessageA},
{FALSE,  18, "BroadcastSystemMessageW",       BroadcastSystemMessageW},
{FALSE,  19, "CallMsgFilter",                 CallMsgFilterA},
{FALSE,  20, "CallMsgFilterA",                CallMsgFilterA},
{FALSE,  21, "CallMsgFilterW",                CallMsgFilterW},
{FALSE,  22, "CallNextHookEx",                CallNextHookEx},
{TRUE,   23, "CallWindowProcA",               CallWindowProcA},
{TRUE,   24, "CallWindowProcW",               CallWindowProcW},
{FALSE,  25, "CascadeChildWindows",           CascadeChildWindows},
{FALSE,  26, "CascadeWindows",                CascadeWindows},
{FALSE,  27, "ChangeClipboardChain",          ChangeClipboardChain},
{FALSE,  28, "ChangeDisplaySettingsA",        ChangeDisplaySettingsA},
{FALSE,  29, "ChangeDisplaySettingsExA",      ChangeDisplaySettingsExA},
{FALSE,  30, "ChangeDisplaySettingsExW",      ChangeDisplaySettingsExW},
{FALSE,  31, "ChangeDisplaySettingsW",        ChangeDisplaySettingsW},
{FALSE,  32, "ChangeMenuA",                   ChangeMenuA},
{FALSE,  33, "ChangeMenuW",                   ChangeMenuW},
{TRUE,   34, "CharLowerA",                    CharLowerA},
{FALSE,  35, "CharLowerBuffA",                CharLowerBuffA},
{FALSE,  36, "CharLowerBuffW",                CharLowerBuffW},
{TRUE,   37, "CharLowerW",                    CharLowerW},
{FALSE,  38, "CharNextA",                     CharNextA},
{FALSE,  39, "CharNextExA",                   CharNextExA},
{FALSE,  40, "CharNextW",                     CharNextW},
{FALSE,  41, "CharPrevA",                     CharPrevA},
{FALSE,  42, "CharPrevExA",                   CharPrevExA},
{FALSE,  43, "CharPrevW",                     CharPrevW},
{FALSE,  44, "CharToOemA",                    CharToOemA},
{FALSE,  45, "CharToOemBuffA",                CharToOemBuffA},
{FALSE,  46, "CharToOemBuffW",                CharToOemBuffW},
{FALSE,  47, "CharToOemW",                    CharToOemW},
{TRUE,   48, "CharUpperA",                    CharUpperA},
{FALSE,  49, "CharUpperBuffA",                CharUpperBuffA},
{FALSE,  50, "CharUpperBuffW",                CharUpperBuffW},
{TRUE,   51, "CharUpperW",                    CharUpperW},
{TRUE,   52, "CheckDlgButton",                CheckDlgButton},
{TRUE,   53, "CheckMenuItem",                 CheckMenuItem},
{TRUE,   54, "CheckMenuRadioItem",            CheckMenuRadioItem},
{TRUE,   55, "CheckRadioButton",              CheckRadioButton},
{FALSE,  56, "ChildWindowFromPoint",          ChildWindowFromPoint},
{FALSE,  57, "ChildWindowFromPointEx",        ChildWindowFromPointEx},
{FALSE,  58, "CliImmSetHotKey",               CliImmSetHotKey},
{FALSE,  59, "ClientThreadSetup",             ClientThreadSetup},
{TRUE,   60, "ClientToScreen",                ClientToScreen},
{TRUE,   61, "ClipCursor",                    ClipCursor},
{TRUE,   62, "CloseClipboard",                CloseClipboard},
{FALSE,  63, "CloseDesktop",                  CloseDesktop},
{TRUE,   64, "CloseWindow",                   CloseWindow},
{FALSE,  65, "CloseWindowStation",            CloseWindowStation},
{FALSE,  66, "CopyAcceleratorTableA",         CopyAcceleratorTableA},
{FALSE,  67, "CopyAcceleratorTableW",         CopyAcceleratorTableW},
{TRUE,   68, "CopyIcon",                      CopyIcon},
{TRUE,   69, "CopyImage",                     CopyImage},
{TRUE,   70, "CopyRect",                      CopyRect},
{FALSE,  71, "CountClipboardFormats",         CountClipboardFormats},
{FALSE,  72, "CreateAcceleratorTableA",       CreateAcceleratorTableA},
{FALSE,  73, "CreateAcceleratorTableW",       CreateAcceleratorTableW},
{FALSE,  74, "CreateCaret",                   CreateCaret},
{FALSE,  75, "CreateCursor",                  CreateCursor},
{FALSE,  76, "CreateDesktopA",                CreateDesktopA},
{FALSE,  77, "CreateDesktopW",                CreateDesktopW},
{TRUE,   78, "CreateDialogIndirectParamA",    CreateDialogIndirectParamA},
{FALSE,  79, "CreateDialogIndirectParamAorW", CreateDialogIndirectParamAorW},
{TRUE,   80, "CreateDialogIndirectParamW",    CreateDialogIndirectParamW},
{TRUE,   81, "CreateDialogParamA",            CreateDialogParamA},
{TRUE,   82, "CreateDialogParamW",            CreateDialogParamW},
{FALSE,  83, "CreateIcon",                    CreateIcon},
{FALSE,  84, "CreateIconFromResource",        CreateIconFromResource},
{FALSE,  85, "CreateIconFromResourceEx",      CreateIconFromResourceEx},
{FALSE,  86, "CreateIconIndirect",            CreateIconIndirect},
{FALSE,  87, "CreateMDIWindowA",              CreateMDIWindowA},
{FALSE,  88, "CreateMDIWindowW",              CreateMDIWindowW},
{TRUE,   89, "CreateMenu",                    CreateMenu},
{TRUE,   90, "CreatePopupMenu",               CreatePopupMenu},
{TRUE,   91, "CreateWindowExA",               CreateWindowExA},
{TRUE,   92, "CreateWindowExW",               CreateWindowExW},
{FALSE,  93, "CreateWindowStationA",          CreateWindowStationA},
{FALSE,  94, "CreateWindowStationW",          CreateWindowStationW},
{FALSE,  95, "CtxInitUser32",                 CtxInitUser32},
{FALSE,  96, "DdeAbandonTransaction",         DdeAbandonTransaction},
{FALSE,  97, "DdeAccessData",                 DdeAccessData},
{FALSE,  98, "DdeAddData",                    DdeAddData},
{FALSE,  99, "DdeClientTransaction",          DdeClientTransaction},
{FALSE, 100, "DdeCmpStringHandles",           DdeCmpStringHandles},
{FALSE, 101, "DdeConnect",                    DdeConnect},
{FALSE, 102, "DdeConnectList",                DdeConnectList},
{FALSE, 103, "DdeCreateDataHandle",           DdeCreateDataHandle},
{FALSE, 104, "DdeCreateStringHandleA",        DdeCreateStringHandleA},
{FALSE, 105, "DdeCreateStringHandleW",        DdeCreateStringHandleW},
{FALSE, 106, "DdeDisconnect",                 DdeDisconnect},
{FALSE, 107, "DdeDisconnectList",             DdeDisconnectList},
{FALSE, 108, "DdeEnableCallback",             DdeEnableCallback},
{FALSE, 109, "DdeFreeDataHandle",             DdeFreeDataHandle},
{FALSE, 110, "DdeFreeStringHandle",           DdeFreeStringHandle},
{FALSE, 111, "DdeGetData",                    DdeGetData},
{FALSE, 112, "DdeGetLastError",               DdeGetLastError},
{FALSE, 113, "DdeGetQualityOfService",        DdeGetQualityOfService},
{FALSE, 114, "DdeImpersonateClient",          DdeImpersonateClient},
{FALSE, 115, "DdeInitializeA",                DdeInitializeA},
{FALSE, 116, "DdeInitializeW",                DdeInitializeW},
{FALSE, 117, "DdeKeepStringHandle",           DdeKeepStringHandle},
{FALSE, 118, "DdeNameService",                DdeNameService},
{FALSE, 119, "DdePostAdvise",                 DdePostAdvise},
{FALSE, 120, "DdeQueryConvInfo",              DdeQueryConvInfo},
{FALSE, 121, "DdeQueryNextServer",            DdeQueryNextServer},
{FALSE, 122, "DdeQueryStringA",               DdeQueryStringA},
{FALSE, 123, "DdeQueryStringW",               DdeQueryStringW},
{FALSE, 124, "DdeReconnect",                  DdeReconnect},
{FALSE, 125, "DdeSetQualityOfService",        DdeSetQualityOfService},
{FALSE, 126, "DdeSetUserHandle",              DdeSetUserHandle},
{FALSE, 127, "DdeUnaccessData",               DdeUnaccessData},
{FALSE, 128, "DdeUninitialize",               DdeUninitialize},
{FALSE, 129, "DefDlgProcA",                   DefDlgProcA},
{FALSE, 130, "DefDlgProcW",                   DefDlgProcW},
{FALSE, 131, "DefFrameProcA",                 DefFrameProcA},
{FALSE, 132, "DefFrameProcW",                 DefFrameProcW},
{FALSE, 133, "DefMDIChildProcA",              DefMDIChildProcA},
{FALSE, 134, "DefMDIChildProcW",              DefMDIChildProcW},
{TRUE,  135, "DefWindowProcA",                DefWindowProcA},
{TRUE,  136, "DefWindowProcW",                DefWindowProcW},
{FALSE, 137, "DeferWindowPos",                DeferWindowPos},
{TRUE,  138, "DeleteMenu",                    DeleteMenu},
{FALSE, 139, "DeregisterShellHookWindow",     DeregisterShellHookWindow},
{FALSE, 140, "DestroyAcceleratorTable",       DestroyAcceleratorTable},
{FALSE, 141, "DestroyCaret",                  DestroyCaret},
{TRUE,  142, "DestroyCursor",                 DestroyCursor},
{TRUE,  143, "DestroyIcon",                   DestroyIcon},
{TRUE,  144, "DestroyMenu",                   DestroyMenu},
{TRUE,  145, "DestroyWindow",                 DestroyWindow},
{FALSE, 146, "DeviceEventWorker",             DeviceEventWorker},
{TRUE,  147, "DialogBoxIndirectParamA",       DialogBoxIndirectParamA},
{FALSE, 148, "DialogBoxIndirectParamAorW",    DialogBoxIndirectParamAorW},
{TRUE,  149, "DialogBoxIndirectParamW",       DialogBoxIndirectParamW},
{TRUE,  150, "DialogBoxParamA",               DialogBoxParamA},
{TRUE,  151, "DialogBoxParamW",               DialogBoxParamW},
{TRUE,  152, "DispatchMessageA",              DispatchMessageA},
{TRUE,  153, "DispatchMessageW",              DispatchMessageW},
{FALSE, 154, "DlgDirListA",                   DlgDirListA},
{FALSE, 155, "DlgDirListComboBoxA",           DlgDirListComboBoxA},
{FALSE, 156, "DlgDirListComboBoxW",           DlgDirListComboBoxW},
{FALSE, 157, "DlgDirListW",                   DlgDirListW},
{FALSE, 158, "DlgDirSelectComboBoxExA",       DlgDirSelectComboBoxExA},
{FALSE, 159, "DlgDirSelectComboBoxExW",       DlgDirSelectComboBoxExW},
{FALSE, 160, "DlgDirSelectExA",               DlgDirSelectExA},
{FALSE, 161, "DlgDirSelectExW",               DlgDirSelectExW},
{FALSE, 162, "DragDetect",                    DragDetect},
{FALSE, 163, "DragObject",                    DragObject},
{FALSE, 164, "DrawAnimatedRects",             DrawAnimatedRects},
{FALSE, 165, "DrawCaption",                   DrawCaption},
{FALSE, 166, "DrawCaptionTempA",              DrawCaptionTempA},
{FALSE, 167, "DrawCaptionTempW",              DrawCaptionTempW},
{FALSE, 168, "DrawEdge",                      DrawEdge},
{FALSE, 169, "DrawFocusRect",                 DrawFocusRect},
{FALSE, 170, "DrawFrame",                     DrawFrame},
{FALSE, 171, "DrawFrameControl",              DrawFrameControl},
{FALSE, 172, "DrawIcon",                      DrawIcon},
{FALSE, 173, "DrawIconEx",                    DrawIconEx},
{TRUE,  174, "DrawMenuBar",                   DrawMenuBar},
{FALSE, 175, "DrawMenuBarTemp",               DrawMenuBarTemp},
{FALSE, 176, "DrawStateA",                    DrawStateA},
{FALSE, 177, "DrawStateW",                    DrawStateW},
{FALSE, 178, "DrawTextA",                     DrawTextA},
{FALSE, 179, "DrawTextExA",                   DrawTextExA},
{FALSE, 180, "DrawTextExW",                   DrawTextExW},
{FALSE, 181, "DrawTextW",                     DrawTextW},
{FALSE, 182, "EditWndProc",                   EditWndProc},
{TRUE,  183, "EmptyClipboard",                EmptyClipboard},
{TRUE,  184, "EnableMenuItem",                EnableMenuItem},
{FALSE, 185, "EnableScrollBar",               EnableScrollBar},
{TRUE,  186, "EnableWindow",                  EnableWindow},
{FALSE, 187, "EndDeferWindowPos",             EndDeferWindowPos},
{TRUE,  188, "EndDialog",                     EndDialog},
{TRUE,  189, "EndMenu",                       EndMenu},
{TRUE,  190, "EndPaint",                      EndPaint},
{FALSE, 191, "EndTask",                       EndTask},
{FALSE, 192, "EnumChildWindows",              EnumChildWindows},
{FALSE, 193, "EnumClipboardFormats",          EnumClipboardFormats},
{FALSE, 194, "EnumDesktopWindows",            EnumDesktopWindows},
{FALSE, 195, "EnumDesktopsA",                 EnumDesktopsA},
{FALSE, 196, "EnumDesktopsW",                 EnumDesktopsW},
{FALSE, 197, "EnumDisplayDevicesA",           EnumDisplayDevicesA},
{FALSE, 198, "EnumDisplayDevicesW",           EnumDisplayDevicesW},
{FALSE, 199, "EnumDisplayMonitors",           EnumDisplayMonitors},
{FALSE, 200, "EnumDisplaySettingsA",          EnumDisplaySettingsA},
{FALSE, 201, "EnumDisplaySettingsExA",        EnumDisplaySettingsExA},
{FALSE, 202, "EnumDisplaySettingsExW",        EnumDisplaySettingsExW},
{FALSE, 203, "EnumDisplaySettingsW",          EnumDisplaySettingsW},
{FALSE, 204, "EnumPropsA",                    EnumPropsA},
{FALSE, 205, "EnumPropsExA",                  EnumPropsExA},
{FALSE, 206, "EnumPropsExW",                  EnumPropsExW},
{FALSE, 207, "EnumPropsW",                    EnumPropsW},
{TRUE,  208, "EnumThreadWindows",             EnumThreadWindows},
{FALSE, 209, "EnumWindowStationsA",           EnumWindowStationsA},
{FALSE, 210, "EnumWindowStationsW",           EnumWindowStationsW},
{TRUE,  211, "EnumWindows",                   EnumWindows},
{TRUE,  212, "EqualRect",                     EqualRect},
{FALSE, 213, "ExcludeUpdateRgn",              ExcludeUpdateRgn},
{FALSE, 214, "ExitWindowsEx",                 ExitWindowsEx},
{FALSE, 215, "FillRect",                      FillRect},
{FALSE, 216, "FindWindowA",                   FindWindowA},
{FALSE, 217, "FindWindowExA",                 FindWindowExA},
{FALSE, 218, "FindWindowExW",                 FindWindowExW},
{FALSE, 219, "FindWindowW",                   FindWindowW},
{FALSE, 220, "FlashWindow",                   FlashWindow},
{FALSE, 221, "FlashWindowEx",                 FlashWindowEx},
{FALSE, 222, "FrameRect",                     FrameRect},
{FALSE, 223, "FreeDDElParam",                 FreeDDElParam},
{FALSE, 224, "GetActiveWindow",               GetActiveWindow},
{FALSE, 225, "GetAltTabInfo",                 GetAltTabInfoA},
{FALSE, 226, "GetAltTabInfoA",                GetAltTabInfoA},
{FALSE, 227, "GetAltTabInfoW",                GetAltTabInfoW},
{FALSE, 228, "GetAncestor",                   GetAncestor},
{FALSE, 230, "GetAppCompatFlags",             GetAppCompatFlags},
{FALSE, 229, "GetAppCompatFlags2",            GetAppCompatFlags2},
{FALSE, 231, "GetAsyncKeyState",              GetAsyncKeyState},
{FALSE, 232, "GetCapture",                    GetCapture},
{FALSE, 233, "GetCaretBlinkTime",             GetCaretBlinkTime},
{FALSE, 234, "GetCaretPos",                   GetCaretPos},
{FALSE, 235, "GetClassInfoA",                 GetClassInfoA},
{FALSE, 236, "GetClassInfoExA",               GetClassInfoExA},
{FALSE, 237, "GetClassInfoExW",               GetClassInfoExW},
{FALSE, 238, "GetClassInfoW",                 GetClassInfoW},
{FALSE, 239, "GetClassLongA",                 GetClassLongA},
{FALSE, 240, "GetClassLongW",                 GetClassLongW},
{TRUE,  241, "GetClassNameA",                 GetClassNameA},
{TRUE,  242, "GetClassNameW",                 GetClassNameW},
{FALSE, 243, "GetClassWord",                  GetClassWord},
{TRUE,  244, "GetClientRect",                 GetClientRect},
{FALSE, 245, "GetClipCursor",                 GetClipCursor},
{FALSE, 246, "GetClipboardData",              GetClipboardData},
{FALSE, 247, "GetClipboardFormatNameA",       GetClipboardFormatNameA},
{FALSE, 248, "GetClipboardFormatNameW",       GetClipboardFormatNameW},
{FALSE, 249, "GetClipboardOwner",             GetClipboardOwner},
{FALSE, 250, "GetClipboardSequenceNumber",    GetClipboardSequenceNumber},
{FALSE, 251, "GetClipboardViewer",            GetClipboardViewer},
{FALSE, 252, "GetComboBoxInfo",               GetComboBoxInfo},
{TRUE,  253, "GetCursor",                     GetCursor},
{FALSE, 254, "GetCursorFrameInfo",            GetCursorFrameInfo},
{FALSE, 255, "GetCursorInfo",                 GetCursorInfo},
{TRUE,  256, "GetCursorPos",                  GetCursorPos},
{TRUE,  257, "GetDC",                         GetDC},
{FALSE, 258, "GetDCEx",                       GetDCEx},
{TRUE,  259, "GetDesktopWindow",              GetDesktopWindow},
{FALSE, 260, "GetDialogBaseUnits",            GetDialogBaseUnits},
{TRUE,  261, "GetDlgCtrlID",                  GetDlgCtrlID},
{TRUE,  262, "GetDlgItem",                    GetDlgItem},
{TRUE,  263, "GetDlgItemInt",                 GetDlgItemInt},
{TRUE,  264, "GetDlgItemTextA",               GetDlgItemTextA},
{TRUE,  265, "GetDlgItemTextW",               GetDlgItemTextW},
{FALSE, 266, "GetDoubleClickTime",            GetDoubleClickTime},
{TRUE,  267, "GetFocus",                      GetFocus},
{FALSE, 268, "GetForegroundWindow",           GetForegroundWindow},
{FALSE, 269, "GetGUIThreadInfo",              GetGUIThreadInfo},
{FALSE, 270, "GetGuiResources",               GetGuiResources},
{FALSE, 271, "GetIconInfo",                   GetIconInfo},
{FALSE, 272, "GetInputDesktop",               GetInputDesktop},
{FALSE, 273, "GetInputState",                 GetInputState},
{FALSE, 274, "GetInternalWindowPos",          GetInternalWindowPos},
{FALSE, 275, "GetKBCodePage",                 GetKBCodePage},
{FALSE, 276, "GetKeyNameTextA",               GetKeyNameTextA},
{FALSE, 277, "GetKeyNameTextW",               GetKeyNameTextW},
{FALSE, 278, "GetKeyState",                   GetKeyState},
{FALSE, 279, "GetKeyboardLayout",             GetKeyboardLayout},
{FALSE, 280, "GetKeyboardLayoutList",         GetKeyboardLayoutList},
{FALSE, 281, "GetKeyboardLayoutNameA",        GetKeyboardLayoutNameA},
{FALSE, 282, "GetKeyboardLayoutNameW",        GetKeyboardLayoutNameW},
{FALSE, 283, "GetKeyboardState",              GetKeyboardState},
{TRUE,  284, "GetKeyboardType",               GetKeyboardType},
{FALSE, 285, "GetLastActivePopup",            GetLastActivePopup},
{FALSE, 286, "GetLastInputInfo",              GetLastInputInfo},
{FALSE, 287, "GetListBoxInfo",                GetListBoxInfo},
{TRUE,  288, "GetMenu",                       GetMenu},
{FALSE, 289, "GetMenuBarInfo",                GetMenuBarInfo},
{TRUE,  290, "GetMenuCheckMarkDimensions",    GetMenuCheckMarkDimensions},
{FALSE, 291, "GetMenuContextHelpId",          GetMenuContextHelpId},
{TRUE,  292, "GetMenuDefaultItem",            GetMenuDefaultItem},
{FALSE, 293, "GetMenuInfo",                   GetMenuInfo},
{TRUE,  294, "GetMenuItemCount",              GetMenuItemCount},
{TRUE,  295, "GetMenuItemID",                 GetMenuItemID},
{TRUE,  296, "GetMenuItemInfoA",              GetMenuItemInfoA},
{TRUE,  297, "GetMenuItemInfoW",              GetMenuItemInfoW},
{TRUE,  298, "GetMenuItemRect",               GetMenuItemRect},
{TRUE,  299, "GetMenuState",                  GetMenuState},
{TRUE,  300, "GetMenuStringA",                GetMenuStringA},
{TRUE,  301, "GetMenuStringW",                GetMenuStringW},
{TRUE,  302, "GetMessageA",                   GetMessageA},
{FALSE, 303, "GetMessageExtraInfo",           GetMessageExtraInfo},
{FALSE, 304, "GetMessagePos",                 GetMessagePos},
{FALSE, 305, "GetMessageTime",                GetMessageTime},
{TRUE,  306, "GetMessageW",                   GetMessageW},
{FALSE, 307, "GetMonitorInfoA",               GetMonitorInfoA},
{FALSE, 308, "GetMonitorInfoW",               GetMonitorInfoW},
{FALSE, 309, "GetMouseMovePointsEx",          GetMouseMovePointsEx},
{FALSE, 310, "GetNextDlgGroupItem",           GetNextDlgGroupItem},
{FALSE, 311, "GetNextDlgTabItem",             GetNextDlgTabItem},
{FALSE, 312, "GetOpenClipboardWindow",        GetOpenClipboardWindow},
{TRUE,  313, "GetParent",                     GetParent},
{FALSE, 314, "GetPriorityClipboardFormat",    GetPriorityClipboardFormat},
{FALSE, 315, "GetProcessDefaultLayout",       GetProcessDefaultLayout},
{FALSE, 316, "GetProcessWindowStation",       GetProcessWindowStation},
{FALSE, 317, "GetProgmanWindow",              GetProgmanWindow},
{FALSE, 318, "GetPropA",                      GetPropA},
{FALSE, 319, "GetPropW",                      GetPropW},
{FALSE, 320, "GetQueueStatus",                GetQueueStatus},
{FALSE, 321, "GetScrollBarInfo",              GetScrollBarInfo},
{FALSE, 322, "GetScrollInfo",                 GetScrollInfo},
{TRUE,  323, "GetScrollPos",                  GetScrollPos},
{TRUE,  324, "GetScrollRange",                GetScrollRange},
{FALSE, 325, "GetShellWindow",                GetShellWindow},
{TRUE,  326, "GetSubMenu",                    GetSubMenu},
{TRUE,  327, "GetSysColor",                   GetSysColor},
{FALSE, 328, "GetSysColorBrush",              GetSysColorBrush},
{TRUE,  329, "GetSystemMenu",                 GetSystemMenu},
{TRUE,  330, "GetSystemMetrics",              GetSystemMetrics},
{FALSE, 331, "GetTabbedTextExtentA",          GetTabbedTextExtentA},
{FALSE, 332, "GetTabbedTextExtentW",          GetTabbedTextExtentW},
{FALSE, 333, "GetTaskmanWindow",              GetTaskmanWindow},
{FALSE, 334, "GetThreadDesktop",              GetThreadDesktop},
{FALSE, 335, "GetTitleBarInfo",               GetTitleBarInfo},
{FALSE, 336, "GetTopWindow",                  GetTopWindow},
{FALSE, 337, "GetUpdateRect",                 GetUpdateRect},
{FALSE, 338, "GetUpdateRgn",                  GetUpdateRgn},
{FALSE, 339, "GetUserObjectInformationA",     GetUserObjectInformationA},
{FALSE, 340, "GetUserObjectInformationW",     GetUserObjectInformationW},
{FALSE, 341, "GetUserObjectSecurity",         GetUserObjectSecurity},
{FALSE, 342, "GetWinStationInfo",             GetWinStationInfo},
{TRUE,  343, "GetWindow",                     GetWindow},
{FALSE, 344, "GetWindowContextHelpId",        GetWindowContextHelpId},
{FALSE, 345, "GetWindowDC",                   GetWindowDC},
{FALSE, 346, "GetWindowInfo",                 GetWindowInfo},
{TRUE,  347, "GetWindowLongA",                GetWindowLongA},
{TRUE,  348, "GetWindowLongW",                GetWindowLongW},
{FALSE, 349, "GetWindowModuleFileName",       GetWindowModuleFileNameA},
{FALSE, 350, "GetWindowModuleFileNameA",      GetWindowModuleFileNameA},
{FALSE, 351, "GetWindowModuleFileNameW",      GetWindowModuleFileNameW},
{FALSE, 352, "GetWindowPlacement",            GetWindowPlacement},
{TRUE,  353, "GetWindowRect",                 GetWindowRect},
{FALSE, 354, "GetWindowRgn",                  GetWindowRgn},
{TRUE,  355, "GetWindowTextA",                GetWindowTextA},
{TRUE,  356, "GetWindowTextLengthA",          GetWindowTextLengthA},
{TRUE,  357, "GetWindowTextLengthW",          GetWindowTextLengthW},
{TRUE,  358, "GetWindowTextW",                GetWindowTextW},
{FALSE, 359, "GetWindowThreadProcessId",      GetWindowThreadProcessId},
{FALSE, 360, "GetWindowWord",                 GetWindowWord},
{FALSE, 361, "GrayStringA",                   GrayStringA},
{FALSE, 362, "GrayStringW",                   GrayStringW},
{FALSE, 363, "HideCaret",                     HideCaret},
{TRUE,  364, "HiliteMenuItem",                HiliteMenuItem},
{FALSE, 365, "IMPGetIMEA",                    IMPGetIMEA},
{FALSE, 366, "IMPGetIMEW",                    IMPGetIMEW},
{FALSE, 367, "IMPQueryIMEA",                  IMPQueryIMEA},
{FALSE, 368, "IMPQueryIMEW",                  IMPQueryIMEW},
{FALSE, 369, "IMPSetIMEA",                    IMPSetIMEA},
{FALSE, 370, "IMPSetIMEW",                    IMPSetIMEW},
{FALSE, 371, "ImpersonateDdeClientWindow",    ImpersonateDdeClientWindow},
{FALSE, 372, "InSendMessage",                 InSendMessage},
{FALSE, 373, "InSendMessageEx",               InSendMessageEx},
{TRUE,  374, "InflateRect",                   InflateRect},
{FALSE, 375, "InitializeLpkHooks",            InitializeLpkHooks},
{FALSE, 376, "InitializeWin32EntryTable",     InitializeWin32EntryTable},
{TRUE,  377, "InsertMenuA",                   InsertMenuA},
{TRUE,  378, "InsertMenuItemA",               InsertMenuItemA},
{TRUE,  379, "InsertMenuItemW",               InsertMenuItemW},
{TRUE,  380, "InsertMenuW",                   InsertMenuW},
{FALSE, 381, "InternalGetWindowText",         InternalGetWindowText},
{TRUE,  382, "IntersectRect",                 IntersectRect},
{TRUE,  383, "InvalidateRect",                InvalidateRect},
{FALSE, 384, "InvalidateRgn",                 InvalidateRgn},
{FALSE, 385, "InvertRect",                    InvertRect},
{FALSE, 386, "IsCharAlphaA",                  IsCharAlphaA},
{FALSE, 387, "IsCharAlphaNumericA",           IsCharAlphaNumericA},
{FALSE, 388, "IsCharAlphaNumericW",           IsCharAlphaNumericW},
{FALSE, 389, "IsCharAlphaW",                  IsCharAlphaW},
{FALSE, 390, "IsCharLowerA",                  IsCharLowerA},
{FALSE, 391, "IsCharLowerW",                  IsCharLowerW},
{FALSE, 392, "IsCharUpperA",                  IsCharUpperA},
{FALSE, 393, "IsCharUpperW",                  IsCharUpperW},
{TRUE,  394, "IsChild",                       IsChild},
{FALSE, 395, "IsClipboardFormatAvailable",    IsClipboardFormatAvailable},
{FALSE, 396, "IsDialogMessage",               IsDialogMessageA},
{FALSE, 397, "IsDialogMessageA",              IsDialogMessageA},
{FALSE, 398, "IsDialogMessageW",              IsDialogMessageW},
{TRUE,  399, "IsDlgButtonChecked",            IsDlgButtonChecked},
{FALSE, 400, "IsHungAppWindow",               IsHungAppWindow},
{FALSE, 401, "IsIconic",                      IsIconic},
{TRUE,  402, "IsMenu",                        IsMenu},
{TRUE,  403, "IsRectEmpty",                   IsRectEmpty},
{FALSE, 404, "IsWindow",                      IsWindow},
{TRUE,  405, "IsWindowEnabled",               IsWindowEnabled},
{TRUE,  406, "IsWindowUnicode",               IsWindowUnicode},
{FALSE, 407, "IsWindowVisible",               IsWindowVisible},
{FALSE, 408, "IsZoomed",                      IsZoomed},
{FALSE, 409, "KillSystemTimer",               KillSystemTimer},
{TRUE,  410, "KillTimer",                     KillTimer},
{TRUE,  411, "LoadAcceleratorsA",             LoadAcceleratorsA},
{TRUE,  412, "LoadAcceleratorsW",             LoadAcceleratorsW},
{TRUE,  413, "LoadBitmapA",                   LoadBitmapA},
{TRUE,  414, "LoadBitmapW",                   LoadBitmapW},
{TRUE,  415, "LoadCursorA",                   LoadCursorA},
{FALSE, 416, "LoadCursorFromFileA",           LoadCursorFromFileA},
{FALSE, 417, "LoadCursorFromFileW",           LoadCursorFromFileW},
{TRUE,  418, "LoadCursorW",                   LoadCursorW},
{TRUE,  419, "LoadIconA",                     LoadIconA},
{TRUE,  420, "LoadIconW",                     LoadIconW},
{TRUE,  421, "LoadImageA",                    LoadImageA},
{TRUE,  422, "LoadImageW",                    LoadImageW},
{FALSE, 423, "LoadKeyboardLayoutA",           LoadKeyboardLayoutA},
{FALSE, 424, "LoadKeyboardLayoutEx",          LoadKeyboardLayoutEx},
{FALSE, 425, "LoadKeyboardLayoutW",           LoadKeyboardLayoutW},
{FALSE, 426, "LoadLocalFonts",                LoadLocalFonts},
{TRUE,  427, "LoadMenuA",                     LoadMenuA},
{TRUE,  428, "LoadMenuIndirectA",             LoadMenuIndirectA},
{TRUE,  429, "LoadMenuIndirectW",             LoadMenuIndirectW},
{TRUE,  430, "LoadMenuW",                     LoadMenuW},
{FALSE, 431, "LoadRemoteFonts",               LoadRemoteFonts},
{TRUE,  432, "LoadStringA",                   LoadStringA},
{TRUE,  433, "LoadStringW",                   LoadStringW},
{FALSE, 434, "LockSetForegroundWindow",       LockSetForegroundWindow},
{FALSE, 435, "LockWindowStation",             LockWindowStation},
{FALSE, 436, "LockWindowUpdate",              LockWindowUpdate},
{FALSE, 437, "LockWorkStation",               LockWorkStation},
{FALSE, 438, "LookupIconIdFromDirectory",     LookupIconIdFromDirectory},
{FALSE, 439, "LookupIconIdFromDirectoryEx",   LookupIconIdFromDirectoryEx},
{FALSE, 440, "MBToWCSEx",                     MBToWCSEx},
{FALSE, 441, "MB_GetString",                  MB_GetString},
{FALSE, 442, "MapDialogRect",                 MapDialogRect},
{FALSE, 443, "MapVirtualKeyA",                MapVirtualKeyA},
{FALSE, 444, "MapVirtualKeyExA",              MapVirtualKeyExA},
{FALSE, 445, "MapVirtualKeyExW",              MapVirtualKeyExW},
{FALSE, 446, "MapVirtualKeyW",                MapVirtualKeyW},
{TRUE,  447, "MapWindowPoints",               MapWindowPoints},
{TRUE,  448, "MenuItemFromPoint",             MenuItemFromPoint},
{FALSE, 449, "MenuWindowProcA",               MenuWindowProcA},
{FALSE, 450, "MenuWindowProcW",               MenuWindowProcW},
{TRUE,  451, "MessageBeep",                   MessageBeep},
{TRUE,  452, "MessageBoxA",                   MessageBoxA},
{TRUE,  453, "MessageBoxExA",                 MessageBoxExA},
{TRUE,  454, "MessageBoxExW",                 MessageBoxExW},
{FALSE, 455, "MessageBoxIndirectA",           MessageBoxIndirectA},
{FALSE, 456, "MessageBoxIndirectW",           MessageBoxIndirectW},
{TRUE,  457, "MessageBoxW",                   MessageBoxW},
{TRUE,  458, "ModifyMenuA",                   ModifyMenuA},
{TRUE,  459, "ModifyMenuW",                   ModifyMenuW},
{FALSE, 460, "MonitorFromPoint",              MonitorFromPoint},
{FALSE, 461, "MonitorFromRect",               MonitorFromRect},
{FALSE, 462, "MonitorFromWindow",             MonitorFromWindow},
{TRUE,  463, "MoveWindow",                    MoveWindow},
{FALSE, 464, "MsgWaitForMultipleObjects",     MsgWaitForMultipleObjects},
{FALSE, 465, "MsgWaitForMultipleObjectsEx",   MsgWaitForMultipleObjectsEx},
{FALSE, 466, "NotifyWinEvent",                NotifyWinEvent},
{FALSE, 467, "OemKeyScan",                    OemKeyScan},
{FALSE, 468, "OemToCharA",                    OemToCharA},
{FALSE, 469, "OemToCharBuffA",                OemToCharBuffA},
{FALSE, 470, "OemToCharBuffW",                OemToCharBuffW},
{FALSE, 471, "OemToCharW",                    OemToCharW},
{TRUE,  472, "OffsetRect",                    OffsetRect},
{TRUE,  473, "OpenClipboard",                 OpenClipboard},
{FALSE, 474, "OpenDesktopA",                  OpenDesktopA},
{FALSE, 475, "OpenDesktopW",                  OpenDesktopW},
{FALSE, 476, "OpenIcon",                      OpenIcon},
{FALSE, 477, "OpenInputDesktop",              OpenInputDesktop},
{FALSE, 478, "OpenWindowStationA",            OpenWindowStationA},
{FALSE, 479, "OpenWindowStationW",            OpenWindowStationW},
{FALSE, 480, "PackDDElParam",                 PackDDElParam},
{FALSE, 481, "PaintDesktop",                  PaintDesktop},
{TRUE,  482, "PeekMessageA",                  PeekMessageA},
{TRUE,  483, "PeekMessageW",                  PeekMessageW},
{TRUE,  484, "PostMessageA",                  PostMessageA},
{TRUE,  485, "PostMessageW",                  PostMessageW},
{TRUE,  486, "PostQuitMessage",               PostQuitMessage},
{FALSE, 487, "PostThreadMessageA",            PostThreadMessageA},
{FALSE, 488, "PostThreadMessageW",            PostThreadMessageW},
{FALSE, 489, "PrivateExtractIconExA",         PrivateExtractIconExA},
{FALSE, 490, "PrivateExtractIconExW",         PrivateExtractIconExW},
{FALSE, 491, "PrivateExtractIconsA",          PrivateExtractIconsA},
{FALSE, 492, "PrivateExtractIconsW",          PrivateExtractIconsW},
{FALSE, 493, "PrivateSetDbgTag",              PrivateSetDbgTag},
{FALSE, 494, "PrivateSetRipFlags",            PrivateSetRipFlags},
{TRUE,  495, "PtInRect",                      PtInRect},
{FALSE, 496, "QuerySendMessage",              QuerySendMessage},
{FALSE, 497, "QueryUserCounters",             QueryUserCounters},
{FALSE, 498, "RealChildWindowFromPoint",      RealChildWindowFromPoint},
{FALSE, 499, "RealGetWindowClass",            RealGetWindowClassA},
{FALSE, 500, "RealGetWindowClassA",           RealGetWindowClassA},
{FALSE, 501, "RealGetWindowClassW",           RealGetWindowClassW},
{TRUE,  502, "RedrawWindow",                  RedrawWindow},
{TRUE,  503, "RegisterClassA",                RegisterClassA},
{TRUE,  504, "RegisterClassExA",              RegisterClassExA},
{TRUE,  505, "RegisterClassExW",              RegisterClassExW},
{TRUE,  506, "RegisterClassW",                RegisterClassW},
{FALSE, 507, "RegisterClipboardFormatA",      RegisterClipboardFormatA},
{FALSE, 508, "RegisterClipboardFormatW",      RegisterClipboardFormatW},
{FALSE, 509, "RegisterDeviceNotificationA",   RegisterDeviceNotificationA},
{FALSE, 510, "RegisterDeviceNotificationW",   RegisterDeviceNotificationW},
{FALSE, 511, "RegisterHotKey",                RegisterHotKey},
{FALSE, 512, "RegisterLogonProcess",          RegisterLogonProcess},
{FALSE, 513, "RegisterServicesProcess",       RegisterServicesProcess},
{FALSE, 514, "RegisterShellHookWindow",       RegisterShellHookWindow},
{FALSE, 515, "RegisterSystemThread",          RegisterSystemThread},
{FALSE, 516, "RegisterTasklist",              RegisterTasklist},
{FALSE, 517, "RegisterWindowMessageA",        RegisterWindowMessageA},
{FALSE, 518, "RegisterWindowMessageW",        RegisterWindowMessageW},
{TRUE,  519, "ReleaseCapture",                ReleaseCapture},
{TRUE,  520, "ReleaseDC",                     ReleaseDC},
{FALSE, 521, "RemoveMenu",                    RemoveMenu},
{FALSE, 522, "RemovePropA",                   RemovePropA},
{FALSE, 523, "RemovePropW",                   RemovePropW},
{FALSE, 524, "ReplyMessage",                  ReplyMessage},
{FALSE, 525, "ResolveDesktopForWOW",          ResolveDesktopForWOW},
{FALSE, 526, "ReuseDDElParam",                ReuseDDElParam},
{TRUE,  527, "ScreenToClient",                ScreenToClient},
{FALSE, 528, "ScrollChildren",                ScrollChildren},
{FALSE, 529, "ScrollDC",                      ScrollDC},
{FALSE, 530, "ScrollWindow",                  ScrollWindow},
{FALSE, 531, "ScrollWindowEx",                ScrollWindowEx},
{TRUE,  532, "SendDlgItemMessageA",           SendDlgItemMessageA},
{TRUE,  533, "SendDlgItemMessageW",           SendDlgItemMessageW},
{FALSE, 534, "SendIMEMessageExA",             SendIMEMessageExA},
{FALSE, 535, "SendIMEMessageExW",             SendIMEMessageExW},
{FALSE, 536, "SendInput",                     SendInput},
{TRUE,  537, "SendMessageA",                  SendMessageA},
{FALSE, 538, "SendMessageCallbackA",          SendMessageCallbackA},
{FALSE, 539, "SendMessageCallbackW",          SendMessageCallbackW},
{FALSE, 540, "SendMessageTimeoutA",           SendMessageTimeoutA},
{FALSE, 541, "SendMessageTimeoutW",           SendMessageTimeoutW},
{TRUE,  542, "SendMessageW",                  SendMessageW},
{FALSE, 543, "SendNotifyMessageA",            SendNotifyMessageA},
{FALSE, 544, "SendNotifyMessageW",            SendNotifyMessageW},
{FALSE, 545, "SetActiveWindow",               SetActiveWindow},
{TRUE,  546, "SetCapture",                    SetCapture},
{FALSE, 547, "SetCaretBlinkTime",             SetCaretBlinkTime},
{FALSE, 548, "SetCaretPos",                   SetCaretPos},
{FALSE, 549, "SetClassLongA",                 SetClassLongA},
{FALSE, 550, "SetClassLongW",                 SetClassLongW},
{FALSE, 551, "SetClassWord",                  SetClassWord},
{TRUE,  552, "SetClipboardData",              SetClipboardData},
{FALSE, 553, "SetClipboardViewer",            SetClipboardViewer},
{FALSE, 554, "SetConsoleReserveKeys",         SetConsoleReserveKeys},
{TRUE,  555, "SetCursor",                     SetCursor},
{FALSE, 556, "SetCursorContents",             SetCursorContents},
{TRUE,  557, "SetCursorPos",                  SetCursorPos},
{FALSE, 558, "SetDebugErrorLevel",            SetDebugErrorLevel},
{FALSE, 559, "SetDeskWallpaper",              SetDeskWallpaper},
{TRUE,  560, "SetDlgItemInt",                 SetDlgItemInt},
{TRUE,  561, "SetDlgItemTextA",               SetDlgItemTextA},
{TRUE,  562, "SetDlgItemTextW",               SetDlgItemTextW},
{FALSE, 563, "SetDoubleClickTime",            SetDoubleClickTime},
{TRUE,  564, "SetFocus",                      SetFocus},
{TRUE,  565, "SetForegroundWindow",           SetForegroundWindow},
{FALSE, 566, "SetInternalWindowPos",          SetInternalWindowPos},
{FALSE, 567, "SetKeyboardState",              SetKeyboardState},
{FALSE, 568, "SetLastErrorEx",                SetLastErrorEx},
{FALSE, 569, "SetLayeredWindowAttributes",    SetLayeredWindowAttributes},
{FALSE, 570, "SetLogonNotifyWindow",          SetLogonNotifyWindow},
{TRUE,  571, "SetMenu",                       SetMenu},
{FALSE, 572, "SetMenuContextHelpId",          SetMenuContextHelpId},
{TRUE,  573, "SetMenuDefaultItem",            SetMenuDefaultItem},
{FALSE, 574, "SetMenuInfo",                   SetMenuInfo},
{TRUE,  575, "SetMenuItemBitmaps",            SetMenuItemBitmaps},
{TRUE,  576, "SetMenuItemInfoA",              SetMenuItemInfoA},
{TRUE,  577, "SetMenuItemInfoW",              SetMenuItemInfoW},
{FALSE, 578, "SetMessageExtraInfo",           SetMessageExtraInfo},
{FALSE, 579, "SetMessageQueue",               SetMessageQueue},
{FALSE, 580, "SetParent",                     SetParent},
{FALSE, 581, "SetProcessDefaultLayout",       SetProcessDefaultLayout},
{FALSE, 582, "SetProcessWindowStation",       SetProcessWindowStation},
{FALSE, 583, "SetProgmanWindow",              SetProgmanWindow},
{FALSE, 584, "SetPropA",                      SetPropA},
{FALSE, 585, "SetPropW",                      SetPropW},
{TRUE,  586, "SetRect",                       SetRect},
{TRUE,  587, "SetRectEmpty",                  SetRectEmpty},
{FALSE, 588, "SetScrollInfo",                 SetScrollInfo},
{TRUE,  589, "SetScrollPos",                  SetScrollPos},
{TRUE,  590, "SetScrollRange",                SetScrollRange},
{FALSE, 591, "SetShellWindow",                SetShellWindow},
{FALSE, 592, "SetShellWindowEx",              SetShellWindowEx},
{FALSE, 593, "SetSysColors",                  SetSysColors},
{FALSE, 594, "SetSysColorsTemp",              SetSysColorsTemp},
{FALSE, 595, "SetSystemCursor",               SetSystemCursor},
{FALSE, 596, "SetSystemMenu",                 SetSystemMenu},
{FALSE, 597, "SetSystemTimer",                SetSystemTimer},
{FALSE, 598, "SetTaskmanWindow",              SetTaskmanWindow},
{FALSE, 599, "SetThreadDesktop",              SetThreadDesktop},
{TRUE,  600, "SetTimer",                      SetTimer},
{FALSE, 601, "SetUserObjectInformationA",     SetUserObjectInformationA},
{FALSE, 602, "SetUserObjectInformationW",     SetUserObjectInformationW},
{FALSE, 603, "SetUserObjectSecurity",         SetUserObjectSecurity},
{FALSE, 604, "SetWinEventHook",               SetWinEventHook},
{FALSE, 605, "SetWindowContextHelpId",        SetWindowContextHelpId},
{TRUE,  606, "SetWindowLongA",                SetWindowLongA},
{TRUE,  607, "SetWindowLongW",                SetWindowLongW},
{FALSE, 608, "SetWindowPlacement",            SetWindowPlacement},
{TRUE,  609, "SetWindowPos",                  SetWindowPos},
{FALSE, 610, "SetWindowRgn",                  SetWindowRgn},
{FALSE, 611, "SetWindowStationUser",          SetWindowStationUser},
{TRUE,  612, "SetWindowTextA",                SetWindowTextA},
{TRUE,  613, "SetWindowTextW",                SetWindowTextW},
{FALSE, 614, "SetWindowWord",                 SetWindowWord},
{FALSE, 615, "SetWindowsHookA",               SetWindowsHookA},
{FALSE, 616, "SetWindowsHookExA",             SetWindowsHookExA},
{FALSE, 617, "SetWindowsHookExW",             SetWindowsHookExW},
{FALSE, 618, "SetWindowsHookW",               SetWindowsHookW},
{FALSE, 619, "ShowCaret",                     ShowCaret},
{FALSE, 620, "ShowCursor",                    ShowCursor},
{FALSE, 621, "ShowOwnedPopups",               ShowOwnedPopups},
{FALSE, 622, "ShowScrollBar",                 ShowScrollBar},
{FALSE, 623, "ShowStartGlass",                ShowStartGlass},
{TRUE,  624, "ShowWindow",                    ShowWindow},
{FALSE, 625, "ShowWindowAsync",               ShowWindowAsync},
{FALSE, 626, "SoftModalMessageBox",           SoftModalMessageBox},
{TRUE,  627, "SubtractRect",                  SubtractRect},
{FALSE, 628, "SwapMouseButton",               SwapMouseButton},
{FALSE, 629, "SwitchDesktop",                 SwitchDesktop},
{FALSE, 630, "SwitchToThisWindow",            SwitchToThisWindow},
{TRUE,  631, "SystemParametersInfoA",         SystemParametersInfoA},
{TRUE,  632, "SystemParametersInfoW",         SystemParametersInfoW},
{FALSE, 633, "TabbedTextOutA",                TabbedTextOutA},
{FALSE, 634, "TabbedTextOutW",                TabbedTextOutW},
{FALSE, 635, "TileChildWindows",              TileChildWindows},
{FALSE, 636, "TileWindows",                   TileWindows},
{FALSE, 637, "ToAscii",                       ToAscii},
{FALSE, 638, "ToAsciiEx",                     ToAsciiEx},
{FALSE, 639, "ToUnicode",                     ToUnicode},
{FALSE, 640, "ToUnicodeEx",                   ToUnicodeEx},
{FALSE, 641, "TrackMouseEvent",               TrackMouseEvent},
{TRUE,  642, "TrackPopupMenu",                TrackPopupMenu},
{TRUE,  643, "TrackPopupMenuEx",              TrackPopupMenuEx},
{TRUE,  644, "TranslateAccelerator",          TranslateAcceleratorA},
{TRUE,  645, "TranslateAcceleratorA",         TranslateAcceleratorA},
{TRUE,  646, "TranslateAcceleratorW",         TranslateAcceleratorW},
{FALSE, 647, "TranslateMDISysAccel",          TranslateMDISysAccel},
{TRUE,  648, "TranslateMessage",              TranslateMessage},
{FALSE, 649, "TranslateMessageEx",            TranslateMessageEx},
{FALSE, 650, "UnhookWinEvent",                UnhookWinEvent},
{FALSE, 651, "UnhookWindowsHook",             UnhookWindowsHook},
{FALSE, 652, "UnhookWindowsHookEx",           UnhookWindowsHookEx},
{TRUE,  653, "UnionRect",                     UnionRect},
{FALSE, 654, "UnloadKeyboardLayout",          UnloadKeyboardLayout},
{FALSE, 655, "UnlockWindowStation",           UnlockWindowStation},
{FALSE, 656, "UnpackDDElParam",               UnpackDDElParam},
{TRUE,  657, "UnregisterClassA",              UnregisterClassA},
{TRUE,  658, "UnregisterClassW",              UnregisterClassW},
{FALSE, 659, "UnregisterDeviceNotification",  UnregisterDeviceNotification},
{FALSE, 660, "UnregisterHotKey",              UnregisterHotKey},
{FALSE, 661, "UpdateLayeredWindow",           UpdateLayeredWindow},
{FALSE, 662, "UpdatePerUserSystemParameters", UpdatePerUserSystemParameters},
{TRUE,  663, "UpdateWindow",                  UpdateWindow},
{FALSE, 664, "User32InitializeImmEntryTable", User32InitializeImmEntryTable},
{FALSE, 665, "UserClientDllInitialize",       UserClientDllInitialize},
{FALSE, 666, "UserHandleGrantAccess",         UserHandleGrantAccess},
{FALSE, 667, "UserLpkPSMTextOut",             UserLpkPSMTextOut},
{FALSE, 668, "UserLpkTabbedTextOut",          UserLpkTabbedTextOut},
{FALSE, 669, "UserRealizePalette",            UserRealizePalette},
{FALSE, 670, "UserRegisterWowHandlers",       UserRegisterWowHandlers},
{FALSE, 671, "VRipOutput",                    VRipOutput},
{FALSE, 672, "VTagOutput",                    VTagOutput},
{FALSE, 673, "ValidateRect",                  ValidateRect},
{FALSE, 674, "ValidateRgn",                   ValidateRgn},
{FALSE, 675, "VkKeyScanA",                    VkKeyScanA},
{FALSE, 676, "VkKeyScanExA",                  VkKeyScanExA},
{FALSE, 677, "VkKeyScanExW",                  VkKeyScanExW},
{FALSE, 678, "VkKeyScanW",                    VkKeyScanW},
{FALSE, 679, "WCSToMBEx",                     WCSToMBEx},
{FALSE, 680, "WINNLSEnableIME",               WINNLSEnableIME},
{FALSE, 681, "WINNLSGetEnableStatus",         WINNLSGetEnableStatus},
{FALSE, 682, "WINNLSGetIMEHotkey",            WINNLSGetIMEHotkey},
{FALSE, 683, "WaitForInputIdle",              WaitForInputIdle},
{FALSE, 684, "WaitMessage",                   WaitMessage},
{FALSE, 685, "Win32PoolAllocationStats",      Win32PoolAllocationStats},
{TRUE,  686, "WinHelpA",                      WinHelpA},
{TRUE,  687, "WinHelpW",                      WinHelpW},
{FALSE, 688, "WindowFromDC",                  WindowFromDC},
{TRUE,  689, "WindowFromPoint",               WindowFromPoint},
{FALSE, 690, "keybd_event",                   keybd_event},
{FALSE, 691, "mouse_event",                   mouse_event},
{TRUE,  692, "wsprintfA",                     wsprintfA},
{TRUE,  693, "wsprintfW",                     wsprintfW},
{TRUE,  694, "wvsprintfA",                    wvsprintfA},
{TRUE,  695, "wvsprintfW",                    wvsprintfW},
{FALSE,   0, NULL,                            NULL}};
